/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2012 by Chuck Chen <ycchen@realtek.com>
 *
 * Time initialization.
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/system.h>
#include <asm/interrupt.h>
#include <asm/thread.h>
#include <rbus/timer_reg.h>

DECLARE_GLOBAL_DATA_PTR;
extern int smp_processor_id(unsigned int mpidr);

struct timer_reg
{
	unsigned int tvr; // target value register
	unsigned int cvr; // current value register
	unsigned int cr; // control register
	unsigned int icr_ie_shift; // interrupt enable shift value
};

typedef struct _rt_timer {
	TimerCb pTimerCb;
	u32 u32FirstTimeMs;
	u32 u32PeriodTimeMs;
	u32 started;
	char name[32];
} Rt_Timer;

struct timer_reg rtk_timer[12] =
{
	{TIMER_TC0TVR_reg, TIMER_TC0CVR_reg, TIMER_TC0CR_reg, TIMER_TCICR_tc0ie_shift},
	{TIMER_TC1TVR_reg, TIMER_TC1CVR_reg, TIMER_TC1CR_reg, TIMER_TCICR_tc1ie_shift},
	{TIMER_TC2TVR_reg, TIMER_TC2CVR_reg, TIMER_TC2CR_reg, TIMER_TCICR_tc2ie_shift},
	{TIMER_TC3TVR_reg, TIMER_TC3CVR_reg, TIMER_TC3CR_reg, TIMER_TCICR_tc3ie_shift},
	{TIMER_TC4TVR_reg, TIMER_TC4CVR_reg, TIMER_TC4CR_reg, TIMER_TCICR_tc4ie_shift},
	{TIMER_TC5TVR_reg, TIMER_TC5CVR_reg, TIMER_TC5CR_reg, TIMER_TCICR_tc5ie_shift},
	{TIMER_TC6TVR_reg, TIMER_TC6CVR_reg, TIMER_TC6CR_reg, TIMER_TCICR_tc6ie_shift},
	{TIMER_TC7TVR_reg, TIMER_TC7CVR_reg, TIMER_TC7CR_reg, TIMER_TCICR_tc7ie_shift},
	{TIMER_TC8TVR_reg, TIMER_TC8CVR_reg, TIMER_TC8CR_reg, TIMER_TCICR_tc8ie_shift},
	{TIMER_TC9TVR_reg, TIMER_TC9CVR_reg, TIMER_TC9CR_reg, TIMER_TCICR_tc9ie_shift},
	{TIMER_TC10TVR_reg, TIMER_TC10CVR_reg, TIMER_TC10CR_reg, TIMER_TCICR_tc10ie_shift},
	{TIMER_TC11TVR_reg, TIMER_TC11CVR_reg, TIMER_TC11CR_reg, TIMER_TCICR_tc11ie_shift}
};
u32 g_rt_timer_init = 0;

static struct timer_reg *timer_base = &rtk_timer[SYSTEM_TIMER];

/*
 * Nothing really to do with interrupts, just starts up a counter.
 */

#define TIMER_OVERFLOW_VAL			0xffffffff
#define TIMER_LOAD_VAL				0xffffffff
#define TIMER_IS_NOT_INITIALIZED	0xFFFFFFFF
#define TIMER_HANDLER_IS_FULL		0xFFFFFF
#define TIMER_FAILED				0xFFFF
#define TIMER_SUCCESS				0x0
#define TIMERINFO_TIMER_ENABLE		0xC0000000
#define TIMERINFO_TIMER_DISABLE		0x00000000
#define RT_TIMER_TOTAL_NUM_PRE_CORE	2
#define TICKS_MSEC					27000

#define ARCH_TIMER_CTRL_ENABLE		(1 << 0)
#define ARCH_TIMER_CTRL_IT_MASK		(1 << 1)
#define ARCH_TIMER_CTRL_IT_STAT		(1 << 2)

#define ARCH_TIMER_REG_CTRL		0
#define ARCH_TIMER_REG_TVAL		1

#define ARCH_TIMER_PHYS_ACCESS		0
#define ARCH_TIMER_VIRT_ACCESS		1

#define WRAP_A7_APB_TSG_EN0    0xB805C800
#define WRAP_A7_APB_TSG_EN1    0xB805C804

#define PERIOD_TIMER		1
#define ONE_SHOT_TIMER		2
Rt_Timer g_rt_timer[4][RT_TIMER_TOTAL_NUM_PRE_CORE];


/*
 * These register accessors are marked inline so the 
compiler can
 * nicely work out which register we want, and chuck away 
the rest of
 * the code. At least it does so with a recent GCC (4.6.3).
 */
static inline void arch_timer_reg_write(const int access, const int reg, u32 val)
{
    if (access == ARCH_TIMER_PHYS_ACCESS)
    {
        switch (reg)
        {
            case ARCH_TIMER_REG_CTRL:
                asm volatile("mcr p15, 0, %0, c14, c2, 1" : : "r" (val));
            break;
            case ARCH_TIMER_REG_TVAL:
                asm volatile("mcr p15, 0, %0, c14, c2, 0" : : "r" (val));
            break;
        }
    }

    if (access == ARCH_TIMER_VIRT_ACCESS)
    {
        switch (reg)
        {
            case ARCH_TIMER_REG_CTRL:
                asm volatile("mcr p15, 0, %0, c14, c3, 1" : : "r" (val));
            break;
            case ARCH_TIMER_REG_TVAL:
                asm volatile("mcr p15, 0, %0, c14, c3, 0" : : "r" (val));
            break;
        }
    }

    isb();
}

static inline u32 arch_timer_reg_read(const int access, const int reg)
{
    u32 val = 0;

    if (access == ARCH_TIMER_PHYS_ACCESS)
    {
        switch (reg)
        {
            case ARCH_TIMER_REG_CTRL:
                asm volatile("mrc p15, 0, %0, c14, c2, 1" : "=r" (val));
            break;
            case ARCH_TIMER_REG_TVAL:
                asm volatile("mrc p15, 0, %0, c14, c2, 0" : "=r" (val));
            break;
        }
    }

    if (access == ARCH_TIMER_VIRT_ACCESS)
    {
        switch (reg)
        {
            case ARCH_TIMER_REG_CTRL:
                asm volatile("mrc p15, 0, %0, c14, c3, 1" : "=r" (val));
            break;
            case ARCH_TIMER_REG_TVAL:
                asm volatile("mrc p15, 0, %0, c14, c3, 0" : "=r" (val));
            break;
       }
    }
    return val;
}

static inline u64 arch_counter_get_cntpct(void)
{
    u64 cval;

    isb();
    asm volatile("mrrc p15, 0, %Q0, %R0, c14" : "=r" (cval));
    return cval;
}
#ifdef CONFIG_SYS_SUPPORT_MULTITHREAD
static void _timer_config(void)
{
    // configure coresight CNTCR
    rtd_outl(WRAP_A7_APB_TSG_EN0, 0);
    rtd_outl(WRAP_A7_APB_TSG_EN1, 1);

    // configure coresight CNTFID0
    rtd_outl(WRAP_A7_APB_TSG_EN0, 0x20);
    rtd_outl(WRAP_A7_APB_TSG_EN1, 0x019bfcc0);
}

static void _timer_start(unsigned long evt)
{
    unsigned long ctrl;

    ctrl = arch_timer_reg_read(ARCH_TIMER_PHYS_ACCESS, ARCH_TIMER_REG_CTRL);
    ctrl |= ARCH_TIMER_CTRL_ENABLE;
    ctrl &= ~ARCH_TIMER_CTRL_IT_MASK;

    arch_timer_reg_write(ARCH_TIMER_PHYS_ACCESS, ARCH_TIMER_REG_TVAL, evt);
    arch_timer_reg_write(ARCH_TIMER_PHYS_ACCESS, ARCH_TIMER_REG_CTRL, ctrl);
}

#endif

unsigned long get_boot_time_ms(void)
{
	return (rtd_inl(TIMER_TC7CVR_reg) / (TIMER_CLOCK / CONFIG_SYS_HZ));
}

unsigned long get_boot_time_us(void)
{
	return (rtd_inl(TIMER_TC7CVR_reg) / (TIMER_CLOCK / (CONFIG_SYS_HZ * 1000)));
}

unsigned long get_firstboot_time_us(void)
{
	unsigned int* first_boot_time_ptr = (unsigned int *)FIRST_BOOT_BOOTING_TIME_VALUE;
	return *first_boot_time_ptr;
}

#ifdef CONFIG_SYS_SUPPORT_MULTITHREAD
static void _timer_handler(void)
{
	s32 u32CoreId;

	if (g_rt_timer_init == 0)
		return;

	u32CoreId = smp_processor_id(get_MPIDR());

	// FIXME: need to check which timer has expired
	if (g_rt_timer[u32CoreId][0].started == 1)
	{
		if (g_rt_timer[u32CoreId][0].pTimerCb != NULL)
		{
			if (g_rt_timer[u32CoreId][0].u32PeriodTimeMs == 0) // is one-shot
			{
				g_rt_timer[u32CoreId][0].started = 0;
			}

			g_rt_timer[u32CoreId][0].pTimerCb(0, 0);
		}
	}
}

int arch_timer_handler(void)
{
	unsigned long ctrl;

	ctrl = arch_timer_reg_read(ARCH_TIMER_PHYS_ACCESS, ARCH_TIMER_REG_CTRL);
	if (ctrl & ARCH_TIMER_CTRL_IT_STAT) {
		ctrl |= ARCH_TIMER_CTRL_IT_MASK;
		arch_timer_reg_write(ARCH_TIMER_PHYS_ACCESS, ARCH_TIMER_REG_CTRL, ctrl);
		_timer_handler(); // call handler
		return 1;
	}
	return 0;
}

u32 arch_timer_init(void)
{
	if (g_rt_timer_init == 1)
		return TIMER_SUCCESS;

	memset(g_rt_timer, 0, sizeof(g_rt_timer));

	_timer_config();

//	RtOS_AttachInterrupt(PPI_PHYS_SECURE, rtk_arch_timer_handler);
	RtOS_EnableInterrupt(PPI_PHYS_SECURE);

	g_rt_timer_init = 1;

	return TIMER_SUCCESS;
}
#endif

int timer_init(void)
{
#ifdef CONFIG_REALTEK_TIMER
	//Disable Interrupt
	rtd_outl(TIMER_TCICR_reg, 1 << timer_base->icr_ie_shift);

	//Disable Timer
	rtd_outl(timer_base->cr, TIMERINFO_TIMER_DISABLE);

	//Set The Initial Value
	rtd_outl(timer_base->tvr, TIMER_LOAD_VAL);

	//Enable Timer Mode
	rtd_outl(timer_base->cr, TIMERINFO_TIMER_ENABLE);
#endif

	return 0;
}

#ifdef CONFIG_SYS_SUPPORT_MULTITHREAD
u32 arch_timer_init_core(void)
{
	RtOS_EnableInterrupt(PPI_PHYS_SECURE);

	return TIMER_SUCCESS;
}


static u32 __arch_create_timer (TimerCb pTimerCb, u32 u32FirstTimeMs, u32  u32PeriodTimeMs, u32  bStartTimer, char *pName,u32  u32CoreId)
{
    u32 timerId = 0;

    if (g_rt_timer_init == 0)
        return TIMER_IS_NOT_INITIALIZED;

    if (pTimerCb == NULL)
        return TIMER_FAILED;

    while (g_rt_timer[u32CoreId][timerId].pTimerCb != NULL)
    {
        timerId++;
        if (timerId >= RT_TIMER_TOTAL_NUM_PRE_CORE)
            return TIMER_HANDLER_IS_FULL;
    }

    memset(&g_rt_timer[u32CoreId][timerId], 0, sizeof(Rt_Timer));

    g_rt_timer[u32CoreId][timerId].pTimerCb = pTimerCb;

    g_rt_timer[u32CoreId][timerId].u32FirstTimeMs = u32FirstTimeMs;

    if (u32PeriodTimeMs == 0xF5F5F5F5) // one-shot
        g_rt_timer[u32CoreId][timerId].u32PeriodTimeMs = 0;
    else // period
        g_rt_timer[u32CoreId][timerId].u32PeriodTimeMs = u32PeriodTimeMs;

    strncpy(g_rt_timer[u32CoreId][timerId].name, pName, 32);

    g_rt_timer[u32CoreId][timerId].started = bStartTimer;

    if (bStartTimer)
    {
        _timer_start(u32FirstTimeMs * TICKS_MSEC);
    }

    return (timerId + u32CoreId * 100);
}


static u32 __arch_reset_timer (s32   s32TimerId,u32  u32FirstTimeMs,u32  u32PeriodTimeMs,u32 bStartTimer, u32 u32CoreId)
{
    u32 timerId = 0;

    if (g_rt_timer_init == 0)
        return TIMER_IS_NOT_INITIALIZED;

    timerId = s32TimerId % 100;

    if ((u32CoreId >= 4) || (timerId >= RT_TIMER_TOTAL_NUM_PRE_CORE))
        return TIMER_FAILED;

    g_rt_timer[u32CoreId][timerId].u32FirstTimeMs = u32FirstTimeMs;

    if (u32PeriodTimeMs == 0xF5F5F5F5) // one-shot
        g_rt_timer[u32CoreId][timerId].u32PeriodTimeMs = 0;
    else // period
        g_rt_timer[u32CoreId][timerId].u32PeriodTimeMs = u32PeriodTimeMs;

    g_rt_timer[u32CoreId][timerId].started = bStartTimer;

    if (bStartTimer)
    {
        _timer_start(g_rt_timer[u32CoreId][timerId].u32FirstTimeMs * TICKS_MSEC);
    }

	return TIMER_SUCCESS;
}

static u32 __arch_start_timer (s32 s32TimerId, u32 u32CoreId)
{
	u32 timerId = 0;

	if (g_rt_timer_init == 0)
		return TIMER_IS_NOT_INITIALIZED;

	timerId = s32TimerId % 100;

	if ((u32CoreId >= 4) || (timerId >= 
RT_TIMER_TOTAL_NUM_PRE_CORE))
		return TIMER_FAILED;

	g_rt_timer[u32CoreId][timerId].started = 1;

	return TIMER_SUCCESS;
}
u32 arch_create_timer(TimerCb pTimerCb, u32 ElapseTimeMs, u32 Mode, char*pName, u32 CoreID)
{
    if(Mode == ONE_SHOT_TIMER)
        return __arch_create_timer(pTimerCb, ElapseTimeMs, 0xF5F5F5F5, 1, pName, CoreID);
    else
        return __arch_create_timer(pTimerCb, ElapseTimeMs, ElapseTimeMs, 1, pName, CoreID);
}

u32 arch_reset_timer(u32 u32TimerId, u32 ElapseTimeMs, u32 Mode)
{
    u32 CoreID;
    CoreID = u32TimerId/100;

    if(Mode == ONE_SHOT_TIMER)
        return __arch_reset_timer(u32TimerId, ElapseTimeMs, 0xF5F5F5F5, 1, CoreID);
    else
        return __arch_reset_timer(u32TimerId, ElapseTimeMs, ElapseTimeMs, 1, CoreID);
}

u32 arch_start_timer(u32 u32TimerId)
{
    u32 CoreID;
    CoreID = u32TimerId/100;

    return __arch_start_timer(u32TimerId, CoreID);
}

u64 arch_counter_get_ticks(void)
{
    return arch_counter_get_cntpct();
}

#endif
/*
 * timer without interrupts
 */
unsigned long get_timer(unsigned long base)
{
#ifdef CONFIG_REALTEK_TIMER
	return get_timer_masked() - base;
#else
	return 0;
#endif
}

/* delay x useconds */
void __udelay(unsigned long usec)
{
#ifdef CONFIG_REALTEK_TIMER
	long tmo = usec * (TIMER_CLOCK / 1000) / 1000;
	unsigned long now, last = rtd_inl(timer_base->cvr);

	while (tmo > 0) {
		now = rtd_inl(timer_base->cvr);
		if (now < last) /* count up timer overflow */
			tmo -= TIMER_OVERFLOW_VAL - last + now + 1;
		else
			tmo -= now - last;
		last = now;
	}
#endif
}

unsigned long get_timer_masked(void)
{
#ifdef CONFIG_REALTEK_TIMER
	/* current tick value */
	unsigned long now = rtd_inl(timer_base->cvr) / (TIMER_CLOCK / CONFIG_SYS_HZ);

	if (now >= gd->lastinc)	/* normal mode (non roll) */
		/* move stamp fordward with absoulte diff ticks */
		gd->tbl += (now - gd->lastinc);
	else	/* we have rollover of incrementer */
		gd->tbl += ((TIMER_LOAD_VAL / (TIMER_CLOCK / CONFIG_SYS_HZ))
			     - gd->lastinc) + now;
	gd->lastinc = now;
	return gd->tbl;
#else
	return 0;
#endif
}

static unsigned long g_last_usec = 0;
static unsigned long g_last_tbl = 0;
unsigned long get_timer_run_usec(void)
{
#ifdef CONFIG_REALTEK_TIMER
	/* current tick value */
	unsigned long now = rtd_inl(timer_base->cvr) / (TIMER_CLOCK / (CONFIG_SYS_HZ * CONFIG_SYS_HZ));

	if (now >= g_last_usec)	/* normal mode (non roll) */
		/* move stamp fordward with absoulte diff ticks */
		g_last_tbl += (now - g_last_usec);
	else	/* we have rollover of incrementer */
		g_last_tbl += ((TIMER_LOAD_VAL / (TIMER_CLOCK / (CONFIG_SYS_HZ * CONFIG_SYS_HZ)))
			     - g_last_usec) + now;
	g_last_usec = now;
	return g_last_tbl;
#else
	return 0;
#endif
}

#ifndef CONFIG_BSP_REALTEK // mark unused code
/*
 * This function is derived from PowerPC code (read timebase as long long).
 * On ARM it just returns the timer value.
 */
unsigned long long get_ticks(void)
{
#ifdef CONFIG_REALTEK_TIMER
	return get_timer(0);
#else
	return 0;
#endif
}

/*
 * This function is derived from PowerPC code (timebase clock frequency).
 * On ARM it returns the number of timer ticks per second.
 */
unsigned long get_tbclk(void)
{
	return CONFIG_SYS_HZ;
}
#endif

