/* Copyright (C) 2011
 *  * Corscience GmbH & Co. KG - Simon Schwarz <schwarz@corscience.de>
 *   *  - Added prep subcommand support
 *    *  - Reorganized source - modeled after powerpc version
 *     *
 *      * (C) Copyright 2002
 *       * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 *        * Marius Groeger <mgroeger@sysgo.de>
 *         *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
 *
 */

#include <common.h>
#include <command.h>
#include <image.h>
#include <u-boot/zlib.h>
#include <asm/byteorder.h>
#include <fdt.h>
#include <libfdt.h>
#include <fdt_support.h>
#include <asm/bootm.h>
#include <exports.h>
#include <environment.h>
#include <wakeup.h>
#include <asm/arch/system.h>
#include <asm/arch/fw_info.h>
#include <asm/arch/bootparam.h>
#include <platform_lib/board/pcb_mgr.h>
#include <watchdog.h>
#include <factorylib.h>

#define BOOTREASON_PRI_mask     (0x000000FF)
#define BOOTREASON_REG          STB_SW_DATA12_reg  //0xB806016C
typedef enum {
	COLD_BOOT,
	WARM_BOOT,
	WATCHDOG,
	RECOVERY,
	KERNEL_PANIC,
	REBOOT,
	SHUTDOWN,
	RTC_WAKEUP
}eBootreason;
#define arg_next(p)             (char *)((unsigned long)p+strlen(p))
#define WDT_CNT_CLEAR		0xF00
extern int boot_power_on_mode;
void set_boot_reason(char *kargs)
{
	extern uchar boot_ac_on;
	uchar wdt_count = 0;
	eBootreason ebr = COLD_BOOT;
	uint boot_cause = 0;
	if (boot_ac_on) {
		if (boot_mode != BOOT_RESCUE_MODE)
		{
			ebr = COLD_BOOT;
		}
		else
		{
			ebr = RECOVERY;
		}
	}
	else
	{
		if(boot_power_on_mode == BACKLIGHT_OFF)
		{
			ebr = RTC_WAKEUP;
		}
		else
		{
			ebr = WARM_BOOT;
		}
	}
	wdt_count = (rtd_inl(ISO_MISC_TCWCR_reg)>> 8)&0x7;
	if(wdt_count)
	{
		ebr = WATCHDOG;
		/* Clear watchdog counts. */
		boot_cause = rtd_inl(ISO_MISC_TCWCR_reg);
		boot_cause |= WDT_CNT_CLEAR;
		rtd_outl(ISO_MISC_TCWCR_reg, boot_cause);
	}

	boot_cause = rtd_inl(BOOTREASON_REG) & BOOTREASON_PRI_mask;
	if(boot_cause) {
		if(boot_cause == REBOOT)
			ebr = REBOOT;
		else if(boot_cause == KERNEL_PANIC)
			ebr = KERNEL_PANIC;
		else if(boot_cause == SHUTDOWN)
			ebr = SHUTDOWN;
	}


	switch(ebr)
	{
		case COLD_BOOT:
			sprintf(arg_next(kargs),"androidboot.bootreason=cold ");
		break;
		case WARM_BOOT:
			sprintf(arg_next(kargs),"androidboot.bootreason=warm ");
		break;
		case WATCHDOG:
			sprintf(arg_next(kargs),"androidboot.bootreason=watchdog ");
		break;
		case RECOVERY:
			sprintf(arg_next(kargs),"androidboot.bootreason=recovery ");
		break;
		case KERNEL_PANIC:
			sprintf(arg_next(kargs),"androidboot.bootreason=kernel_panic ");
		break;
		case REBOOT:
			sprintf(arg_next(kargs),"androidboot.bootreason=reboot ");
		break;
		case SHUTDOWN:
			sprintf(arg_next(kargs),"androidboot.bootreason=shutdown ");
		break;
		case RTC_WAKEUP:
			sprintf(arg_next(kargs),"androidboot.bootreason=rtc androidboot.quiescent=1 ");
		break;
		default:
			printf("BOOT REASON UNKNOWN !!!! \n");
	}
	/* Shift current reason to previous reason bit field */
	boot_cause = rtd_inl(BOOTREASON_REG);
	rtd_outl(BOOTREASON_REG, (boot_cause << 16));
}
