/*
 * cmd_test_dhrystone.c - Realtek
 *
 *  Copyright (C) 2020 Realtek
 *   HsinChu Chen <hcchen@realtek.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <common.h>
#include <command.h>
#include <linux/compiler.h>
#include <asm/arch/system.h>
#include <malloc.h>
#include <arm_test/saxpy_main.h>

extern int main_dhrystone(void);
extern void Cache_miss_test(void);
extern void arm_ok(void);
extern void arm_hello_world(void);
extern void main_crypto(void);

void arm_pattern_cmd_help(void)
{
    printf("\n");
    printf("\e[1;34mUsage: arm_pattern <pattern>\e[0m\n\n");

    printf("\t The most commonly used arm_pattern commands are:\n\n");
    printf("\t %-20s %s\n", "arm_pattern dhrystone" , "ARM pattern test by dhrystone.(cache, integer pipeline)\n");
    printf("\t %-20s %s\n", "arm_pattern saxpy"     , "ARM pattern test by Saxpy_SIMD.(neon)\n");
    printf("\t %-20s %s\n", "arm_pattern crypto"    , "ARM pattern test by crypto.(crypto engine)\n");
    printf("\t %-20s %s\n", "arm_pattern cache"     , "ARM pattern test by cache_miss.(cache)\n");
    printf("\t %-20s %s\n", "arm_pattern arm_ok"    , "ARM pattern test by Hello_world.(fetch, execution)\n");
    printf("\t %-20s %s\n", "arm_pattern hello"     , "ARM pattern test by arm_ok.(branch, ALU)\n");
    //printf("\t\n");
    printf("----------------------------------------\n");

}

void print_pass_msg(void) {
    printf("** TEST PASSED OK **\n\n");
    return;
}
void print_fail_msg(void) {
    printf("** TEST FAILED **\n\n");
    return;
}

int do_dhrystone(void)
{
    printf("** ARM test pattern : dhrystone start **\n");
    main_dhrystone();
    //printf("** ARM test pattern : dhrystone end **\n");
    return 0;
}

/*
 * Term    : Pattern testing by NEON instruction
 * Features: Basic_Linear_Algebra_Subprograms: Vector operations on strided arrays: dot products.
 * Function: y[]=a*x[]+y[]
 * @param y vector y (array)
 * @param x vector x (array)
 * @param a factor a.
 */

extern unsigned long get_timer_run_usec(void);
extern void lowlevel_init(void);

static float *x2 __attribute__ ((aligned (64))) = NULL;
static float *y2 __attribute__ ((aligned (64))) = NULL;

int do_saxpy_simd(void)
{
    int index = 0, array = 0;
    unsigned long start = 0;
    unsigned long neon_elapsed = 0;
    unsigned long normal_elapsed = 0;
    
    x2 = malloc(sizeof(saxpy_x));
    y2 = malloc(sizeof(saxpy_y));

    if((x2 == NULL) || (y2 == NULL)) {
        printf(" %s:%d Allocate memory failed \n", __FUNCTION__, __LINE__);
        return 1;
    }

    // Want to get hex number into the float variable
    // without a int->float conversion. Got to use pointers to
    // prevent C from casting to float.
    const unsigned int int_const_d = 0x4e696969;
    const float *float_const_d = (float *) &int_const_d;

#if 0
    asm volatile (
    /* Enable the the VFP */
       " mrc p15, 0, r1, c1, c0, 2 \n\t"
       " orr r1, r1, #(0x3 << 20) \n\t"
       " orr r1, r1, #(0x3 << 20) \n\t"
       " mcr p15, 0, r1, c1, c0, 2 \n\t"
       " isb   \n\t"
       " fmrx    r1, FPEXC \n\t"
       " orr r1,r1, #(1<<30) \n\t"
       " fmxr    FPEXC, r1 \n\t"
       );
#endif
    memcpy(x2, saxpy_x, sizeof(x2));
    memcpy(y2, saxpy_y, sizeof(y2));

    /* Enable NEON and FPU */
    lowlevel_init();  // arch/arm/cpu/armv7/start.S

    printf("\n ** ARM test pattern : Saxpy by NEON instruction start **\n\n");
    printf(" - Function = y[]=a*x[]+y[] , Array size = %d \n", SAXPY_ARRAY_SIZE);
    //printf(" (x[0],y[0], a)=(%d , %d, %d) \n", (int)x[0], (int)y[0], (int)*float_const_d);
    // Start statistics capture here
    MAINSTART
    BENCHSTART

    start = get_timer_run_usec();
    for(index = 0; index < NLOOPS; index++)
    {
        LOOPSTART
        saxpy_asm(SAXPY_ARRAY_SIZE, saxpy_x, saxpy_y, *float_const_d);
        LOOPEND
        //printf("(x[0],y[0])=(%d , %d) \n", (int)x[0], (int)y[0]);
    }
    start = get_timer_run_usec() - start;
    printf("\n 1. NEON Elapsed time: %lu us\n", start);
    neon_elapsed = start;
    // End statistics capture
    BENCHFINISHED
    MAINEND

    start = get_timer_run_usec();

    for(index = 0; index < NLOOPS; index++)
    {
        for(array = 0; array < SAXPY_ARRAY_SIZE; array++)
        {
            y2[array] = (*float_const_d)*x2[array] + y2[array];
        }
    }
    start = get_timer_run_usec() - start;
    printf("\n 2. None NEON Elapsed time: %lu us\n", start);
    normal_elapsed = start;

    if( neon_elapsed < (normal_elapsed/10) )
        printf("\n ** TEST PASSED **\t\n");
    else
        printf("\n ** TEST FAILED **\t\n");

    //printf("\n ** ARM test pattern - Saxpy by NEON instruction end **\n");

    if(x2) {
        free(x2);
        x2 = NULL;
    }
    if(y2) {
        free(y2);
        y2 = NULL;
    }
    return 0;
}

int do_cache_miss(void) {
    printf("\n ** ARM test pattern : Cache Miss **\t\n");
    Cache_miss_test();
    printf("\n");
    return 0;
}

int do_arm_ok(void) {
    printf("\n ** ARM test pattern : Branch, ALU **\t\n");
    arm_ok();
    printf("\n");
    return 0;
}

int do_hello_world(void) {
    printf("\n ** ARM test pattern : Fetch, Execution **\t\n");
    arm_hello_world();
    printf("\n");
    return 0;
}

int do_crypto(void) {
    printf("\n ** ARM test pattern : Crypto engine **\t\n");
    main_crypto();
    printf("\n");
    return 0;
}

int arm_pattern_command(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    if(argc == 2) {
        if (strcmp(argv[1], "dhrystone") == 0) {
            return do_dhrystone();
        }
        if (strcmp(argv[1], "saxpy") == 0) {
            return do_saxpy_simd();
        }
        if (strcmp(argv[1], "cache") == 0) {
            return do_cache_miss();
        }
        if (strcmp(argv[1], "arm_ok") == 0) {
            return do_arm_ok();
        }
        if (strcmp(argv[1], "hello") == 0) {
            return do_hello_world();
        }
        if (strcmp(argv[1], "crypto") == 0) {
            return do_crypto();
        }
        else {
            goto FAIL;
        }
    }
    else if(argc == 1) {
        goto FAIL;
    }

//OK:
    return 0;

FAIL:
    arm_pattern_cmd_help();
    return 1;
}

/* -------------------------------------------------------------------- */

U_BOOT_CMD(
	arm_pattern,	2,	1,	arm_pattern_command,
	"ARM Pattern Test",
	""
);
