#include <common.h>
#include <command.h>

#include <config.h>
#include <ns16550.h>
#include <watchdog.h>
#include <linux/types.h>
#include <asm/io.h>
#include <asm/arch/system.h>

//#define UART1_DMA

#define UART_LCRVAL UART_LCR_8N1		/* 8 data, 1 stop, no parity */
#define UART_MCRVAL (UART_MCR_DTR | \
		     UART_MCR_RTS)		/* RTS/DTR */
#define UART_FCRVAL (UART_FCR_FIFO_EN |	\
		     UART_FCR_RXSR |	\
		     UART_FCR_TXSR)		/* Clear & enable FIFOs */


struct NS16550Dma {
	UART_REG(rbr);		/* 0 */
	UART_REG(ier);		/* 1 */
	UART_REG(fcr);		/* 2 */
	UART_REG(lcr);		/* 3 */
	UART_REG(mcr);		/* 4 */
	UART_REG(lsr);		/* 5 */
	UART_REG(msr);		/* 6 */
	UART_REG(spr);		/* 7 */

};

#define thr rbr
#define iir fcr
#define dll rbr
#define dlm ier

typedef struct NS16550Dma *NS16550Dma_t;

 #define serial_out(x, y)	rtd_outl((unsigned int)y, x)
 #define serial_in(y)		rtd_inl((unsigned int)y)

DECLARE_GLOBAL_DATA_PTR;


struct uart_8250_rtk_dma {
	void *tx_dma_ring_buf_head;
	void *tx_dma_ring_buf_align;
	dma_addr_t tx_dma_addr;
	int tx_dma_ring_buf_size;


	unsigned char *dma_membase;
	int dma_membase_ofst;
	unsigned int dma_ring_buf_len;
	unsigned int dma_tx_thd;
	unsigned int dma_tx_empty_thd;
};


#if 0
#if (UART_PORT == 0)
 #define V_NS16550_CLK			27000000
 #define CONFIG_SYS_NS16550_COM1	UART0_BASE
#elif (UART_PORT == 1)
 #define V_NS16550_CLK			27000000 // 432000000
 #define CONFIG_SYS_NS16550_COM1	UART1_BASE
#endif
#endif

// #define UART0_BASE			(0x18062300)
// #define UART1_BASE			(0x1801b100)

 #define V_NS16550_CLK			27000000 // 432000000
 #define CONFIG_SYS_NS16550_COM1_DMA	UART1_BASE


#ifdef CONFIG_REALTEK_UART_DMA
#define UART_DMA_CTRL		(0)
#define UART_DMA_STS		(1)
#define UART_DMA_INT		(2)
#define UART_DMA_INT_EN		(3)
#define UART_DMA_TXBASE		(4)
#define UART_DMA_TXLIMIT	(5)
#define UART_DMA_TXWRPTR	(6)
#define UART_DMA_TXRDPTR	(7)
#define UART_DMA_TXTHD		(8)
#define UART_DMA_GATING_EN_CTRL	(9)
#endif

#define UR1_DMA_CTRL			(0x18039200)
#define UR1_DMA_CTRL_reg		(0xB8039200)
#define UR1_DMAIFEN(x)			(x << 6)
#define UR1_TXEMPTHR(x)			(x & 0x1F)
#define UR1_DMAM(x)			(x << 3)

#define UR1_DMA_STS			(0x18039204)
#define UR1_DMA_STS_reg			(0xB8039204)

#define UR1_DMA_INT			(0x18039208)
#define UR1_DMA_INT_reg			(0xB8039208)
#define UR1_UR_TX_THD			(1 << 1)
#define UR1_WRITE_DATA(x)		(x << 0)

#define UR1_DMA_INT_EN			(0x1803920C)
#define UR1_DMA_INT_EN_reg		(0xB803920C)
#define UR1_UR_TX_THD_EN(x)		(x << 0)

#define UR1_DMA_TXBASE			(0x18039210)
#define UR1_DMA_TXBASE_reg		(0xB8039210)
#define UR1_TX_BASE(x)			(x & 0x7FFFFFF8)


#define UR1_DMA_TXLIMIT			(0x18039214)
#define UR1_DMA_TXLIMIT_reg		(0xB8039214)
#define UR1_TX_LIMIT(x)			(x & 0x7FFFFFF8)

#define UR1_DMA_TXWRPTR			(0x18039218)
#define UR1_DMA_TXWRPTR_reg		(0xB8039218)
#define UR1_TX_WRPTR(x)			(x & 0x7FFFFFFF)

#define UR1_DMA_TXRDPTR			(0x1803921C)
#define UR1_DMA_TXRDPTR_reg		(0xB803921C)
#define UR1_TX_TXRDPTR(x)		(x & 0x7FFFFFFF)

#define UR1_DMA_TXTHD			(0x18039220)
#define UR1_DMA_TXTHD_reg		(0xB8039220)
#define UR1_TX_THD(x)			(x)

#define UR1_DMA_GATING_EN_CTRL		(0x18039224)
#define UR1_DMA_GATING_EN_CTRL_reg	(0xB8039224)
#define UR1_GATING_EN(x)		(x << 0)



int calc_divisor(void)
{

#define MODE_X_DIV 16

	/* Compute divisor value. Normally, we should simply return:
	 *   CONFIG_SYS_NS16550_CLK) / MODE_X_DIV / gd->baudrate
	 * but we need to round that value by adding 0.5.
	 * Rounding is especially important at high baud rates.
	 */
	return (CONFIG_SYS_NS16550_CLK + (gd->baudrate * (MODE_X_DIV / 2))) /
		(MODE_X_DIV * gd->baudrate);
}


//void initUart1(NS16550_t com_port, int baud_divisor)
void initUart1( void )
{
//	com_port
//	baud_divisor
	int baud_divisor;
	NS16550Dma_t com_port = (NS16550Dma_t)CONFIG_SYS_NS16550_COM1_DMA;

	//baud_divisor = calc_divisor(com_port);
	baud_divisor = calc_divisor();

	serial_out(UART_LCR_BKSE | UART_LCRVAL, (ulong)&com_port->lcr);
	serial_out(0, &com_port->dll);
	serial_out(0, &com_port->dlm);
	serial_out(UART_LCRVAL, &com_port->lcr);
	serial_out(UART_MCRVAL, &com_port->mcr);
	serial_out(UART_FCRVAL, &com_port->fcr);
	serial_out(UART_LCR_BKSE | UART_LCRVAL, &com_port->lcr);
	serial_out(baud_divisor & 0xff, &com_port->dll);
	serial_out((baud_divisor >> 8) & 0xff, &com_port->dlm);
	serial_out(UART_LCRVAL, &com_port->lcr);

}


void _NS16550_putc(NS16550Dma_t com_port, char c)
{
	while ((serial_in(&com_port->lsr) & UART_LSR_THRE) == 0)
		;
	serial_out(c, &com_port->thr);

	/*
	 * Call watchdog_reset() upon newline. This is done here in putc
	 * since the environment code uses a single puts() to print the complete
	 * environment upon "printenv". So we can't put this watchdog call
	 * in puts().
	 */
	if (c == '\n')
		WATCHDOG_RESET();
}

void _uart1_putc(const char c)
{
	NS16550Dma_t com_port = (NS16550Dma_t)CONFIG_SYS_NS16550_COM1_DMA;

	_NS16550_putc(com_port, c);
}

void outputUart1(const char *s)
{
	while (*s) {
		_uart1_putc (*s++);
	}
}

/***dma section***/

void initUart1Dma( void )
{
//	com_port
//	baud_divisor
	int baud_divisor;
	NS16550Dma_t com_port = (NS16550Dma_t)CONFIG_SYS_NS16550_COM1_DMA;

	//baud_divisor = calc_divisor(com_port);
	baud_divisor = calc_divisor();

	serial_out(UART_LCR_BKSE | UART_LCRVAL, (ulong)&com_port->lcr);
	serial_out(0, &com_port->dll);
	serial_out(0, &com_port->dlm);
	serial_out(UART_LCRVAL, &com_port->lcr);
	serial_out(UART_MCRVAL, &com_port->mcr);
	serial_out(UART_FCRVAL, &com_port->fcr);
	serial_out(UART_LCR_BKSE | UART_LCRVAL, &com_port->lcr);
	serial_out(baud_divisor & 0xff, &com_port->dll);
	serial_out((baud_divisor >> 8) & 0xff, &com_port->dlm);
	serial_out(UART_LCRVAL, &com_port->lcr);
}


void outputUart1Dma(const char *s)
{

	int length = strlen(s);

	NS16550Dma_t com_port = (NS16550Dma_t)CONFIG_SYS_NS16550_COM1_DMA;
	int ierVal = serial_in(&com_port->ier);

	unsigned int dma_base = 0x2B300200 ; //refer from dw_em_make_script()


	ierVal = ierVal & (~UART_IER_THRI);  // disable tx interrupt

	//dma_base = malloc_noncache( sizeof(char) * length);
/** data copy **/

	serial_out(UART_LCRVAL, &com_port->lcr);

	serial_out(ierVal , &com_port->ier);

	serial_out(UR1_TX_BASE(dma_base) , UR1_DMA_TXBASE_reg);

	serial_out(UR1_TX_LIMIT(dma_base+length*4) , UR1_DMA_TXLIMIT_reg);

	serial_out(UR1_TX_WRPTR(dma_base) , UR1_DMA_TXWRPTR_reg);

	serial_out(UR1_TX_TXRDPTR(dma_base) , UR1_DMA_TXRDPTR);

	serial_out(UR1_TX_THD(dma_base) , UR1_DMA_TXTHD_reg);

	serial_out(UR1_GATING_EN(1) , UR1_DMA_GATING_EN_CTRL_reg);

	serial_out(UR1_DMAIFEN(1) | UR1_TXEMPTHR(8) , UR1_DMA_CTRL_reg);
}

/***dma section***/

int do_uart1(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	if (argc < 2)
		return CMD_RET_USAGE;

	printf("begin print this string by uart1\n\n");

	printf("print this string  %s    by uart0\n" , argv[1] );

#ifdef UART1_DMA
	initUart1Dma();
	outputUart1Dma(argv[1]);
#else
	initUart1();
	outputUart1(argv[1]);
#endif

	printf("\nend print this string by uart1\n");

	return 0;
}

U_BOOT_CMD(
	uart1,	2,	1,	do_uart1,
	"send output string by uart1",
	"\"xxx\"\n"
	"    - send output string by uart1\n"
);
