#include <common.h>
#include <burn.h>
#include <linux/mtd/mtd.h>
#include "rtk_spi_nand.h"
#include "rtk_spi_nand_base.h"

#define REG8_READ(register)          rtd_inb(register)
#define REG32_READ(register)          rtd_inl(register)
#define REG8_WRITE(register, value)   rtd_outb(register, value)
#define REG32_WRITE(register, value)  rtd_outl(register, value)
#define REG32_MASK(register, andMask, orMask)  rtd_outl(register, ((rtd_inl(register) & (andMask)) | (orMask)))


#define BYTE(d,n)		(((d) >> ((n) << 3)) & 0xFF)
#define SWAPEND32(d)	((BYTE(d,0) << 24) | (BYTE(d,1) << 16) | (BYTE(d,2) << 8) | (BYTE(d,3) << 0))
#define _SPI_NAND_CTRL
DEBUG_SN_SET_LEVEL(DEBUG_LEVEL_INFO);

#ifdef CONFIG_MTD_TABLE_TEST
int set_erase_fail = 0;
int set_write_fail = 0;
int set_read_fail = 0;
#endif

int wait_device_ready(int wait_delay);

void dump_byte(char *buf, int buf_size)
{
    int loop = 0;
    SN_ERR("buf 0x%x size 0x%x", buf, buf_size);
    for(loop = 0; loop < buf_size; loop++)
    {
        if(loop%16 == 0)
        {
            SN_ERR("\n%08xh:",loop); 
        }
        SN_ERR(" %02x",*buf);
        buf++;
    }
    SN_ERR("\n");	   
}

void dump_reg(unsigned int base_reg, unsigned int reg_num)
{
    int loop = 0;
    SN_ERR("base register 0x%x number %d",base_reg,reg_num);  
    for(loop = 0; loop < reg_num; loop++)
    {
        if(loop%4 == 0)
	    {
            SN_ERR("\n%08xh:",(base_reg+loop*4));
	    }
	    SN_ERR(" %08x",REG32_READ(base_reg+4*loop));  
    }
    SN_ERR("\n");	
}

void dump_sn_reg()
{
    int reg_addr = SFC_REG_BASE;
    SN_ERR("%s *******begin********** \n",__FUNCTION__);  
    dump_reg(reg_addr,8);
    reg_addr = MD_REG_BASE + 0x88;
    dump_reg(reg_addr,4);
    reg_addr = 0x18000848;
    dump_reg(reg_addr,7);
    SN_ERR("%s ******end************ \n",__FUNCTION__); 
}

static int swap_endian(UINT32 input)
{
    UINT32 output;
    output = (input & 0xff000000)>>24|
			 (input & 0x00ff0000)>>8|
			 (input & 0x0000ff00)<<8|
			 (input & 0x000000ff)<<24;
	
    return output;
}

void sfc_reset(void)
{	
    UINT8 tmp = 0;
    /*send reset command*/
    REG32_WRITE(SFC_OPCODE,CMD_RESET&CMD_MASK);
    /*data  address disable*/
    REG32_WRITE(SFC_CTL,0x00000000);    /* dataen = 0 adren = 0, dmycnt = 0 */
    tmp = REG8_READ(SPI_NAND_BASE_ADDR);
    /*wait reset ok*/
    if (wait_device_ready(SFC_DELAY)!=0) 
    {
        SN_ERR("%s device not ready \n",__FUNCTION__);
		dump_sn_reg();
    }
}

#ifdef _SPI_NAND_CTRL
void set_spi_n_address(unsigned int addr_cnt, unsigned int addr)
{
    REG32_WRITE(SPI_NAND_0,((addr_cnt&ADR_CNT_MASK)<<ADR_CNT_OFFSET)|(0x1<<ADRCNT_EN_OFFSET)|(addr&ADDR_MASK));
}

void clr_spi_n_address()
{
    REG32_WRITE(SPI_NAND_0,0);
}
#endif

void set_spi_nand_pin_mux()
{
	REG32_WRITE(0x1801a960,0x00000000);
	/*pad_gpio_64_io SPI_HOLD <I/O>,<HS>[31:28] = 0x2*/
	REG32_MASK(0x18000848, 0x0fffffff, (2 << 28));

	/*pad_gpio_69_io SPI_WP <I/O>,<HS>[23:20] = 0x2*/
	REG32_MASK(0x1800084C, 0xff0fffff, (2 << 20));
	
	/*pad_gpio_84_io SPI_DO <I/O>,<HS>[31:28] = 0x2*/
	REG32_MASK(0x1800085C, 0x0fffffff, (2 << 28));

	/*pad_gpio_67_io SPI_SCLK <I/O>,<HS>[7:4] = 0x2*/
	REG32_MASK(0x18000848, 0xffffff0f, (2 << 4));
	
	/*pad_gpio_75_io SPI_CS_N <I/O>,<HS>[7:4] = 0x2*/
	REG32_MASK(0x18000850, 0xffffff0f, (2 << 4));
	
	/*pad_gpio_73_io SPI_DI <I/O>,<HS>[23:20] = 0x2*/
	REG32_MASK(0x18000850, 0xff0fffff, (2 << 20));
}

void rtk_spi_nand_read_id(struct mtd_info *mtd,unsigned int *id)
{
    unsigned int spi_address = 0;
    /*send read id command*/
    REG32_WRITE(SFC_OPCODE,CMD_READ_ID&CMD_MASK);

    /*set address enable data enable*/  
    REG32_WRITE(SFC_CTL,(1<<DATEN_OFFSET)|(1<<ADREN_OFFSET)); /* dataen = 1, adren = 1, dmycnt = 0 */

    /*send address bit[0:7]  */
#ifdef _SPI_NAND_CTRL
    set_spi_n_address(SPI_ADDR_CNT_1, 0x00);
    spi_address = SPI_NAND_BASE_ADDR;
#else
    spi_address = SPI_NAND_BASE_ADDR + (0x00 << 16);
#endif
    sync();
    udelay(SFC_DELAY);

    /*read data content*/
    *id = REG32_READ(spi_address);
#ifdef LITTLE_ENDIAN
    *id= SWAPEND32(*id);
#endif

    SN_INFO("%s  get spi-nandflash id 0x%x \n",__FUNCTION__,*id);
}

void sfc_get_feature(unsigned char address, unsigned char *data)
{
    unsigned int spi_address = 0;
    /*send get feature command*/
    REG32_WRITE(SFC_OPCODE,CMD_GET_FEATURE&CMD_MASK);
    /*set address enable data enable*/  
    REG32_WRITE(SFC_CTL,(1<<DATEN_OFFSET)|(1<<ADREN_OFFSET)); /* dataen = 1, adren = 1, dmycnt = 0 */
    udelay(SFC_READ_DELAY);

    /*send address bit[0:7]*/
#ifdef _SPI_NAND_CTRL
    set_spi_n_address(SPI_ADDR_CNT_1, 0);
    spi_address = SPI_NAND_BASE_ADDR + address;
#else
    spi_address = SPI_NAND_BASE_ADDR + (address << 16);
#endif
    sync();
    udelay(SFC_READ_DELAY);
    
    /*get data content*/
    *data = REG8_READ(spi_address);
    SN_DEBUG("%s address 0x%x spi_address 0x%x data 0x%x \n",__FUNCTION__,address,spi_address,*data);

#ifdef _SPI_NAND_CTRL
    clr_spi_n_address();
#endif
}

void sfc_set_feature(unsigned char address, unsigned char data)
{
    unsigned int spi_address = 0;
    UINT8 tmp = 0;
    /*send set feature command*/
    REG32_WRITE(SFC_OPCODE,CMD_SET_FEATURE&CMD_MASK);

#ifdef _SPI_NAND_CTRL
    /*set address enable data enable*/ 
    REG32_WRITE(SFC_CTL,(1<<DATEN_OFFSET)|(1<<ADREN_OFFSET)); /* dataen = 1, adren = 1, dmycnt = 0 */
    udelay(SFC_DELAY);
	
    /*send address bit[0:7]*/
    set_spi_n_address(SPI_ADDR_CNT_1, 0);
    spi_address = SPI_NAND_BASE_ADDR  + address;
    tmp = REG8_READ(spi_address);
    sync();
    udelay(SFC_DELAY);

    /*send data*/
    REG8_WRITE(spi_address,data);
#else
    REG32_WRITE(SFC_CTL,0x00000008); /* dataen = 0, adren = 1, dmycnt = 0 */
    spi_address = SPI_NAND_BASE_ADDR + (address << 16) + (data << 8);
    udelay(SFC_DELAY);
    REG32_WRITE(spi_address,0x0);
#endif

#ifdef _SPI_NAND_CTRL
    clr_spi_n_address();
#endif
}

int wait_device_ready(int wait_delay)
{
    int ret = 0, retry = 0;
    UINT8 spi_1byte_data = 0;
    do {
        udelay(wait_delay); //SFC_READ_DELAY 
        sfc_get_feature(GET_STATUS_ADDR, &spi_1byte_data);
    } while ((spi_1byte_data & STA_OIP_MASK)&&(++retry < STATUS_TIMEOUT));

    if (retry == STATUS_TIMEOUT) 
    {
        SN_ERR("%s wait timet out spi_1byte_data 0x%x\n",__FUNCTION__,spi_1byte_data);
        ret = -1;
    }
    return ret;
}

int check_ecc()
{
    int ret = 0;
    UINT8 data = 0;
    sfc_get_feature(GET_STATUS_ADDR, &data);
    switch ((data>>STA_ECCS0_OFFSET)& STA_ECC_MASK) {
        case ECC_RESERVED:
        //bit error & corrected (for GD)
            break;
        case ECC_ERR:
         //bit error & NOT corrected
            ret = -1;
            break;
        case ECC_CORRECT:
        //bit error & corrected
            break;
        case ECC_NO_ERROR:
        //no bit error
            break;
        default:
        //impossible;
            ret = -1;
    }
    return ret;
}

int sfc_read_page_to_cache_b(const unsigned int block_addr,const unsigned int page_addr)
{
    int ret = 0;
    unsigned int spi_address = 0;
    UINT8 tmp = 0;
    /*send read page command*/
    REG32_WRITE(SFC_OPCODE,CMD_READ_PAGE&CMD_MASK);
   
    /*set address enable*/
    REG32_WRITE(SFC_CTL,1<<ADREN_OFFSET); /* dataen = 0, adren = 1, dmycnt = 0 */

    /*send 24bit row address-->16bit row address page address[0:5] block address[6:15] */
#ifdef _SPI_NAND_CTRL
    set_spi_n_address(SPI_ADDR_CNT_3, 0);
    spi_address = SPI_NAND_BASE_ADDR+((block_addr&BLOCK_MASK)<<BLOCK_OFFSET)+(page_addr&PAGE_MASK);
#else
    spi_address = SPI_NAND_BASE_ADDR+((block_addr&BLOCK_MASK)<<BLOCK_OFFSET)+(page_addr&PAGE_MASK);
#if _LITTLE_ENDIAN
    spi_address = spi_address^0x3;
#endif
#endif
    tmp = REG8_READ(spi_address);
    SN_DEBUG("%s block_addr %d page_addr %d spi_address 0x%x %d\n",
    __FUNCTION__,block_addr,page_addr,spi_address, REG8_READ(spi_address));
	//dump_sn_reg();


#ifdef _SPI_NAND_CTRL
    clr_spi_n_address();
#endif
    
    /*wait device ready*/
    if ((ret = wait_device_ready(SFC_READ_DELAY))== 0)
    {
        /*check ecc */
        if ((ret = check_ecc()) < 0) 
        {
            SN_ERR("%s ecc error block_addr %d page_addr %d \n",__FUNCTION__,block_addr,page_addr);
        }
    }
    return ret;  
}

void sfc_set_read_cache_cmd(cache_mode read_cache_mode)
{
    int cache_cmd = 0;
    switch(read_cache_mode)
	{
        case CACHE_1X:
            cache_cmd = CMD_READ_CACHE&CMD_MASK;
			break;
        case CACHE_2X:
            cache_cmd = (CMD_READ_2XCACHE&CMD_MASK)|(DUAL_READ<<DUAL_QUAD_MODE_OFFSET);
            break;
        case CACHE_4X:
            cache_cmd = (CMD_READ_4XCACHE&CMD_MASK)|(QUAD_READ<<DUAL_QUAD_MODE_OFFSET);
            break;
        case CACHE_4X_IO:
            cache_cmd = (CMD_READ_4XCACHE_IO&CMD_MASK)|(QUAD_READ_IO<<DUAL_QUAD_MODE_OFFSET);
            break;
        default:
            cache_cmd = CMD_READ_CACHE&CMD_MASK;
            break;
	}
	//cache_cmd = CMD_READ_CACHE&CMD_MASK;
	REG32_WRITE(SFC_OPCODE,cache_cmd);
}

void sfc_read_cache_to_mem_md(cache_mode read_cache_mode,unsigned int block_addr,UINT8 *dst, UINT32 len)
{
    UINT32 spi_address  = 0;
#ifdef _SPI_NAND_CTRL
    spi_address = SPI_NAND_BASE_ADDR; 
#else
    spi_address = SPI_NAND_BASE_ADDR + ((block_addr & 0x1) << (12 + 8));	//choose plane for micron
#endif

    // flush cache before DMA
    flush_dcache_all();//dcache_flush_invalidate(); //sys_dcache_flush_all();
    
    /*send read cache command*/
    sfc_set_read_cache_cmd(read_cache_mode);
	
#ifdef _SPI_NAND_CTRL
    /*set address and data enable and dummy data = 1*/
    REG32_WRITE(SFC_CTL,(1<<DATEN_OFFSET)|(1<<ADREN_OFFSET)|((1&DMYCNT_MASK)<<DMYCNT_OFFSET));  /* dataen = 1, adren = 1, dmycnt = 1 */
    /*enable spi nandflash and set bit[0:15]-->cloumn address page */
    set_spi_n_address(SPI_ADDR_CNT_2, 0); // address don't care, use MD address
#else
    REG32_WRITE(SFC_CTL,0x00000018); /* dataen = 1, adren = 1, dmycnt = 0 */
#endif
    udelay(SFC_READ_DELAY);
    /*set ddr address*/
    REG32_WRITE(MD_FDMA_DDR_SADDR,(UINT32)(dst));//(UINT32)PHYS(dst);
    /*set spi nandflash addess*/
    REG32_WRITE(MD_FDMA_FL_SADDR,(UINT32)(spi_address - SPI_NAND_BASE_ADDR)) ;
	
    /*set dma mode  fix DMA_XFER_LEN-->BYTE_512_XFER_LEN */
    REG32_WRITE(MD_FDMA_CTRL2,((FOWAD_MAP<<FMAP_MODE_OFFSET)|(BYTE_512_XFER_LEN<<FDMA_XFER_OFFSET)) | (len & DMA_ADDR_MASK)); /*don't swap (bit28:0)*/
	
    sync();
	
	//dump_sn_reg();
    /*set dma start*/
    REG32_WRITE(MD_FDMA_CTRL1,(1<<FDMA_GO_OFFSET)|(1<<WRITE_EN_OFFSET));  // go
    while ((REG32_READ(MD_FDMA_CTRL1) & FDMA_GO_MASK) != 0);
	//dump_byte(dst,2048);
	//SN_ERR("***************after ****************\n");
   
#ifdef _SPI_NAND_CTRL
    clr_spi_n_address();
#endif
}

void sfc_read_cache_to_mem_cpu(unsigned int block_addr,UINT8 *dst,
	unsigned int offset,unsigned int len)
{
    UINT32 spi_address = 0,loop = 0,spi_data = 0,tmp_addr = 0;
    UINT32 *dst_data = (UINT32 *)dst;
	
    /*send read cache command*/
    REG32_WRITE(SFC_OPCODE,CMD_READ_CACHE&CMD_MASK);

#ifdef _SPI_NAND_CTRL
    /*set data and address enable and dummy byte = 1*/
    REG32_WRITE(SFC_CTL,(1<<DATEN_OFFSET)|(1<<ADREN_OFFSET)|((1&DMYCNT_MASK)<<DMYCNT_OFFSET)) ; /* dataen = 1, adren = 1, dmycnt = 1 */
    /*enable spi nandflash and set bit[0:15],column addrss 16bit*/
    set_spi_n_address(SPI_ADDR_CNT_2, 0);

    /*read cache data*/
    if(len < 4)
    {
        spi_address = SPI_NAND_BASE_ADDR +offset;
        *dst = REG8_READ(spi_address);
        udelay(SFC_READ_DELAY);
    }
    else
    {
        for (loop = 0; loop< (len/4); loop++) 
        {
            spi_address = SPI_NAND_BASE_ADDR + loop*4+offset;
            spi_data    = REG32_READ(spi_address);
#ifdef _LITTLE_ENDIAN
            spi_data = SWAPEND32(spi_data);
#endif  

        SN_DEBUG("%s spi_address 0x%x spi_data 0x%x loop %d \n",
            __FUNCTION__,spi_address,spi_data,loop);

            dst_data[loop] = spi_data;
        }
    }
    clr_spi_n_address();
#else
    REG32_WRITE(SFC_CTL,0x00000018); /* dataen = 1, adren = 1, dmycnt = 0 */
    udelay(SFC_DELAY);
    for (loop = 0; loop< (len/4); loop++) 
    {
        tmp_addr = ((block_addr & 0x1) << 12) + loop*4;
        spi_data= REG32_READ(SPI_NAND_BASE_ADDR + (tmp_addr << 8));
#ifdef _LITTLE_ENDIAN
        spi_data = SWAPEND32(spi_data);
#endif 
        copy_memory((UINT8 *)dst,(UINT8 *)&spi_data,sizeof(spi_data));
        dst = dst + sizeof(spi_data);
	}
#endif
}



int rtk_spi_read_ecc_page(struct mtd_info *mtd, unsigned int page, u_char *data_buf, u_char *oob_buf)
{
    int ret = 0;
    unsigned int block_addr = 0;
    struct spi_nand_chip *this=(struct spi_nand_chip *)mtd->priv;
    block_addr = page/this->block_per_page;

#ifdef CONFIG_MTD_TABLE_TEST
    if(set_read_fail == 1){
        set_read_fail = 0;
        printf("%s set page %d block %d read fail ecc error case! \n",
				__FUNCTION__,page,block_addr);
        return -1;
    }
#endif

    /*read to cache*/
    if((ret = sfc_read_page_to_cache_b(block_addr,page)) == 0)
    {
        /*read cache to memeroy*/
        sfc_read_cache_to_mem_md(this->support_cache_mode,block_addr,data_buf,this->page_size);
		//sfc_read_cache_to_mem_cpu(block_addr,data_buf,0,this->page_size);
        if(oob_buf != NULL)
        {
            sfc_read_cache_to_mem_cpu(block_addr,oob_buf,this->page_size,this->oob_size);
        }    
    }
    else
    {
        SN_ERR("%s page %d read error!\n",__FUNCTION__,page);
    }
    return ret;
}

int rtk_spi_read_oob(struct mtd_info *mtd, unsigned int page, u_char *data_buf, int len)
{
    int ret = 0;
    unsigned int block_addr = 0;
    struct spi_nand_chip *this=(struct spi_nand_chip *)mtd->priv;
    /*read to cache*/
    block_addr = page/this->block_per_page;
    if((ret = sfc_read_page_to_cache_b(block_addr,page)) == 0)
    {
        /*read cache to memeroy*/
        sfc_read_cache_to_mem_cpu(block_addr,data_buf,this->page_size,len);
    }
    return ret;
}

static int sfc_write_enable()
{
    int ret = 0,retry = 0;
    UINT8 spi_1byte_data = 0,tmp = 0;

    do{
        /*set write enable command*/
        REG32_WRITE(SFC_OPCODE,CMD_WRITE_ENABLE&CMD_MASK);
		    
        /*data address disable*/
        REG32_WRITE(SFC_CTL,0x00000000); /* dataen = 0, adren = 0, dmycnt = 0 */
        udelay(SFC_DELAY);
		    
        /*send write enable address*/
        tmp = REG8_READ(SPI_NAND_BASE_ADDR);
        udelay(SFC_DELAY);
        
        /*get write enable status  */
        sfc_get_feature(GET_STATUS_ADDR, &spi_1byte_data);
    } while (((spi_1byte_data&STA_WEL_MASK)== 0)&& (++retry < STATUS_TIMEOUT));

    if (retry == STATUS_TIMEOUT) 
    {
        SN_ERR("%s: timeout set feature: status 0x%x fail. using time out! \n", __FUNCTION__,spi_1byte_data);
        
		dump_sn_reg();
        sfc_reset();  
        //ret = -1;
    }
    return ret;
}


int sfc_set_function(unsigned char address, unsigned char data,unsigned char mask)
{
	int ret = 0;
	UINT8 spi_1byte_data = 0;
	/*write enable*/
	if((ret = sfc_write_enable()) == 0)
	{
		/*set feature*/
		sfc_set_feature(address,data);
		/*wait device ready*/
		if((ret = wait_device_ready(SFC_DELAY)) == 0)
		{
			/*get feature status*/
			sfc_get_feature(address, &spi_1byte_data);
			if (spi_1byte_data&mask == 0) 
			{
				SN_ERR("\%s: set feature: status 0x%x fail.\n", __FUNCTION__,spi_1byte_data);
				ret = - 1;
			}
		}
	}
	return ret;
}

int sfc_unlock_flash()
{
    int ret = 0;
    ret = sfc_set_function(GET_BLOCKLOCK_ADDR,0x00,ALL_LOCK);
    return ret;
}

int sfc_set_cachemode(struct mtd_info *mtd)
{
    int ret = 0;
    struct spi_nand_chip *this=(struct spi_nand_chip *)mtd->priv;
    UINT8 spi_1byte_data = 0,cache_data = 0,mask = 0;
    SN_DEBUG("\%s: set sfc_get_feature: status 0x%x .cache_bit_offset 0x%x\n", __FUNCTION__,
		this->support_cache_mode,this->cache_bit_offset);
    if((this->support_cache_mode != CACHE_1X)&&(this->cache_bit_offset != 0xF))//emst not need set
    {
        /*get cache status */
        sfc_get_feature(GET_B0_ADDR, &spi_1byte_data);
        SN_DEBUG("\%s: set sfc_get_feature: status 0x%x .\n", __FUNCTION__,spi_1byte_data);

        /*default close cache mode -->set cache mode*/
        if((spi_1byte_data&(1<<this->cache_bit_offset))== 0)
        {
            cache_data = (spi_1byte_data|(1<<this->cache_bit_offset));
            ret = sfc_set_function(GET_B0_ADDR,cache_data,cache_data);
        }
    }
    return ret;
}


static void sfc_cpu_write_mem2cache(const UINT8 *buf,
	unsigned int offset,unsigned int len)
{
    UINT32 spi_data = 0, spi_address = 0,loop = 0;
    /*send program load command*/
    REG32_WRITE(SFC_OPCODE,CMD_PROGRAM_LOAD&CMD_MASK);

#ifdef _SPI_NAND_CTRL

    /*address and data enable*/
    REG32_WRITE(SFC_CTL,(1<<DATEN_OFFSET)|(1<<ADREN_OFFSET)) ; /* dataen = 1, adren = 1, dmycnt = 0 */

    /*enable spi nandflash and set bit[0:15],column addrss 16bit*/
    set_spi_n_address(SPI_ADDR_CNT_2, 0);
    udelay(SFC_DELAY);

    /*send data to cache*/
    if(len < 4)
    {
        spi_address = SPI_NAND_BASE_ADDR +offset;
        REG8_WRITE(spi_address,*buf);
        udelay(SFC_DELAY);
    }
    else
    {
        for(loop = 0; loop< (len/4); loop++) 
        {
            spi_address = SPI_NAND_BASE_ADDR + loop*4+offset;
#ifdef _LITTLE_ENDIAN
            spi_data = SWAPEND32(*(UINT32*)buf);
#else
            spi_data = *(UINT32*)buf;
#endif  
            REG32_WRITE(spi_address,spi_data);
            buf += 4;
            udelay(SFC_DELAY);
        }
    }
    clr_spi_n_address();
#else
    REG32_WRITE(SFC_CTL,0x00000010); /* dataen = 1, adren = 0, dmycnt = 0 */
    for(loop = 0; loop< (len/4); loop++) 
    {
        spi_address = SPI_NAND_BASE_ADDR + loop*4+offset;
	    udelay(SFC_DELAY);
        REG32_WRITE(SPI_BASE_ADDR,(((spi_address + 2) & 0x1fff) << 16) + (spi_data & 0xffff)) ;
        spi_data = *(UINT32*)buf;
        buf += 4;
    }
#endif
}

static void sfc_md_write_mem2cache(unsigned int block_addr,const UINT8 *buf, UINT32 len)
{
    UINT32 spi_address  = 0;
#ifdef _SPI_NAND_CTRL
    spi_address = SPI_NAND_BASE_ADDR; 
#else
    spi_address = SPI_NAND_BASE_ADDR + ((block_addr & 0x1) << (12 + 8));	//choose plane for micron
#endif
    
    /*send program load command*/
    REG32_WRITE(SFC_OPCODE,CMD_PROGRAM_LOAD&CMD_MASK);

#ifdef _SPI_NAND_CTRL
    /*address and data enable*/
    REG32_WRITE(SFC_CTL,(1<<DATEN_OFFSET)|(1<<ADREN_OFFSET)); /* dataen = 1, adren = 1, dmycnt = 0 */
    /*enable spi nandflash and set bit[0:15]-->cloumn address page */
    set_spi_n_address(SPI_ADDR_CNT_2, 0); // address don't care, use MD address
    udelay(SFC_DELAY);

    flush_dcache_all(); //sys_dcache_flush_all();
    /*set dma address*/
    REG32_WRITE(MD_FDMA_DDR_SADDR,(UINT32)buf); //(UINT32)PHYS(buf);
    /*set spi nandflash address*/
    REG32_WRITE(MD_FDMA_FL_SADDR,(UINT32)(spi_address - SPI_NAND_BASE_ADDR));
	                 
    /*set dma mode fix DMA_XFER_LEN-->BYTE_512_XFER_LEN*/
    REG32_WRITE(MD_FDMA_CTRL2, ((FOWAD_MAP<<FMAP_MODE_OFFSET)|(BYTE_512_XFER_LEN<<FDMA_XFER_OFFSET)|(1<<FDMA_DIR_OFFSET)) 
	         | (len & DMA_ADDR_MASK));
	
    sync();
    flush_dcache_all(); //sys_dcache_flush((unsigned long)src, len);
    
    /*set dma start*/
    REG32_WRITE(MD_FDMA_CTRL1,(1<<FDMA_GO_OFFSET)|(1<<WRITE_EN_OFFSET));  // go
    while ((REG32_READ(MD_FDMA_CTRL1) & FDMA_GO_MASK) != 0);
    //dump_sn_reg();
    clr_spi_n_address();
#else
    UINT8 *wbuf;
	wbuf = (UINT8 *)PAGE_TMP_ADDR3;
    REG32_WRITE(SFC_CTL,0x00000010); /* dataen = 1, adren = 0, dmycnt = 0 */
    udelay(SFC_DELAY);

	// for spi-nand cmd + add + add + data
    wbuf[0] = ((UINT32)dst >> 8) & 0xff;
    wbuf[1] = (UINT32)dst & 0xff;
	copy_memory(&wbuf[2], buf, len);

    // flush cache before DMA
    sys_dcache_flush_all();

    REG32_WRITE(MD_FDMA_DDR_SADDR,(UINT32)PHYS(wbuf));
    REG32_WRITE(MD_FDMA_FL_SADDR,(UINT32)(SPI_NAND_BASE_ADDR - SPI_NAND_BASE_ADDR));
    REG32_WRITE(MD_FDMA_CTRL2,0x2e000000 | ((len) & 0x1ffffff)); /*don't swap (bit28:0)*/

    sync();
    REG32_WRITE(MD_FDMA_CTRL1,3);	// go
    while ((REG32_READ(MD_FDMA_CTRL1) & 0x1) != 0);

    sn_write_byte((UINT32)buf + len - 2, *((UINT8 *)buf + len - 2));
    sn_write_byte((UINT32)buf + len - 1, *((UINT8 *)buf + len - 1));
#endif
}

static int sfc_program_execute(const UINT32 block, const UINT32 page)
{
    int ret = 0;
    UINT32 spi_address = 0;
    UINT8 spi_1byte_data = 0,tmp = 0;

    /*send program execute command*/
    REG32_WRITE(SFC_OPCODE,CMD_PROGRAM_EXECUTE&CMD_MASK);
    /*set address enable*/
    REG32_WRITE(SFC_CTL,(1<<ADREN_OFFSET)) ; /* dataen = 0, adren = 1, dmycnt = 0 */

#ifdef _SPI_NAND_CTRL
    /*enable spi nand and set bit[0:24]-->16bit row address page address[0:5] block address[6:15] */
    set_spi_n_address(SPI_ADDR_CNT_3, 0);
    udelay(SFC_DELAY);
    
    /*send address*/
    spi_address = SPI_NAND_BASE_ADDR + ((block & BLOCK_MASK) << 6)+(page & PAGE_MASK); 
    tmp = REG8_READ(spi_address); //NOTICE-->need do this-->or write fail
    clr_spi_n_address();
#else
    spi_address = SPI_NAND_BASE_ADDR + ((block & BLOCK_MASK) << 6)+(page & PAGE_MASK);    //choose plane for micron
#ifdef _LITTLE_ENDIAN
    spi_address = spi_address^0x3;
#endif
    REG8_WRITE(spi_address,0x0);
#endif

    /*wait device  ready*/
    if ((ret = wait_device_ready(SFC_DELAY))== 0)
    {
        /*get program status*/
        sfc_get_feature(GET_STATUS_ADDR,&spi_1byte_data);
        if (spi_1byte_data&STA_PFAIL_MASK) 
        {
            ret = -1;
            SN_ERR("%s program fail: block %d page %d status %x\n", __FUNCTION__,block, page,spi_1byte_data);
        }
    }
    return ret;
}

int rtk_spi_write_ecc_page (struct mtd_info *mtd, unsigned int page,
			const u_char *data_buf, const  u_char *oob_buf)
{
    int ret = 0;
    unsigned int block_addr = 0;
    struct spi_nand_chip *this=(struct spi_nand_chip *)mtd->priv;
    block_addr = page/this->block_per_page;
#ifdef CONFIG_MTD_TABLE_TEST
    if(set_write_fail != 0){
        printk("%s:set page %d block %d write_fail_flag %d error mode!!\n",
				__FUNCTION__, page,block_addr,set_write_fail);
        set_write_fail = set_write_fail > 0 ?(set_write_fail -1):0;
	    return -1;
    }
#endif

    /*write enable*/
    if((ret = sfc_write_enable()) == 0)
    {
        /*write data to cache*/
		//sfc_cpu_write_mem2cache(data_buf,0,this->page_size);
        sfc_md_write_mem2cache(block_addr,data_buf,this->page_size);
        if(oob_buf != NULL)
        {
            /*write bad block tag*/
            sfc_cpu_write_mem2cache(oob_buf,this->page_size+this->bad_tag_offset,1);
            /*write magic tag*/
            oob_buf += this->magic_tag_offset;
            sfc_cpu_write_mem2cache(oob_buf,this->page_size+this->magic_tag_offset,4);
        }
        
        /*write cahce to spi nand*/
        ret = sfc_program_execute(block_addr,page);
    }
    return ret;
}

int rtk_spi_write_oob(struct mtd_info *mtd, unsigned int page,const u_char *buf,int len)
{
    int ret = 0;
    unsigned int block_addr = 0;
    struct spi_nand_chip *this=(struct spi_nand_chip *)mtd->priv;
    block_addr = page/this->block_per_page;

    /*write enable*/
    if((ret = sfc_write_enable()) == 0)
    {
        /*write data to cache*/
        sfc_cpu_write_mem2cache(buf,this->page_size+this->bad_tag_offset,1);
        buf += this->magic_tag_offset;
        sfc_cpu_write_mem2cache(buf,this->page_size+this->magic_tag_offset,4);
        /*write cahce to spi nand*/
        ret = sfc_program_execute(block_addr,page);
    }
    return ret;
}

int sfc_erase_block(unsigned int block_addr)
{
    int ret = 0;
    UINT32  spi_address = 0;
    UINT8   spi_1byte_data = 0,tmp  = 0;

    /*write enable*/
    if((ret = sfc_write_enable()) == 0)
    {
        spi_address = SPI_NAND_BASE_ADDR + ((block_addr & BLOCK_MASK) << BLOCK_OFFSET); 
#ifdef _LITTLE_ENDIAN
        spi_address = spi_address^0x3;
#endif  
        
        /*set erase command*/ 
        REG32_WRITE(SFC_OPCODE,CMD_ERASE&CMD_MASK);
        
        /*address enable*/
        REG32_WRITE(SFC_CTL,(1<<ADREN_OFFSET)); /* dataen = 0, adren = 1, dmycnt = 0 */
        udelay(SFC_DELAY);
        
        /*send erase address [0:23] */
        set_spi_n_address(SPI_ADDR_CNT_3, 0);
        tmp = REG8_READ(spi_address);
        
        /*wait device ready*/
        if ((ret = wait_device_ready(SFC_DELAY))== 0)
        {
            /*get erase status*/
            sfc_get_feature(GET_STATUS_ADDR,&spi_1byte_data);
            if (spi_1byte_data&STA_EFAIL_MASK) 
            {
                ret = -1;
                SN_ERR("%s erase block %d fail: status %x\n", __FUNCTION__,block_addr, spi_1byte_data);
            }
        }
        else
        {
            SN_ERR("%s erase block %d fail! \n",__FUNCTION__,block_addr);
            dump_sn_reg();
        }
    }
    return ret;
}

int rtk_spi_erase_block(struct mtd_info *mtd,unsigned int block)
{
    int ret = 0;
#ifdef CONFIG_MTD_TABLE_TEST
    if(set_erase_fail != 0){
        SN_ERR("%s: set block %d erase fail!!\n", __FUNCTION__, block);
        set_erase_fail = 0;
        return -1;
    }
#endif

    ret = sfc_erase_block(block);
    return ret;
}
