#include "demod_rtk_common.h"
#include "rtk_crt.c"
#include "../IFD_qc.h"
const double PI = 3.14159265359f;
static unsigned long int seed = 1;

int my_rand(void) //RAND_MAX: 32767
{
	seed *= 1103515245 + 12345;
	return (unsigned int)(seed/65536) % 32768;
}
double my_floor(double f) {
    return (double) (int) f;
}
double my_fmod(double f, double n) {
    return f - n * my_floor(f / n);
}

int mmax(double *buf, double *max, int *max_num, int size)
{
	int tmp = 0;
	double max_tmp;
	int max_num_tmp;

	if (max == NULL)
		max = &max_tmp;
	if (max_num == NULL)
		max_num = &max_num_tmp;

	*max = buf[0];
	*max_num = 0;

	for (int i=0; i<size; i++)
	{
		if (buf[i] >= *max)
		{
			*max = buf[i];
			tmp = i;
		}

		if (buf[i] == *max)
			(*max_num)++;
		else
			*max_num = 0;
	}
	return tmp;
}
int mmin(double *buf, double *min, int size)
{
	int tmp = 0;
    double min_tmp;

	if (min == NULL)
		min = &min_tmp;

	*min = buf[0];

	for (int i=0; i<size; i++)
	{
		if (buf[i] <= *min)
		{
			*min = buf[i];
			tmp = i;
		}
	}
	return tmp;
}
void normalbymax(double *buf, double max, int size)
{
	for (int i=0; i<size; i++)
		buf[i] = buf[i]/max;
}

double summ(double *buf, int start, int end)
{
	double tmp = 0;
	for (start; start <= end; start++)
		tmp += buf[start];
	return tmp;
}
double sin_t(double t) {
    // First we clamp t to the interval [0, 2*pi) 
    // because this approximation loses precision for 
    // values of t not close to 0. We do this by 
    // taking fmod(t, 2*pi) because sin is a periodic
    // function with period 2*pi.
    t = my_fmod(t, 2.0f * PI);

    // Next we clamp to [-pi, pi] to get our t as
    // close to 0 as possible. We "reflect" any values 
    // greater than pi by subtracting them from pi. This 
    // works because sin is an odd function and so 
    // sin(-t) = -sin(t), and the particular shape of sin
    // combined with the choice of pi as the endpoint
    // takes care of the negative.
    if (t >= PI) {
        t = PI - t;
    }

    // You can precompute these if you want, but
    // the compiler will probably optimize them out.
    // These are the reciprocals of odd factorials.
    // (1/n! for odd n)
    const long double c0 = 1.0f;
    const double c1 = c0 / (2.0f * 3.0f);
    const double c2 = c1 / (4.0f * 5.0f);
    const double c3 = c2 / (6.0f * 7.0f);
    const double c4 = c3 / (8.0f * 9.0f);
    const double c5 = c4 / (10.0f * 11.0f);
    const double c6 = c5 / (12.0f * 13.0f);
    const double c7 = c6 / (14.0f * 15.0f);
    const double c8 = c7 / (16.0f * 17.0f);

    // Increasing odd powers of t.
    const double t3  = t * t * t;
    const double t5  = t3 * t * t;
    const double t7  = t5 * t * t;
    const double t9  = t7 * t * t;
    const double t11 = t9 * t * t;
    const double t13 = t9 * t * t;
    const double t15 = t9 * t * t;
    const double t17 = t9 * t * t;

    return c0 * t - c1 * t3 + c2 * t5 - c3 * t7 + c4 * t9 - c5 * t11 + c6 * t13 - c7 * t15 + c8 * t17;
	//return c0 * t - c1 * t3 + c2 * t5 - c3 * t7;

}

double cos_t(double t) {
	return sin_t(t+PI/2);
}

void DC_cancel(complex_t *input, int size)
{
	// it's time domain
	double sum = 0;
	double mean = 0;
	for (int i=0; i<size; i++)
		sum += input[i].real;
	mean = sum/(double)size;
	for (int i=0; i<size; i++)
		input[i].real -= mean;
}

void hamming_window(complex_t *buf, int size)
{
	//it's time domain 	
	const double c0 = 0.355768f;
	const double c1 = 0.487396f;
	const double c2 = 0.144232f;
	const double c3 = 0.012604f;
	for (int i=0; i<size; i++)
	{
		double tmp;
		tmp = c0 - c1*cos_t(2*PI*i/size) + c2*cos_t(4*PI*i/size); - c3*cos_t(6*PI*i/size);

		//IFD_QC_MSG_INFO("---hamming_window--->{t0:%d,t1:%d,t2:%d,t3:%d, tmp:%d}---\n", (int)c0,(int)c1,(int)c2,(int)c3,(int)tmp);
		buf[i].real *= tmp;
	}
}
double mylog10(double input)
{
	//unsupport range 0~1
	double fraction;
	double log2x;
	int value = 2;
	int pow = 0; //power of 2
	while (value <= input)
	{    
		value*=2;
		pow++;
	}
	 fraction = input / (double)(1<<pow);

    //logx = (log2)log2x~0.6931472log2x
	//log2x~-0.344845x2 +2.024658x-1.674873 [1~2]
	const double c0 = -1.674873f;
	const double c1 = 2.024658f;
	const double c2 = -0.344845f;
	const double mylog2 = 0.30102999566398f;
	
	log2x = c2*(fraction*fraction)+c1*fraction+c0;
	
	return mylog2*(log2x + pow);
	
}

void logproc(double *buf, int size)
{
	for(int i=0; i<size; i++)
	{
		buf[i] = 10*mylog10(buf[i]);
		IFD_QC_MSG_INFO("--logprocess:%d---\n", (int)buf[i]);

	}
}
void _fft(complex_t *buf, complex_t *out, int n, int step)
{
	if (step < n) {
		_fft(out, buf, n, step * 2);
		_fft(out + step, buf + step, n, step * 2);
		for (int i = 0; i < n; i += 2 * step) {
			complex_t t;
			double tmp[4];
			tmp[0] = cos_t(PI*i/n) * out[i + step].real;
			tmp[1] = sin_t(PI*i/n) * out[i + step].imag;
			tmp[2] = cos_t(PI*i/n) * out[i + step].imag;
			tmp[3] = sin_t(PI*i/n) * out[i + step].real * (-1);
			t.real = tmp[0] + tmp[1];
			t.imag = tmp[2] + tmp[3];
			//cplx t = cexp(-I * PI * i / n) * out[i + step];
			buf[i / 2].real = out[i].real + t.real;
			buf[i / 2].imag = out[i].imag + t.imag;

			buf[(i + n)/2].real = out[i].real - t.real;
			buf[(i + n)/2].imag = out[i].imag - t.imag;
		}
	}
}
 
void fft(complex_t *buf, int n)
{
	complex_t out[4096];
	for (int i = 0; i < n; i++)
	{
		out[i].real = buf[i].real;
		out[i].imag = buf[i].imag;
	}
	_fft(buf, out, n, 1);
}
//Cooley-TukeyFFT

void _colleyFFT(complex_t *x, int N)
{
	if (N <= 1)
		return;
	//split even and odd
	complex_t even[N/2], odd[N/2];
	for(int i=0; i<N/2; i++)
	{
		even[i] = x[i*2];
		odd[i] = x[i*2+1];
	} 
    //split on tasks
	_colleyFFT(even, N/2);
	_colleyFFT(odd, N/2);

	//calculate DFT
	for(int k=0;k<N/2;k++)
	{
		complex_t t;
		double tmp[4];
		tmp[0] = odd[k].real * cos_t(2*PI*k/N);
		tmp[1] = odd[k].imag * sin_t(2*PI*k/N);
		tmp[2] = odd[k].imag * cos_t(2*PI*k/N);
		tmp[3] = odd[k].real * sin_t(2*PI*k/N) * (-1);
		t.real = tmp[0] + tmp[1];
		t.imag = tmp[2] + tmp[3];

		x[k].real = even[k].real + t.real;
		x[k].imag = even[k].imag + t.imag;
		x[N/2+k].real = even[k].real - t.real;
		x[N/2+k].imag = even[k].imag - t.imag;
	}

}
void colleyFFT(complex_t *x_in, complex_t *x_out, int N) 
{
	for (int i = 0; i < N; i++) {
		x_out[i].real = x_in[i].real;
		x_out[i].imag = x_in[i].imag;
	}
	// Start recursion
	_colleyFFT(x_out, N);
}


void power(complex_t *input, double *output, int size)
{
	for(int i=0; i<size; i++)
		output[i] = (input[i].real) * (input[i].real) + (input[i].imag) * (input[i].imag);
}

void GPIOout(int SNR)
{
	if (SNR < 0)
		SNR = 0;
	unsigned int bits[8];
	bits[6] = (SNR & 0x40) ? 1:0;
	bits[5] = (SNR & 0x20) ? 1:0;
	bits[4] = (SNR & 0x10) ? 1:0;
	bits[3] = (SNR & 0x08) ? 1:0;
	bits[2] = (SNR & 0x04) ? 1:0;
	bits[1] = (SNR & 0x02) ? 1:0;
	bits[0] = (SNR & 0x01) ? 1:0;

	IFD_QC_MSG_INFO("GPIO:{%d,%d,%d,%d,%d,%d,%d}\n",bits[6],bits[5],bits[4],bits[3],bits[2],bits[1],bits[0])
	//7bits set GPIO on board
	rtd_part_outl(0x1800087c,23,20,0xf); //bit 6:PC_A8_GP_100 (AB24)
	rtd_part_outl(0x1800088c,23,20,0xf); //bit 5:PC_A7_GP_116 (AD25)
	rtd_part_outl(0x18000894,23,20,0xf); //bit 4:PC_A3_GP_124 (AA25)
	rtd_part_outl(0x18000894,7,4,0xf); //bit 3:PC_A2_GP_126 (AA26)
	rtd_part_outl(0x180008a0,23,20,0xf); //bit 2:PC_D2_GP_136 (U26)
	rtd_part_outl(0x180008a0,15,12,0xf); //bit 1:PC_CD1_N_GP_137 (AC17)
	rtd_part_outl(0x180008a0,7,4,0xf); //bit 0:PC_D3_GP_138 (AB18)
	rtd_outl(0xb801bd0c, 0x1); //(write enable)
    rtd_outl(0xb801bd10, 0x1); //(write enable)
	rtd_outl(0xb801bd24, 0x1); //(write enable)
	rtd_outl(0xb801bd28, 0x1); //(write enable)
	
	//bit6,5 IO set
	//bit 6 IO_set:0xb801bd0c[8]
	//bit 5 IO_set:0xb801bd0c[24]
	rtd_outl(0xb801bd0c, 0x01000101); //(write enable)
	//bit 4,3,2,1,0 IO set
	//bit 4 IO_set:0xb801bd10[1]
	//bit 3 IO_set:0xb801bd10[3]
	//bit 2 IO_set:0xb801bd10[13]
	//bit 1 IO_set:0xb801bd10[14]
	//bit 0 IO_set:0xb801bd10[15]
	rtd_outl(0xb801bd10, 0x0000e00b); //(write enable)


	//bit6,5 GPO set
	//bit 6 GPO_set:0xb801bd24[8]
	//bit 5 GPO_set:0xb801bd24[24]
	bits[7] = (bits[6] << 8) | (bits[5] << 24) | 1;
	rtd_outl(0xb801bd24, bits[7]); //(write enable)
	//bit 4,3,2,1,0 GPO set
	//bit 4 GPO_set:0xb801bd28[1]
	//bit 3 GPO_set:0xb801bd28[3]
	//bit 2 GPO_set:0xb801bd28[13]
	//bit 1 GPO_set:0xb801bd28[14]
	//bit 0 GPO_set:0xb801bd28[15]
	bits[7] = (bits[4] << 1) | (bits[3] << 3) | (bits[2] << 13) | (bits[1] << 14) | (bits[0] << 15) | 1;
	rtd_outl(0xb801bd28, bits[7]); //(write enable)

}
perform_t SNRperform(double *buf, int size)
{
	double maxvalue;
	int max_num;
	int max_idx = mmax(buf, &maxvalue, &max_num, size);
	double noise_pow;
	double signal_pow;
	double SNR;
	perform_t result;
	//SNDR


    signal_pow = summ(buf, max_idx-3, max_idx+3);
	noise_pow = summ(buf, 0, max_idx-4) + summ(buf, max_idx+4, size);
	SNR = 10*(mylog10(signal_pow/noise_pow));
	IFD_QC_MSG_INFO("signal_pow:%d\n", (int)signal_pow);
	IFD_QC_MSG_INFO("noise_pow:%d\n", (int)noise_pow);
	IFD_QC_MSG_INFO("max_idx:%d, max_num:%d\n", max_idx, max_num);
	IFD_QC_MSG_INFO("SNR:%d\n", (int)SNR);	

	result.SNDR = 10*(mylog10(signal_pow/noise_pow));	
	result.ENOB_SNDR = (result.SNDR-1.76)/6.02;

	return result;
}

int I2Ccontest(void)
{
	DEMOD ATSC3;
	int I2Ccheck;
    ExtAtsc3DemodInitialization();
    ATSC3.I2CAddr = 0x38; //7 bits address
    demod_i2c_init(&ATSC3);
    IFD_QC_MSG_INFO("I2C_2 init success\n");
	REALTEK_R_ATSC3_System_Init(NULL);
	IFD_QC_MSG_INFO("ATSC3 System Initied\n");
	REALTEK_R_ATSC3_Register(NULL);
	IFD_QC_MSG_INFO("ATSC3 register set\n");
	
	//check I2C connect 
	rtd_4bytes_i2c_wr(NULL, 0xB816F630, 0x2); //adc data_order
	rtd_4bytes_i2c_rd(NULL, 0xB816F630, &I2Ccheck);
	if (I2Ccheck != 0x2)
		return FUNCTION_ERROR;
	else
	{
		rtd_4bytes_i2c_wr(NULL, 0xB816F630, 0x0); //default
		return FUNCTION_SUCCESS;             
	}
}
int ADCdump(complex_t *SRAMdata_comp, double *SRAMdata)
{
	//ADC dump to SRAM
	//set adc debug start
	rtd_4bytes_i2c_mask(NULL, 0xB816F630, 0xFFFFFFFe, 0x00000001);
	tv_osal_usleep(1); //wait for a moment
	//start to read adc data_order
	rtd_4bytes_i2c_mask(NULL, 0xB816F630, 0xfffffffd, 0x00000002);
	//dump
	for (int i=0, j=0; i<8192; i++)
	{
		if (i%2==0) //even
		{
			unsigned int data;
			unsigned int writebytes = 0x3 + (i<<4); 	
			int tmp;
			rtd_4bytes_i2c_wr(NULL, 0xB816F630, writebytes);
			//rtd_4bytes_i2c_mask(NULL, 0xB816F630, 0xfffe000f, i<<4);
			rtd_4bytes_i2c_rd(NULL, 0xB816F634, &data);
			//IFD_QC_MSG_INFO("_addr:%d, data:%x\n",i, data[j]);
			if (((data & 0x800) >> 11) == 1) 
			{
				//2's complement
				data ^= 0xfff;
				data += 1;

				tmp = data;
				tmp = -tmp;
			
			}
			else
				tmp = data;
			if (SRAMdata_comp != NULL)
				SRAMdata_comp[j].real = tmp;
			
			if (SRAMdata != NULL)
				SRAMdata[j] = tmp;
			j++;
		}
	}
	rtd_4bytes_i2c_wr(NULL, 0xB816F630, 0x0); //default
	return FUNCTION_SUCCESS;
}

int ADCdumpIDNL(void)
{
	int code = 4096;
	int max_idx = 0;
	int min_idx = 4096;
	double SRAMdata[4096] = {0};
	double ADCdistcode[4096] = {0};
	double ADCdistcode_cdf[4096] = {0};
	double ADCdistcode_nomcdf[4096] = {0};
	double LSB;
	double dnl[4096] = {0};
	double inl[4096] = {0};
	IFD_QC_MSG_INFO("--------ADCdumpIDNL--------\n");
	for (int i=0; i<250; i++)
	{
		IFD_QC_MSG_INFO("--------SRAMdump:%d--------\n", i);
		ADCdump(NULL, SRAMdata);
		double tmp; 
		mmax(SRAMdata, &tmp, NULL, 4096);
		tmp > max_idx ? (max_idx = tmp) : max_idx;
		mmin(SRAMdata, &tmp, 4096);
		tmp < min_idx ? (min_idx = tmp) : min_idx;
        
		for (int j=0; j<code; j++)
		{
			int idx = (int)SRAMdata[j] + code/2; //shift
			ADCdistcode[idx]++;

		}
		tv_osal_usleep(1.3); //randomize for continuous ADC dump
	}
	
	max_idx += code/2; //shift
	min_idx += code/2; //shift  
	
	for (int i=min_idx; i<= max_idx; i++)
	{
		ADCdistcode_cdf[i] = summ(ADCdistcode, min_idx, i);
	}

	for (int i=0; i<code; i++)
	{
		unsigned long int total = summ(ADCdistcode, 0, 4095);
		double x = PI*ADCdistcode_cdf[i]/total + PI;
		ADCdistcode_nomcdf[i] = cos_t(x);
	}

	LSB = (ADCdistcode_nomcdf[max_idx-5] - ADCdistcode_nomcdf[min_idx+5])/(max_idx-min_idx-10);
	
	for (int i=min_idx+6; i<= max_idx-5; i++)
	{
		dnl[i] = (ADCdistcode_nomcdf[i]-ADCdistcode_nomcdf[i-1])/LSB-1;
		inl[i] = (ADCdistcode_nomcdf[i]-(ADCdistcode_nomcdf[min_idx] + LSB*(i-min_idx)))/LSB;
	}
	for (int i=0; i<4096; i++)
	{
		IFD_QC_MSG_INFO("ADCdistcode:%d\n", (int)ADCdistcode[i]);
		IFD_QC_MSG_INFO("ADCdistcode_cdf:%ld\n", (int)ADCdistcode_cdf[i]);
		IFD_QC_MSG_INFO("ADCdistcode_nomcdf:%d\n",(int)(ADCdistcode_nomcdf[i]*10));
		IFD_QC_MSG_INFO("dnl:%d\n",(int)(dnl[i]*100));
		IFD_QC_MSG_INFO("inl:%d\n",(int)(inl[i]*100));
	}
	IFD_QC_MSG_INFO("--------maxidx:%d--------\n", max_idx);
	IFD_QC_MSG_INFO("--------minidx:%d--------\n", min_idx);
	return FUNCTION_SUCCESS;
}
int ADCdumpSNR(void)
{
	complex_t SRAMdata[4096] = {0};
	complex_t fftout[4096];
	double fft_power[4096];
	
	ADCdump(SRAMdata, NULL);
	//for (int i=0; i<4096; i++)
	//	IFD_QC_MSG_INFO("SRAMdata:%d\n",(int)SRAMdata[i].real);
	
	IFD_QC_MSG_INFO("--------DC_cancel--------\n");
	DC_cancel(SRAMdata, 4096);
	IFD_QC_MSG_INFO("--------DC_cancel finished--------\n");
	
	IFD_QC_MSG_INFO("--------hamming_window--------\n");
	hamming_window(SRAMdata, 4096);
	IFD_QC_MSG_INFO("--------hamming_window finished--------\n");
	
	IFD_QC_MSG_INFO("--------4096FFT--------\n");
	colleyFFT(SRAMdata, fftout, 4096);
	power(fftout, fft_power, 4096);
	IFD_QC_MSG_INFO("--------4096FFT finished--------\n");
	
	perform_t result;

	result = SNRperform(fft_power,2048);
	GPIOout(result.SNDR);
	return FUNCTION_SUCCESS;
}


unsigned int IsAtsc3McmPackage()
{

	//check OTP data for MCM package
	unsigned int Ans;

	Ans = (rtd_inl(0xB8038184) >> 17) & 0x1;

	return Ans;

}

int ExtAtsc3DemodInitialization(void)
{

	//init DEMOD ATS3
    //MCM_RL6681 Reset pinmux and release
    //rtd_outl(0xb8000860, 0xf0f00000);							//<bit15:12>Pinmux GPIO_162 set to 0: atsc_demod_rst_n (MCM_IC default value)
    //rtd_outl(0xb8000120, 0x00200000);							//<bit21>RSTN_ATSC_demod = 0x0(Reset)
    //rtd_outl(0xb8000120, 0x00200001);							//<bit21>RSTN_ATSC_demod = 0x1(Release Reset)
    ExtAtsc3PowOffOn(0);
	TvFrontendPowerControl(0); //pull reset
	DtvIAdcIPgaPowerSetting(0);//turn off I_ADC, IPGA power
	tv_osal_msleep(10);
	ExtAtsc3PowOffOn(1);
	tv_osal_msleep(10);
	return FUNCTION_SUCCESS;
}


void ExtAtsc3PowOffOn(unsigned char on)
{
	//Check IFADC CLK turn on
	if (on == 1){
		CRT_CLK_OnOff(DTVExtAtsc3, CLK_ON, NULL);
		// Merlin6 DEMOD_IF_AGC Pinmux set to GPI
		rtd_maskl(0xb8000844, 0x0FFFFFFF, 0xF0000000);       //<bit31:28>GPIO_61, Set pinmux  to MIS_GPIO_bit61
		rtd_outl(0xb801bd04, 0x80000000);                             //<bit31>GPIO_61=0x0: Configured as input pin, <bit0>write bit = 0x0
	}
	else
		CRT_CLK_OnOff(DTVExtAtsc3, CLK_OFF, NULL);

	IFD_QC_MSG_INFO("ExtAtsc3PowOffOn!! on = %u\n", on);
}

int TvFrontendPowerControl(unsigned char On)
{
	if (On) {
		CRT_CLK_OnOff(TVFRONTEND, CLK_ON, NULL);
		IFD_QC_MSG_INFO("demod is power on.\n");
	} else {
		CRT_CLK_OnOff(TVALL, CLK_OFF, NULL);
		IFD_QC_MSG_INFO("demod is power down.\n");
	}

	return FUNCTION_SUCCESS;
}

int DtvIAdcIPgaPowerSetting(unsigned char On)
{
	CRT_CLK_OnOff(TVADC, CLK_ON, NULL);
	if (On) {
		rtd_maskl(TV_SB1_ANA_IQADC_CTRL3_reg, ~TV_SB1_ANA_IQADC_CTRL3_reg_pow_i_adc_mask,  TV_SB1_ANA_IQADC_CTRL3_reg_pow_i_adc(1));
		rtd_maskl(TV_SB1_ANA_IQADC_CTRL3_reg, ~TV_SB1_ANA_IQADC_CTRL3_reg_pow_i_pga_mask,  TV_SB1_ANA_IQADC_CTRL3_reg_pow_i_pga(1));
		IFD_QC_MSG_INFO("DTV I_ADC & I_PGA Power On \n");
	} else {
		rtd_maskl(TV_SB1_ANA_IQADC_CTRL3_reg, ~TV_SB1_ANA_IQADC_CTRL3_reg_pow_i_adc_mask,  TV_SB1_ANA_IQADC_CTRL3_reg_pow_i_adc(0));
		rtd_maskl(TV_SB1_ANA_IQADC_CTRL3_reg, ~TV_SB1_ANA_IQADC_CTRL3_reg_pow_i_pga_mask,  TV_SB1_ANA_IQADC_CTRL3_reg_pow_i_pga(0));
		IFD_QC_MSG_INFO("DTV I_ADC & I_PGA Power Off \n");
	}

	return FUNCTION_SUCCESS;
}

int REALTEK_R_ATSC3_System_Init(DEMOD* pDemod)
{
	int ret = 0;
//--------CRT All turn on----------------//
	//ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a0, 0x000001ff);
	//ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a4, 0x000001ff);
//---------------------------------------//

//Turn on MISC 												//for GPIO
//<bit8>RSTN_MISC = 0x1(Release reset)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a0, 0x00000101);
//<bit8>CLKEN_MISC = 0x1(Enable)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a4, 0x00000101);


//DTV Demod Sel
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb80004b0, 0xffffffe8, 0x00000010); 				//<bit4>MAIN_POR_EN=0x1, <bit2>Dtv_demod_atb_clk_sel=0(Ldpc_Clk=370MHz), <bit1:0>dtv_demod_sel=00(atsc3.0)

//IfdMbiasPowAndAdcPow
// 1. CRT_CLK_OnOff(TVADC, CLK_ON, NULL);					//Turn on CLKEN_IFADC
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a0, 0x00000081);							//<bit7>RSTN_IFADC, set to 1(Release)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a4, 0x00000081);							//<bit7>CLKEN_IFADC, set to 1
	tv_osal_usleep(50);									//delay 50uS

// 2. MBIAS and PLL_LDO Power turn on
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8033100, 0xfffeffff, 0x00010000);  			//<bit16>REG_POW_MBIAS turn on
	tv_osal_usleep(50);									//delay 50uS
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb80004d4, 0xffffbbff, 0x00004400);  			//<bit14>PLL_POW_LDO11V, <bit10>PLL_POW_LDO turn on
	tv_osal_usleep(50);									//delay 50uS

// 3. turn Off ADC/PGA power
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8033104, 0xfeffffff, 0x00000000);				//IQADC input cm power off <bit24> = 0
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8033124, 0xfffffefe, 0x00000000);				//IPGA/IADC Power Off
	tv_osal_usleep(50);									//delay 50uS

//----------------------------------------------------------//
//CRT_CLK_OnOff(TVALL, CLK_OFF, NULL);        				//Turn off TVALL
//DTV Demod IP reset
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a0, 0x0000005a);							//Demod IP Reset<bit6, bit4, bit3, bit1> set to 0(active low)
	tv_osal_msleep(1);									//driver delay 1mS

//DTV Demod Wrapper reset
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a0, 0x00000124);							//Demod IP Reset<bit8, bit5, bit2> set to 0(active low)
	tv_osal_msleep(1);									//driver delay 1mS

//DTV Demod turn off
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a4, 0x000017e);							//Demod CLK<bit8, bit6, bit5, bit4, bit3, bit2, bit1> set to 0
	tv_osal_usleep(10);									//driver delay 10uS

//----------------------------------------------------------//
//#include "RL6681_TVFrontEnd_1_PLL_default.tbl"				//Load PLL default setting
	ret |= REALTEK_R_ATSC3_PLL_Default(NULL) ;

//--------PLL27X_IFADC-------------------
//RL6681_VCO=648MHz, ADC_CLK=144M, SYS_CLK=518.4MHz
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb80004b0, 0xffffffe8, 0x00000010);				//<bit4>MAIN_POR_EN = 0x1, <bit2>Dtv_demod_atb_clk_sel=0(Ldpc_Clk=370MHz), <bit1:0>dtv_demod_sel=00(atsc3.0)
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb80004dc, 0xfffffffe, 0x00000000);				//Set MUX to DTV Mode<bit0>=0x0
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8000500, 0xffffffef, 0x00000010);				//PLL27x_IFADC DTV CLK<bit4>=0x1
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8000504, 0xfc80fff8, 0x00160004);				//VCO = M<6:0>+2 = 22+2, N:00=1, 27 /1 *24=648MHz, PLL27X_IFADC_M<bit22:16> = 0x16(22), PLL27X_IFADC_N<bit25:24>=0x0(1), PLL27X_IFADC PUMP current<bit2:0>=0x4(0x100:25uF)
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb80004c4, 0xffffff88, 0x00000037);				//System_CLK : PLLDIF_DIVDIG_SEL<bit6:4> = 0x3(/1.25), sys_clk = 648 / 1.25 = 518.4 MHz
	//ADC_CLK : PLLDIF_DIVADC_SEL<2:0> = 0x7(/4.5), Adc_clk = 648 / 4.5 =144MHz
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb80004b8, 0xFFFC03FB, 0x00021400);				//<bit17>ATVADCDIV_RSTB=0x1(normal), <bit13>DIF216M_EN=0x0, <bit12>DTVDIG_EN=0x1, <bit11>LDPC_DIV=0x0(div1.75), <bit10>LDPC_EN=0x1
	tv_osal_msleep(1);//delay 1ms

//*****RL6681_VCO=432MHz, ADC_CLK=144M, SYS_CLK=246.857MHz****//
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8000508, 0xfffffff8, 0x00000000);  			//<2:1>PLL27X_IFADC_VCORSTB, PLL27X_IFADC_RSTB reset = 0(Reset); <0>PLL27X_IFADC_POW turn off
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8000508, 0xfffffffe, 0x00000001);  			//<0>PLL27X_IFADC_POW turn On
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8000508, 0xfffffff9, 0x00000006);  			//<2:1>PLL27X_IFADC_VCORSTB, PLL27X_IFADC_RSTB reset = 1(release)
	tv_osal_msleep(1);//delay 1ms

	tv_osal_usleep(50);									//driver delay 50uS
//#include "RL6681_TVFrontEnd_2_ADC_IFD.tbl"					//Load ADC default setting
	ret |= REALTEK_R_ATSC3_ADC_Default(NULL);

	ret |= rtd_4bytes_i2c_mask(pDemod, 0xB80004b8, 0xfffffdff, 0x00000000);  			//<bit9>IFADC ATV clk enable turn off
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xB8000500, 0xffffffef, 0x00000010);  			//<bit4>PLL27X_IFADC_DTV_CKO_EN setup
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xB80004b8, 0xfffffdff, 0x00000200);  			//<bit9>IFADC ATV clk enable turn on
	tv_osal_usleep(50);									//driver delay 50uS

//----------------------------------------------------------//
//DtvDemodPowerControl(1)
//CRT_CLK_OnOff(TVFRONTEND, CLK_ON, NULL);					//Turn On TVFrontend(memory bank and dtv_forntend)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a0, 0x00000061);							//<bit6>RSTN_dtv_frontend, <bit5>RSTN_dtv_demod_mb, <bit0>write bit = 0x1(release)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a4, 0x00000061);							//<bit6>CLKEN_dtv_frontend, <bit5>Clken_dtv_demod_mb, <bit0>write bit = 0x1(CLKEN)
	tv_osal_usleep(10);									//driver delay 10uS

//----------- Set DTV Frontend path --------------//
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033800, 0x00010010);							//<bit18:16>Dtv_frontend_mode=0x1, <bit8>dtv_idata_sel=0x0(from dn flt), <bit4>Dtv_dnflt_en=0x1(DN6)
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8033804, 0xfffbffff, 0x00040000);				//input fifo clr, dtv_input_fifo_clr<bit18> FIFO waddr & raddr clear
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8033804, 0xfffbffff, 0x00000000);				//input fifo clr release, dtv_input_fifo_clr<bit18>, FIFO waddr & raddr clear

//Set if_agc
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb800081c, 0xFF0FFFFF, 0x00400000); 				//<bit23:20>GPIO_29, Set pinmux DEMOD_IF_AGC

//---------------Start set RL6681 down sample filter (dn6)-----------//
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033900, 0x1ffe1ffc);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033904, 0x00011fff);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033908, 0x00060003);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb803390c, 0x00070007);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033910, 0x00020006);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033914, 0x1ff81ffd);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033918, 0x1ff31ff4);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb803391c, 0x1ffb1ff5);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033920, 0x000d0004);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033924, 0x00170015);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033928, 0x000A0014);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb803392c, 0x1fed1ffc);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033930, 0x1fda1fe0);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033934, 0x1fec1fde);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033938, 0x001a0002);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb803393c, 0x003a002f);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033940, 0x00230037);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033944, 0x1fde0003);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033948, 0x1faa1fbd);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb803394c, 0x1fc51fac);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033950, 0x002a1ff3);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033954, 0x007e005d);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033958, 0x005f0080);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb803395c, 0x1fce001f);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033960, 0x1f4a1f80);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033964, 0x1f661f3e);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033968, 0x00381fbf);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb803396c, 0x011200b4);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033970, 0x01040133);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033974, 0x1fc30082);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033978, 0x1e351eed);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb803397c, 0x1dff1dd6);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033980, 0x00401ecd);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033984, 0x0480023b);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033988, 0x08ad06c2);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb803398c, 0x0a6909f5);
//-----------------End of Set DownSample filter-------------//

//-----------------Enable DTV Demod IP-------------//
	tv_osal_usleep(50);									//driver delay 50uS
//DTV_DEMOD CLK / RESET flow : DTV Demod misc CRT turn on first then Demod IP CRT
//Enable MISC												//Turn On MISC
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a0, 0x00000111);							//<bit8>RSTN_MISC, <bit4>RSTN_DTV_DEMOD, <bit0>write bit = 0x1(release)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a4, 0x00000111);							//<bit8>CLKEN_MISC, <bit4>CLKEN_DTV_DEMOD <bit0>write bit = 0x1(CLKEN)
	tv_osal_usleep(10);									//delay 10uS

//DTV_ATB_Demod CLK/Reset Enable,  Demod wrapper frist then Demod IP
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a0, 0x00000005);							//<bit2>RSTN_atb_demod_wrap, <bit0>write bit set to 1(release)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a4, 0x00000005);							//<bit2>clken_atb_demod_wrap, <bit0>write bit set to 1
	tv_osal_usleep(10);									//delay 10uS
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a0, 0x00000009);							//<bit3>RSTN_atb_demod, <bit0>write bit set to 1(release)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004a4, 0x00000009);							//<bit3>clken_atb_demod, <bit0>write bit set to 1
	tv_osal_usleep(10);									//delay 10uS

	tv_osal_usleep(50);									//delay 50uS

//--------------- Need to Check ----------------//
//Select ATB DTV Demod (0xb816f068 :0x1=DVBT(bit0), 0x2=DVBS(bit1), 0x4=DVBC(bit2), 0x8=DVBC2(bit3))
//rtd_outl(0xb816f068, 0x00000001);							//DTV Dvbt enable<bit0> = 0x1(1)

//modify atb_wrapper_t2dit_addr3 value
//rtd_maskl(0xb816F028, 0xEFFFFFFF, 0x00000000);   			//atb_wrapper_cache off (DVBT2 only)
//rtd_maskl(0xb816F02C, 0xFF000FFF, 0x00e83000); 			//Modify atb_wrapper_t2dit_addr3<23:12> = 0xe83

//----------------------------------------------------------//
//#include "Mac7P_TVFrontEnd_DTV_TestMode_DebugPin.tbl.tbl"
	rtd_4bytes_i2c_mask(NULL, 0xB816F004, 0xBFFFFFFF, 0x40000000);   //atb_wrapper_reset_flow_en
	rtd_4bytes_i2c_wr(NULL, 0xB816F068, 0x00000010);  //DTV_DEMOD_SEL: atsc3_en
	return ret;
}

int REALTEK_R_ATSC3_PLL_Default(DEMOD* pDemod)
{

	int ret = FUNCTION_SUCCESS;
//PLL
//RL6681 register default setup
//rtd_outl(0xb80004b0,0x00000010);				//<bit4>MAIN_POR_EN = 0x1

	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004c4, 0x00000000);				//DTV PLLDIF PLLDIF_DIVDIG_SEL = 000(2), PLLDIF_DIVADC_SEL = 00(divide by 8)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004b8, 0x00003600);
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004d4, 0x80086500);				//<bit31>Pll_xtal_pow=0x1, <bit20:19>PLL_LDO_XTAL_SEL=0x1(1.76V), <bit14>PLL_POW_LDO11V=0x1, <bit13:11>PLL_LDO11V_SEL=0x4,
	//<bit10>PLL_POW_LDO=0x1, <bit8:9>PLL_LDO_SEL=0x1(0.952V)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004d8, 0x00000000);				//Default setting, <bit15:8>PLL_RESERVE=0x0(0=Select ADC CLK From PLL27X_IFADC).
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb80004dc, 0x40000001);				//Default ATV mode, <bit30:29>PLL27X_IFADC_DUTY=0x2,<bit0>PLL_CK_IFADC_MUX=0x0(ATV mode)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8000500, 0x0000041c);				//<bit11:10>PLL27X_IFADC_CK108M_DIV=0x1(VCO=432M),<bit4>PLL27X_IFADC_DTV_CKO_EN=0x1(enable),<bit3:2>PLL27X_IFADC_ATV_CKO_SEL=0x3(135M or 162M (depend on VCO and REG_PLL27x_IFADC_DIV))
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8000504, 0x000e0224);				//PLL27X_IFADC VCO M,N, M<10>+2�P=14(VCO=432M)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8000508, 0x00000000);				//PLL27X_IFADC Poweroff and VCORSTB reset
	tv_osal_usleep(10);						//delay 10uS

//PLL27X
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb80004dc, 0xfffffffe, 0x00000000);	//Set MUX to DTV Mode<bit0>=0x0

//PLL27X_IFADC
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8000500, 0xfffffff3, 0x0000000C);	//PPLL27X_IFADC_DTV_CKO_SEL = 00(/3), PLL27X_IFADC_ATV_CKO_SEL = 11 (135M or 162M (depend on VCO and REG_PLL27x_IFADC_DIV).
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8000508, 0xfffffff8, 0x00000007);	//IFADC, RSTB and VCORSTB Power on
	return ret;
}

int REALTEK_R_ATSC3_ADC_Default(DEMOD* pDemod)
{
	int ret = FUNCTION_SUCCESS;
//RL6681 ADC register default
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033100, 0x01110500);					//<bit25:24>REG_IF_BY_VCM_SEL=0x1, <bit21:20>REG_IPGA_MBIAS_ISEL=0x1, <bit16=1>IFD MBIAS power on,<bit11:10>REG_I_VCM_SEL=0x1(0.632V),<bit9:8>ADC2X_BG_SEL= 0x1(1.216V)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033104, 0x00000300);					//<bit24>IQADC input cm power off, <bit9:8>REG_IPGA_BW=0x3(37.5M)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033118, 0x00000000);					//<bit8>REG_I_CKOUT_EDGE=0x1=0(IADC CKOUT polarity=negative edge)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033124, 0x00000000);					//PGA/ADC Power Off
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033128, 0x00000002);					//Default setting(10=20uA), IQ PGA CURRENT  select
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb803312C, 0x00000000);					//default setting, <bit3:0>REG_IPGA_D=0x0(IPGA gain=0dB)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033130, 0x08000000);					//default setting, <bit27:26>REG_IADC_I_VREF_SEL=0x2(1.5X)
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8033134, 0x11000091);					//default setting, <bit25:24>REG_IADC_VREF_SEL=0x1(1.4V), <bit8:7>REG_IADC_LDO0_SEL=0x1(1.3V), <bit2:0>REG_IADC_LDO_SEL=0x1(0.961V)

//Set IADC/IPGA power on and IPGA input
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8033124, 0xfffffefe, 0x00000101);		//I_PGA<bit8> power on, <bit0>I_ADC power on
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8033128, 0xfff8ffff, 0x00020000);		//<bit18:16>REG_IPGA_INPUT_SEL=0x2(IFD input)

//Set IPGA/QPGA GAIN = 0dB
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb803312c, 0xffffff00, 0x00000000);		//<bit3:0>reg_ipga_d=0x0(0dB)
	return ret;

}

int REALTEK_R_ATSC3_Register(DEMOD* pDemod)
{
	int ret = 0;
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb816643c, 0xFFFFFE00, 0x000000D8);  //bsd_sync_th_weight
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb816646C, 0xFFFF0000, 0x000001e0);   //BSD thd, free_ctrl_v4[15:0]=480
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8160060, 0x00000002); //atsc3_bsd_rst_en  reset dcorr only

	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb816ef40, 0x00000040); // PLPA VLD=1 / ID=0
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb816e810, 0xFFFFFEFF, 0x00000100); //Plpa_force_mode=1, enable from 0416_r5395
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb816ef2c, 0x010c2040); //re-assign plp_id by Plp_dump_all, bit31:disable OIF filter, bit24:VV550 patch
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb816e808, 0xFFFFFFF5, 0x0000000a);
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb816e808, 0xFF01FFFF, 0x00FE0000);
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb816e808, 0x40FFFFFF, 0x81000000); //set debug mode for 2nd frame flag
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb816e80c, 0xFFFFFFFF); // plp_suben_lsb
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8163634, 0xFFFFFFEF, 0x00000000);  //param_h_t_remove_qerr_flag_a3
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb816e45c, 0xFFFFE000, 0x00000020); //atsc3_plp0_conti_err_rst_thd

	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb816c990, 0xFFFFFFFE, 0x00000001);  //params_average_cp_en
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8163448, 0xFFFFFF00, 0x00000020);  //params_cfo_dpll_c1_converge_16k=0x20
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb816344c, 0xFFFFFF00, 0x00000040);  //params_cfo_dpll_c2_converge_16k=0x40
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8163454, 0xFFFFFF00, 0x00000020);  //params_cfo_dpll_c2_converge_32k=0x20

	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb816c9b4, 0x00000008); //params_feq_sfi_det_th_1
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8166464, 0xfffffeff, 0x00000000); //free_ctrl_v2[8], bsd fcfo bypass peak check

	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8165948, 0x00000000); // ldpc_cfg_en
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8165940, 0x0000004b); // ldpc_cfg_addr 0x4B
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8165944, 0x00000048); // ldpc_cfg_dat
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8165948, 0x00000001); // ldpc_cfg_en
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8165948, 0x00000000); // ldpc_cfg_en
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8165940, 0x0000004c); // ldpc_cfg_addr 0x4C
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8165944, 0x0000000d); // ldpc_cfg_dat
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8165948, 0x00000001); // ldpc_cfg_en
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8165948, 0x00000000); // ldpc_cfg_en
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8165940, 0x0000004a); // ldpc_cfg_addr 0x4A
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8165944, 0x00000001); // ldpc_cfg_dat
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8165948, 0x00000001); // ldpc_cfg_en


	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8161c84, 0xffffffef, 0x00000010); //Oif_error_mode
	ret |= rtd_4bytes_i2c_mask(pDemod, 0xb8160010, 0xFFFFFFFE, 0x00000001);  //softcfg_done_r

	//reset DSP
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8160014, 0x00000001);//0x0005 = 0x01//Reset DSP
	ret |= rtd_4bytes_i2c_wr(pDemod, 0xb8160014, 0x00000000);//0x0005 = 0x00//Start DSP

	return ret;

}

void demod_i2c_init(DEMOD* pDemod)
{
    // copy from I2C_DDC_qc.c
    // MCM ATSC3.0 is on I2C_2

	//clken:0
	rtd_outl(0xb8000118,0x00104018);//[20]clken_i2c4[14]i2c3[4]i2c1[3]i2c2
	rtd_outl(0xb8000108,0x00104018);//[20]clken_i2c4[14]rstn_i2c3[4]i2c1[3]i2c2
	rtd_outl(0xb8000118,0x00104019);//[20]clken_i2c4[14]i2c3[4]i2c1[3]i2c2
	rtd_outl(0xb8000118,0x00104018);//[20]clken_i2c4[14]i2c3[4]i2c1[3]i2c2
	rtd_outl(0xb8000108,0x00104019);//[20]clken_i2c4[14]i2c3[4]i2c1[3]i2c2
	rtd_outl(0xb8000118,0x00104019);//[20]clken_i2c4[14]i2c3[4]i2c1[3]i2c2
	rtd_part_outl(0xb8060244,12,12,0x1);//I2C_ctrl_en
	//clock/4Hz(62.5 Hz)
	rtd_part_outl(0xb8000214,13,13,0x1);//I2C2

    //pinmux setting for I2C_2
    rtd_part_outl(0xb80008f0,9,8,0x0);//src0:input mux
    rtd_part_outl(0xb80008f0,13,12,0x0);//src0:input mux
    rtd_part_outl(0xb8000840,31,28,0xb);//sda:R360  Ps_Reg:gpio_57_ps 
    rtd_part_outl(0xb8000840,23,20,0xb);//scl:R362  Ps_Reg:gpio_58_ps 

    //Init I2C2 Master Setting
    rtd_outl(0xb801b56c,0x0);// I2C Enable; disable I2C for I2C
    rtd_outl(0xb801b500,0x63); //I2C_CON:bit[7]=0x22->set to Standard Mode,IC_RESTART_EN
    rtd_part_outl(0xb801b504,12,12,0); //I2C target[0]=0x0 ->7bit Addr,if as master, Bit control must set here
    rtd_part_outl(0xb801b504,9,0,pDemod->I2CAddr);	//IC_TAR:bit [9:0]; invalied in Gneral Call Mode           
    rtd_part_outl(0xb801b500,2,1,0x2);//IC1_CON:set to fast Mode
	rtd_outl(0xb801b51c, 0x8);       //IC1_FS_SCL_HCNT
	rtd_outl(0xb801b520, 0x26);       //IC1_FS_SCL_LCNT
    rtd_outl(0xb801b57c, 0x3);       //IC1_SDA_Hold
	rtd_outl(0xb801b56c,0x1);	// enable I2C2
}
int rtd_4bytes_i2c_mask(DEMOD *pDemod, unsigned int addr, unsigned int mask, unsigned int data)
{

	unsigned int readingData = 0, writingData = 0;

	rtd_4bytes_i2c_rd(pDemod, addr, &readingData);
	writingData = (readingData & mask) | data;
	rtd_4bytes_i2c_wr(pDemod, addr, writingData);

	return 1;
}
int rtd_4bytes_i2c_rd(DEMOD *pDemod, unsigned int addr, unsigned int* data)
{
	unsigned char getBuffer[4];
    addr &= 0x1FFFFFFF;
	unsigned char addrpart[4];
	addrpart[0] = addr & 0xff;
	addrpart[1] = (addr>>8) & 0xff;
	addrpart[2] = (addr>>16) & 0xff;
	addrpart[3] = (addr>>24) & 0xff;

    //I2C  Write reg addr
    rtd_outl(0xb801B510,addrpart[0]); // Write address without stop
    rtd_outl(0xb801B510,addrpart[1]); // Write address without stop
    rtd_outl(0xb801B510,addrpart[2]); // Write address without stop
    rtd_outl(0xb801B510,addrpart[3]); // Write address without stop
    
	while(rtd_inl(0xb801b574)!=0x0);

    rtd_outl(0xb801B510,0x00000500); // Write re-start + Read cmd, +without stop
    rtd_outl(0xb801B510,0x00000100); // Write Read cmd, without stop
    rtd_outl(0xb801B510,0x00000100); // Write Read cmd, without stop
    rtd_outl(0xb801B510,0x00000300); // Write Read cmd, with stop
	rtd_inl(0xb801b560); //clear I2C flag
	//IFD_QC_MSG_INFO("check TX:%x (before)\n",rtd_inl(0xb801b580));
    while(1)
	{
		if (((rtd_inl(0xb801b534) >> 9) & 0x1) == 1)
		{
			//IFD_QC_MSG_INFO("TX flr getVal:%x\n",rtd_inl(0xb801b574));
			int buf = rtd_inl(0xb801b578);
			//IFD_QC_MSG_INFO("Buffer:%d\n",buf);
			//IFD_QC_MSG_INFO("clear Flag is on\n");
			if (buf>0)
			{
				getBuffer[0] = rtd_inl(0xb801B510) & 0xff;
				getBuffer[1] = rtd_inl(0xb801B510) & 0xff;
				getBuffer[2] = rtd_inl(0xb801B510) & 0xff;
				getBuffer[3] = rtd_inl(0xb801B510) & 0xff;  
				break;
			}
		}
		//IFD_QC_MSG_INFO("Flag is werid\n");
	}
	//IFD_QC_MSG_INFO("check TX:%x (after)\n",rtd_inl(0xb801b580));
	//IFD_QC_MSG_INFO("addr:{%x,%x,%x,%x} -> read data:{%x,%x,%x,%x}\n",addrpart[3],addrpart[2],addrpart[1],addrpart[0], getBuffer[3],getBuffer[2],getBuffer[1],getBuffer[0]);
    *data = 0;
    *data |= getBuffer[0];
    *data |= (getBuffer[1] << 8);
    *data |= (getBuffer[2] << 16); 
    *data |= (getBuffer[3] << 24);

    return 1;  
}
int rtd_4bytes_i2c_wr(DEMOD *pDemod, unsigned int addr, unsigned int data)
{
    unsigned int WritingBuffer[8];
    addr &= 0x1FFFFFFF;

    WritingBuffer[0] = addr & 0xff;
	WritingBuffer[1] = (addr >> 8) & 0xff;
	WritingBuffer[2] = (addr >> 16) & 0xff;
	WritingBuffer[3] = (addr >> 24) & 0xff;

	WritingBuffer[4] = data & 0xff;
	WritingBuffer[5] = (data >> 8) & 0xff;
	WritingBuffer[6] = (data >> 16) & 0xff;
	WritingBuffer[7] = (data >> 24) & 0xff;



    //I2C  Write reg addr
    rtd_outl(0xb801B510,WritingBuffer[0]); // Write address without stop
    rtd_outl(0xb801B510,WritingBuffer[1]); // Write address without stop
    rtd_outl(0xb801B510,WritingBuffer[2]); // Write address without stop
    rtd_outl(0xb801B510,WritingBuffer[3]); // Write address without stop

    //I2C  Write reg addr
    rtd_outl(0xb801B510,WritingBuffer[4]); // Write data  without stop
    rtd_outl(0xb801B510,WritingBuffer[5]); // Write data  without stop
    rtd_outl(0xb801B510,WritingBuffer[6]); // Write data  without stop
    WritingBuffer[7] = (data >> 24) & 0xff;
    WritingBuffer[7] |= 0x200; //bit9 is write data stop
    rtd_outl(0xb801B510,WritingBuffer[7]); // Write data  with stop

	//IFD_QC_MSG_INFO("addr:{%x,%x,%x,%x} -> write data:{%x,%x,%x,%x}\n",WritingBuffer[3],WritingBuffer[2],WritingBuffer[1],WritingBuffer[0],WritingBuffer[7],WritingBuffer[6],WritingBuffer[5],WritingBuffer[4]);
	while(rtd_inl(0xb801b574)!=0x0);
	return 1;
}