/** PLEASE be noticed,
 * there are some drivers of HW modules with hard-coded configuration.
 * user should sync with THERMAL SD and CPU WRAPPER owner, and ...
 * to update 
 * - thermal init 
 * - scpu arch timer source 
 * - scpu clock setting.
 *
 * Otherwise, 
 * move them to suitable place to maintain by chip generation,
 * if there are such place.
 **/

#include <qc_verify_common.h>
#include <common.h>
#include <command.h>
#include <coproc.h>
#include <asm/system.h>
#include <pll_reg_reg.h>

#define RETURN_TAG    (MAGIC_TAG_UBOOT_EXIT)

// using hw semaphore to mutex printf
#define smp_printf(args...)				\
	do {						\
		while (rtd_inl(0xb801A620) == 0);	\
		printf(args);				\
		rtd_outl(0xb801A620, 0);		\
	}while(0)					\

#define smp_mutex(exp)					\
	do {						\
							\
		while (rtd_inl(0xb801A620) == 0);	\
		exp;					\
		dsb();					\
		rtd_outl(0xb801A620, 0);		\
	}while(0)					\

extern int _maxpower(int iter, int flag);
int maxpower(int);
extern void arm_ok(void);
#if defined(DHRTSTONE_TEST)
extern int main_dhrystone(void);
#endif //defined(DHRTSTONE_TEST)
void rtd_part_outl(unsigned int addr, unsigned int B, unsigned int C, unsigned int D);

int arg_quiet = 0;
int arg_sync = 1;
int arg_clock = 1000;
int arg_maxpower_loop = 50000;
static int _QC_Verifier_SCPU(void);

/* FIXME, 
** please sync with SCPU designer or SD for update-to-date chip specified parameter.
*/
int scpu_dvfs_clock(int clock) 
{

	if (clock == 1000) {
		rtd_part_outl(0xb805b008,0,0,0);// Oc_en =0
		rtd_part_outl(0xb805b004,15,8,0x25);// set N=37MHz
		rtd_part_outl(0xb805b004,26,16,0x5ed);//set Fcode_t_ssc=1517
		rtd_part_outl(0xb805b008,0,0,0x1);// Oc_en =1
	}
	
	if (clock == 700) {
		rtd_part_outl(0xb805b008,0,0,0);// Oc_en =0
		rtd_part_outl(0xb805b004,15,8,0x16);// set N=22MHz
		rtd_part_outl(0xb805b004,26,16,0x768);//set Fcode_t_ssc=1896
		rtd_part_outl(0xb805b008,0,0,0x1);// Oc_en =1
	}

	return 0;
}

#if CONFIG_QC_SCPU_PWR_CTRL

/* p-channel implement */
#define P_OFF     (0)
#define P_OFF_EMU (1)
#define P_ON      (8)
#define P_DENIED  (0b10)
#define P_ACCPETED (0b01)


unsigned int  PACTIVE_GET(int cpu)
{
	const unsigned int reg[] = {0xb805B4E4, 0xb805B4E8, 0xb805B4EC, 0xb805B4F0, 0xb805B87C};
	unsigned int val;

	val = rtd_inl(reg[cpu]);

	return (val & 0x1a0); //bit 8,7,5
}

unsigned int PSTATE_GET(int cpu)
{
	const unsigned int reg[] = {0xb805B4E0, 0xb805B4E0, 0xb805B4E0, 0xb805B4E0, 0xb805B870};
	unsigned int data;

	data = rtd_inl(reg[cpu]);
	data = (data >> ((cpu&0x3)*8)) & 0x03f;
	
	return data;
}

unsigned int PSTATE_SET_PREQ(int cpu, int val)
{
	const unsigned int reg[] = {0x1805B4E0, 0x1805B4E0, 0x1805B4E0, 0x1805B4E0, 0x1805B870};
	unsigned int data;
	const unsigned int preq = (1<<6);
	const unsigned int write_en = (1<<7);

	data = rtd_inl(reg[cpu]);
	data = (data & ~(0x7f << ((cpu&0x3)*8))) | ((write_en | preq | val) << ((cpu&0x3)*8));
	rtd_outl(reg[cpu], data);

	return data;
}

unsigned int  DENY_ACCEPT(int cpu)
{
	const unsigned int reg[] = {0x1805B4E4, 0x1805B4E8, 0x1805B4EC, 0x1805B4F0, 0x1805B87C};
	unsigned int val;

	val = rtd_inl(reg[cpu]);

	return ((val >> 20) & 0x3);
}

unsigned int PREQ_RELEASE(int cpu)
{
	const unsigned int reg[] = {0xb805B4E0, 0xb805B4E0, 0xb805B4E0, 0xb805B4E0, 0xb805B870};
	unsigned int data;
	const unsigned int preq = (1<<6)<<((cpu&0x3)*8);
	const unsigned int write_en = (1<<7) << ((cpu&0x3)*8);

	data = rtd_inl(reg[cpu]);
	data = (data & ~preq) | write_en;

	rtd_outl(reg[cpu], data);

	return data;
}

unsigned int nCPUPORESET_SET(int cpu)
{
	const unsigned int reg = 0xb805B054;
	const int offset[] = {10, 9, 4, 3, 20};
	unsigned int data;

	data = rtd_inl(reg);
	data = data & ~(1<<offset[cpu]) ;
	rtd_outl(reg, data);

	return data;
}

unsigned int nCPUPORESET_RELEASE(int cpu)
{
	const unsigned int reg = 0xb805B054;
	const int offset[] = {10, 9, 4, 3, 20};
	unsigned int data;

	data = rtd_inl(reg);
	data = data | (1<<offset[cpu]) ;
	rtd_outl(reg, data);

	return data;
}

unsigned int nCPUPORESET_GET(int cpu)
{
	const unsigned int reg = 0xb805B054;
	const int offset[] = {10, 9, 4, 3, 20};
	unsigned int data;

	data = rtd_inl(reg);

	return (data >> offset[cpu]) & 0x1;
}

unsigned int nCPURSTN_SET(int cpu)
{
	const unsigned int reg = 0xb805B054;
	const int offset[] = {12, 11, 6, 5, 21};
	unsigned int data;

	data = rtd_inl(reg);
	data = data & ~(1<<offset[cpu]) ;
	rtd_outl(reg, data);

	return data;
}

unsigned int nCPURSTN_RELEASE(int cpu)
{
	const unsigned int reg = 0xb805B054;
	const int offset[] = {12, 11, 6, 5, 21};
	unsigned int data;

	data = rtd_inl(reg);
	data = data | (1<<offset[cpu]) ;
	rtd_outl(reg, data);

	return data;
}

#define RETRY_COUNT_MAX	(100000)
#define RETRY_DELAY_US	(1)

/* return zero, if successful */
int platform_cpu_kill(unsigned int cpu)
{
	unsigned int status;
	int retry_count = 0;
	int kill_retry_count = 0;

//	printf("CPU%d", cpu);

	retry_count = 0;
	while (PACTIVE_GET(cpu) != P_OFF) {
		dsb();
#if 1 // check timeout
		if (retry_count < RETRY_COUNT_MAX)
		{
			udelay(RETRY_DELAY_US);
			retry_count++;
		}
		else
		{
			printf("Timeout: PACTIVE_GET(%d) is 0x%x (!= 0)\n", cpu, PACTIVE_GET(cpu));
			return -1;
		}
#endif
	}

retry:
	PSTATE_SET_PREQ(cpu, P_OFF);

	/* query denied or accepted */
	retry_count = 0;
	while ((status = DENY_ACCEPT(cpu)) == 0) {
		dsb();
#if 1 // check timeout
		if (retry_count < RETRY_COUNT_MAX)
		{
			udelay(RETRY_DELAY_US);
			retry_count++;
		}
		else
		{
			printf("Timeout: DENY_ACCEPT(%d) is 0x%x (== 0)\n", cpu, DENY_ACCEPT(cpu));
			return -1;
		}
#endif
	}

	/* release request */
	PREQ_RELEASE(cpu);

	if ((status == P_DENIED) && (kill_retry_count++ < 10))
		goto retry;

//	assert(status == P_ACCPETED);

	if (PSTATE_GET(cpu)  == P_OFF) {
		nCPUPORESET_SET(cpu);
//		nCPURSTN_SET(cpu);
//		printf("ok\n");
		return 0;
	}

	printf("FAIL: status(0x%x), PSTATE_GET(%d) = 0x%x\n", status, cpu, PSTATE_GET(cpu));
	return -1;
}

/* return zero, if successful */
int platform_cpu_wake(unsigned int cpu)
{
	unsigned int status;
	int retry_count = 0;
	int wakeup_retry_count = 0;

//	printf("CPU%d", cpu);

	/* 1st wake up without p-channel control */
	if (nCPUPORESET_GET(cpu) == 1) {
		nCPURSTN_RELEASE(cpu);
//		printf("OK\n");
		return 0;
	}

	retry_count = 0;
	while (PACTIVE_GET(cpu) != P_OFF) {
		dsb();
#if 1 // check timeout
		if (retry_count < RETRY_COUNT_MAX)
		{
			udelay(RETRY_DELAY_US);
			retry_count++;
		}
		else
		{
			printf("Timeout: PACTIVE_GET(%d) is 0x%x (!= 0)\n", cpu, PACTIVE_GET(cpu));
			return -1;
		}
#endif
	}

	/* core rst release */
	nCPUPORESET_RELEASE(cpu);
//	nCPURSTN_RELEASE(cpu);

retry:
	PSTATE_SET_PREQ(cpu, P_ON);

	/* query denied or accepted */
	retry_count = 0;
	while ((status = DENY_ACCEPT(cpu)) == 0) {
		dsb();
#if 1 // check timeout
		if (retry_count < RETRY_COUNT_MAX)
		{
			udelay(RETRY_DELAY_US);
			retry_count++;
		}
		else
		{
			printf("Timeout: DENY_ACCEPT(%d) is 0x%x (== 0)\n", cpu, DENY_ACCEPT(cpu));
			return -1;
		}
#endif
	}

	/* release request */
	PREQ_RELEASE(cpu);

	if ((status == P_DENIED) && (wakeup_retry_count++ < 10))
		goto retry;

//	assert(status == P_ACCPETED);

	if (PSTATE_GET(cpu) == P_ON) {
//		printf("ok\n");
		return 0;
	}

	printf("FAIL: status(0x%x),PSTATE_GET(%d) = 0x%x\n", status, cpu, PSTATE_GET(cpu));
	return -1;
}

int scpu_pwr_down(void)
{
	int ret = 0;
	int cpuid = 0;

	/*
	 * magic tag 0x508 = 0x1234cafe
	 * magic tag 0x510 = 0x1234cafe
	 * magic tag 0x518 = 0x1234cafe
	 */

	for(cpuid = 1; cpuid < NR_CPUS; cpuid++)
	{
		*(unsigned int *)(MAGIC_TAG_CORE0_reg + cpuid * 4) = MAGIC_COPROC_OP_POWER_CTRL;
	}
	flush_dcache_range(MAGIC_TAG_CORE0_reg, MAGIC_TAG_CORE0_reg + (4 * (NR_CPUS + 1)));
	asm volatile("SEV"); 
	mdelay(5);
	/*
	 * per-core kill
	 */
	ret |= platform_cpu_kill(1);
	ret |= platform_cpu_kill(2);
	ret |= platform_cpu_kill(3);
	ret |= platform_cpu_kill(4);

	return ret;
}
 
int scpu_pwr_on(void)
{
	int ret = 0;
	int cpu = 0;
	int cpuid = 0;
	int wake_done[NR_CPUS] = {1, 0, 0, 0, 0};
	int retry_count = 0;
	unsigned int tag;

	/*
	 * per-core wake up
	 */

	for(cpuid = 1; cpuid < NR_CPUS; cpuid++)
	{
		ret |= platform_cpu_wake(cpuid);
		tag = MAGIC_TAG_CORE0_reg + cpuid * 4;
		do {
			invalidate_dcache_range(MAGIC_TAG_CORE0_reg, MAGIC_TAG_CORE0_reg + (4 * (NR_CPUS + 1)));
		} while(rtd_inl(tag) != MAGIC_COPROC_IN_REDIRECT) ;
		rtd_outl(tag, MAGIC_COPROC_JUMP_TO_UBOOT);
		flush_dcache_range(MAGIC_TAG_CORE0_reg, MAGIC_TAG_CORE0_reg + (4 * (NR_CPUS + 1)));
	}

	/*
	 * wait for coreX wakeup
	 * wait magic tag 0x508 = 0x508
	 * wait magic tag 0x510 = 0x510
	 * wait magic tag 0x518 = 0x518
	 */
	retry_count = 0;
	while(1)
	{
		for (cpu = 1; cpu < NR_CPUS; cpu++)
		{
			if (wake_done[cpu] == 0)
			{
				invalidate_dcache_range(MAGIC_TAG_CORE0_reg, MAGIC_TAG_CORE0_reg + (4 * (NR_CPUS + 1)));
				if (*(unsigned int *)(MAGIC_TAG_CORE0_reg + cpu * 4) == MAGIC_COPROC_OP_GET_READY)
				{
//					printf(" cpu%d on\n", cpu);
					wake_done[cpu] = 1;
				}
			}
		}

		if (wake_done[1] && wake_done[2] && wake_done[3] && wake_done[4])
		{
			break;
		}

#if 1 // check timeout
		if (retry_count < RETRY_COUNT_MAX)
		{
			udelay(RETRY_DELAY_US);
			retry_count++;
		}
		else
		{
			printf("Timeout: wake_done = {%d,%d,%d,%d,%d}\n",
				wake_done[0], wake_done[1], wake_done[2], wake_done[3], wake_done[4]);
			return -1;
		}
#endif
	}

	return ret;
}
#endif

int QC_Verifier_SCPU(void)
{
	return _QC_Verifier_SCPU();
}	

static int _QC_Verifier_SCPU(void)
{
    int ret;
#if CONFIG_MAX_POWER_MEASURE
    u8 cpu_id;
    u32 regBase;
#endif    
    *(u32*)RETURN_TAG = 0; // reset ready flag

    arm_ok();
    ret = *(u32*)RETURN_TAG;

#if defined(DHRTSTONE_TEST)
    main_dhrystone();
#endif //defined(DHRTSTONE_TEST)

    /* used to sync all core to start test concurrently. */
    rtd_outl(0xb801a604, 0);
    

#if CONFIG_MAX_POWER_MEASURE
    ret |= main_maxpower();
#endif

#if CONFIG_QC_SCPU_PWR_CTRL
	printf("P_off %x\n", ret);
	ret |= scpu_pwr_down();
	printf("P_done %x\n", ret);
	ret |= scpu_pwr_on();
	printf("P_on %x\n", ret);
#endif

	if (ret == 0)
	{
		smp_printf("\n QC_SCPU verify success.\n");
		return SUCCESS;
	}
	else
	{
		smp_printf("QC_SCPU verify FAILED!!!!!\n");
		return FAILED;
	}
}

/* FIXME, 
** please sync with thermal SD for update-to-date chip specified parameter.
*/
void init_rtk_thermal_sensor(void)
{
	//int retry_count=0;
	static int thermal_init;

	if (!thermal_init) {
		//pre-setup
		//Thermal Sensor
		rtd_outl(SYS_REG_SYS_SRST2_reg, SYS_REG_SYS_SRST2_rstn_tm_mask | SYS_REG_SYS_SRST2_write_data_mask);//release thermal rst
		rtd_outl(SYS_REG_SYS_CLKEN2_reg, SYS_REG_SYS_CLKEN2_clken_tm_mask | SYS_REG_SYS_CLKEN2_write_data_mask);//enable thermal clk

		//modify default setting
		rtd_outl(0xb8000380,0x0851A3D7);//reg_a=29'b 0_1000_0101_0001_1010_0011_1101_0111
		rtd_outl(0xb8000384,0xdc8002a2);//reg_b=22'b 11_0111_0010_0000_0000_0000
		rtd_outl(0xb800038c,0x40000000);//reg_r=24'b 0100_0000_0000_0000_0000_0000

		mdelay(20);
		thermal_init=1;
	}
}

/* FIXME, 
** please sync with CPU wrapper spec or designer for update-to-date chip specified parameter.
*/
#define WRAP_A7_APB_TSG_EN0 0xB805C800
#define WRAP_A7_APB_TSG_EN1 0xB805C804
void scpu_wrapper_timer_init(void)
{
	/* init wrapper timer for arm generic timer */
	// configure coresight CNTCR
	rtd_outl(WRAP_A7_APB_TSG_EN0, 0);
	rtd_outl(WRAP_A7_APB_TSG_EN1, 1);

	// configure coresight CNTFID0
	rtd_outl(WRAP_A7_APB_TSG_EN0, 0x20);
	rtd_outl(WRAP_A7_APB_TSG_EN1, 0x019bfcc0);
}

#if CONFIG_MAX_POWER_MEASURE
#define DEBOUNCE_WINDOW 50
int main_maxpower(void)
{
    int ret;
    u8 cpu_id;
    u32 regBase;

	for (cpu_id = 1 ; cpu_id < NR_CPUS ; cpu_id++)
	{
		if (cpu_id == 1)
		{
			regBase = MAGIC_TAG_CORE1_reg;
			smp_printf("CPU %d _________\n", cpu_id);
		}
		else if (cpu_id == 2)
		{
			regBase = MAGIC_TAG_CORE2_reg;
			smp_printf("CPU %d _________\n", cpu_id);
		}
		else if (cpu_id == 3)
		{
			regBase = MAGIC_TAG_CORE3_reg;
			smp_printf("CPU %d _________\n", cpu_id);
		}
		else if (cpu_id == 4)
		{
			regBase = MAGIC_TAG_CORE4_reg;
			smp_printf("CPU %d _________\n", cpu_id);
		}

		/* sanity check, secondary cores should be "READY" state.
		** otherwise, violation of redirection code is hard to debug.
		*/
		invalidate_dcache_range(MAGIC_TAG_CORE0_reg, MAGIC_TAG_CORE0_reg + (4 * (NR_CPUS + 1)));
		if (rtd_inl(regBase) != MAGIC_COPROC_OP_GET_READY) {
			smp_printf("0x%x = 0x%x \n",  regBase, rtd_inl(regBase));
			assert(0);
		}

		rtd_outl(regBase, MAGIC_COPROC_OP_MAX_POWER_TEST);
		flush_dcache_range(MAGIC_TAG_CORE0_reg, MAGIC_TAG_CORE0_reg + (4 * (NR_CPUS + 1)));
	}

    /* free all secondary cores */
#ifdef CONFIG_USE_IRQ
    ipi_send_target(cpuid,2);
#else
    asm volatile("SEV");
#endif
    smp_printf("CPU 0 _________\n");
    ret = maxpower(0);
    return ret;
}

int maxpower(int cpuid)
{
	int count = 0;
	static int stop = 0;
	int key = 0;
	static volatile int cpumask;
	int prev_val = -1;
	int debounce = DEBOUNCE_WINDOW;



	/* sync by core 0, to maximize the power peak */
	if (cpuid == 0) {
		scpu_wrapper_timer_init();
		init_rtk_thermal_sensor();
	}

	// say who come in

	while (rtd_inl(0xb801A620) == 0);		
	cpumask |= (1<<cpuid);
	printf("maxpower enter: cpu_%d mask 0x%x \n", cpuid, cpumask);
	rtd_outl(0xb801A620, 0);		
	//smp_mutex(cpumask |= (1<<cpuid));
	//smp_printf(" %s:%d  \n", __FUNCTION__, __LINE__);
    while(cpumask != ((1<<NR_CPUS) -1)); // All cores wait here and then start.

	while (!stop) {
        //mdelay(2);

		if (!arg_quiet)
			smp_printf("cpu_%d: inter %d \n", cpuid, count++);


		// print thermal 
		if (cpuid == 0) {
			int val = rtd_inl(PLL_REG_THERMAL_RESULT_1_reg);
			
			if (val & (1<<18)) 
				smp_printf("thermal sensor not ready: val=0x%x \n", val);
			else {
				val >>= 10;
				if (val != prev_val) {
					if (debounce++> DEBOUNCE_WINDOW) {
						smp_printf("temperature: %d 'C\n", val);
						prev_val = val;
					}
				}
				else
					debounce = 0;
			}
		}

		//smp_printf("cpu_%d: inter %d, %s:%d  \n", cpuid, count , __FUNCTION__, __LINE__);
		_maxpower(arg_maxpower_loop, arg_sync);
		

		if (tstc()) {
			key = getc();
			smp_printf("key: 0x%x \n", key);
			stop = 1;
		}
	}

	// say who exit
	smp_printf("cpu_%d maxpower exit \n", cpuid);


	while (rtd_inl(0xb801A620) == 0);		
	cpumask &= ~(1<<cpuid);
	printf("maxpower exit: cpu_%d mask 0x%x \n", cpuid, cpumask);
	rtd_outl(0xb801A620, 0);		
	
	//smp_mutex(cpumask &= ~(1<<cpuid));
	//smp_printf("maxpower exit: cpu_%d mask 0x%x \n", cpuid, cpumask);
	while(cpumask != 0x0);
	stop = 0;

	return 0;
}
#endif

int do_change_clock(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	char *ep;
	int rate = -1;
	int argi = 1;

	while (argc > argi) {
		if( strncmp( argv[argi], "clk", 3 ) == 0 ) {
			if (argc > ++argi) {
				rate = (int)simple_strtoul(argv[argi], &ep, 10);
				++argi;
			}

			if (rate != 700 && rate != 1000 ) {
				puts("\n wrong clock argument \n");
				break;
			}
			
			arg_clock = rate;
			scpu_dvfs_clock(arg_clock);
			
			printf("scpu_dvfs_clock: %d Mhz \n", arg_clock);

		}
		else 
		if( strncmp( argv[argi], "sync", 4 ) == 0 ) {
			if (argc > ++argi)  {
				arg_sync = (int)simple_strtoul(argv[argi], &ep, 10);
				++argi;
			}
			else 
				arg_sync = 1;

			arg_sync = (arg_sync != 0);

			printf("sync: %d \n", arg_sync);
		}
		else 
		if( strncmp( argv[argi], "quiet", 4 ) == 0 ) {
			if (argc > ++argi) {
				arg_quiet = (int)simple_strtoul(argv[argi], &ep, 10);
				++argi;
			}
			else 
				arg_quiet = 1;

			// backdoor to setup loop count
			if (arg_quiet > 0x1000) {
				arg_maxpower_loop = arg_quiet;
				
				printf("power loop: %d \n", arg_maxpower_loop);
			}

			arg_quiet = (arg_quiet != 0);
			printf("quiet: %d \n", arg_quiet);
		}
	}

		

	return 0;
}

U_BOOT_CMD(
	sc,	CONFIG_SYS_MAXARGS,	0,	do_change_clock,
	"sc, max power verification config",
	"{clk 700|1000 } {sync 0|1} {quiet 0|1}\n"
	"    - clk, select 700 or 1000 Mhz\n"
	"    - sync, all core begin at same clock to make maximum peak power.\n"
	"    - quiet, less output message\n"
);

