#include "vde_qc_bin.h"
#include "io.h"
#include "timer.h"
#include "util.h"
#include <stdio.h>
#include <ve_mbist_reg.h>
#include <topbist_reg.h>
#include <sys_reg_reg.h>
#include <vde_reg.h>

/************************************************************************
 *  Definitions
 ************************************************************************/
#define VDE_DEBUG
#ifdef VDE_DEBUG
#define debug(fmt,args...)	printf (fmt ,##args)
#else
#define debug(fmt,args...)
#endif

struct hex_data * hex_vde1_dir0_addr = pattern_vde10;
int hex_vde1_dir0_len = (sizeof(pattern_vde10)/sizeof(struct hex_data));
struct hex_data * hex_vde1_dir1_addr = pattern_vde11;
int hex_vde1_dir1_len = (sizeof(pattern_vde11)/sizeof(struct hex_data));

struct hex_data * hex_vde2_dir0_addr = pattern_vde20;
int hex_vde2_dir0_len = (sizeof(pattern_vde20)/sizeof(struct hex_data));
struct hex_data * hex_vde2_dir1_addr = pattern_vde21;
int hex_vde2_dir1_len = (sizeof(pattern_vde21)/sizeof(struct hex_data));

//#define SWAP32(DATA) (((DATA>>24)&0xff)|((DATA>>8)&0xff00)|((DATA<<8)&0x00ff0000)|((DATA<<24)&0xff000000))
#define BYTE(d,n)		(((d) >> ((n) << 3)) & 0xFF)
#define SWAPEND32(d)	((BYTE(d,0) << 24) | (BYTE(d,1) << 16) | (BYTE(d,2) << 8) | (BYTE(d,3) << 0))

/************************************************************************
 *  External variables
 ************************************************************************/
extern int const _f_v_entry_qc_img, _e_v_entry_qc_img;

unsigned int *VideoFW_entry_qc_ptr_V1		= (unsigned int *)0x00002200;
unsigned int *VideoFW_entry_qc_ptr_V2		= (unsigned int *)0x00002210;

#define MIPS_VIDEO_VDE_1_PATTERN_ENTRY_ADDR		(0x03204000)
#define MIPS_VIDEO_VDE_2_PATTERN_ENTRY_ADDR		(0x04004000)

void prepare_ventry(void)
{
	unsigned char *a,*b;

	a = (unsigned char *)& _f_v_entry_qc_img;
	b = (unsigned char *)& _e_v_entry_qc_img;
	// copy v_entry_qc
	debug("copy v_entry_qc  from addr = 0x%x, to 0x%x, size = 0x%x\n", a, MIPS_V_ENTRY_QC_CODE_ADDR, b-a);
	memcpy((unsigned char *)MIPS_V_ENTRY_QC_CODE_ADDR, a, b-a);

	rtd_outl(VCPU_JUMP_ADDR_reg, MIPS_V_ENTRY_QC_CODE_ADDR | MIPS_KSEG1BASE);
	rtd_outl(VCPU2_JUMP_ADDR_reg, MIPS_V_ENTRY_QC_CODE_ADDR | MIPS_KSEG1BASE);

	debug("vcpu from romcode to entry = 0x%x\n", rtd_inl(VCPU_JUMP_ADDR_reg));
}


void prepare_binary(struct hex_data * hex_addr, int len)
{
	debug("%s\n", __FUNCTION__);

	unsigned int i;

	for (i = 0;i < len; i++) {
		rtd_outl(hex_addr[i].addr, hex_addr[i].data);
		//debug("0x%x\n", hex_addr[i].addr);
	}

	debug("write word count: %d\n", i);

	rtd_outl(0x01fff080,0x00000000); //ve compute CRC will use
	rtd_outl(0x01fff084,0x00000000); //ve compute CRC will use
	rtd_outl(0x01fff000,0xFFFF0000); //ve compute CRC will use

	rtd_outl(0x01fff040,0x00000000); //frame decode cnt
	rtd_outl(0x01fff06c,0x00000000); //frame display cnt
	rtd_outl(0x01fff41c,0x00000000); //start_frame
	rtd_outl(0x00002f40,0x20000000); //end_frame
	rtd_outl(0x00002f4c,0x00000000); //0:decoding order
	rtd_outl(0x00002f50,0x14000000); //decoder configuration word
	//rtd_outl(0x00002f50,0x05000000); //decoder configuration word
	rtd_outl(0x00002f54,0x01000000); //crc check
	// rtd_outl(0xb8007004,0x098b0100);

	// for VDE1
	rtd_outl(VDE1_TEST_FLAG_ADDR, SWAPEND32(0xbabebabe));
	rtd_outl(VDE1_TEST_MODE_ADDR, 0x01000000);//vcpu-only test
	rtd_outl(MIPS_VDE1_ENTRY_QC_CODE_ADDR_PTR, SWAPEND32(MIPS_V_ENTRY_QC_CODE_ADDR|MIPS_KSEG1BASE)); //when finish vcpu will jump to this entry again
	rtd_outl(VDE1_CMD_ADDR, VDE1_CMD_START); //start run


	// for VDE2
	rtd_outl(VDE2_TEST_FLAG_ADDR,SWAPEND32(0xbabebabe)); //test pass|fail flag
	rtd_outl(VDE2_TEST_MODE_ADDR, 0x01000000);//vcpu-only test
	rtd_outl(MIPS_VDE2_ENTRY_QC_CODE_ADDR_PTR, SWAPEND32(MIPS_V_ENTRY_QC_CODE_ADDR|MIPS_KSEG1BASE)); //when finish vcpu will jump to this entry again
	rtd_outl(VDE2_CMD_ADDR, VDE2_CMD_START); //start run


	return;
}

/*
 * disable video related clock
 */
int disable_video_clock(int vcpu_no)
{
	int ret = 0;

	switch(vcpu_no)
	{
		case 1:
/*			// clk = 0
			// VP9, VDE, VE_P2, VE, VCPU
			// VDEC, ME, SE
			rtd_clearbits(CRT_SYS_CLKEN0_reg, _BIT13 | _BIT5 | _BIT4 | _BIT3);
			rtd_clearbits(CRT_SYS_CLKEN1_reg, _BIT22 | _BIT12 | _BIT5);

			// rst = 0
			// VP9, VDE, VE_P2, VE, VCPU
			// VDEC, ME, SE
			rtd_clearbits(CRT_SYS_SRST0_reg, _BIT13 | _BIT6 | _BIT5 | _BIT4 | _BIT3);
			rtd_clearbits(CRT_SYS_SRST1_reg, _BIT22 | _BIT12 | _BIT5);
*/
			// merlin2
			// rst = 0
			// VDE, VE_P2, VE, VCPU
			rtd_outl(CRT_SYS_SRST0_reg,  _BIT6 | _BIT5 | _BIT4 | _BIT3);

			// clk = 0
			rtd_outl(CRT_SYS_CLKEN0_reg, _BIT5 | _BIT4 | _BIT3);
			break;

		case 2:
/*
			// clk = 0
			// VDE2, VE2_P2, VE2, VCPU2
			rtd_clearbits(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);

			// rst = 0
			// VDE2, VE2_P2, VE2, VCPU2
			rtd_clearbits(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);
*/
			// clk = 0
			// VE2_P2, VE2, VCPU2
			rtd_outl(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);

			// rst = 0
			// VDE2, VE2_P2, VE2, VCPU2
			rtd_outl(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);
			break;

		default:
			printf("## Unknown option %d\n", vcpu_no);
			ret = 1;
			break;
	}

	return ret;
}

#define vcpu_repair_check_bits    (_BIT31)

/*
 * cpuid:
 *      bit0 -> vcpu1
 */
int do_vcpu_bisr(unsigned int vcpuid)
{
	unsigned int iMaxCount = 10000;
	int ret = 0;

	if (vcpuid & _BIT0) // VCPU
	{
		printf("VCPU: BISR\n");

		// bisr pwr rstn
		rtd_setbits(TOPBIST_cpu_bisr_rstn_reg, _BIT16);

		// bisr rstn
		rtd_setbits(TOPBIST_cpu_bisr_rstn_reg, _BIT1);

		//wait bist done
		while((rtd_inl(TOPBIST_cpu_bisr_done_reg) & _BIT1) != _BIT1) {
			iMaxCount--;
			udelay(100);
			if( iMaxCount == 0 ) 
			{
				printf("VCPU: timeout %x\n", rtd_inl(TOPBIST_cpu_bisr_done_reg) & _BIT1);
				ret = -1;
				while(1);
			}
		}

		// assert hold remap
		rtd_setbits(TOPBIST_cpu_bisr_remap_reg, _BIT1);

		// check fail
		if (rtd_inl(VDE_VCPU_bisr_fail_g_reg) & _BIT0)
		{
//			enable_console();
			printf("VCPU: fail %x\n", rtd_inl(VDE_VCPU_bisr_fail_g_reg) & _BIT0);
			while(1);
		}
		else
		{
			printf("VCPU: ok\n");
		}

		// check repair
		if((rtd_inl(VDE_VCPU_bisr_drf_fail_reg) & vcpu_repair_check_bits) != 0) {
			printf("VCPU: repair\n");
		}
	}

	return ret;
}

/*
 * enable video related clock
 */
int enable_video_clock(void)
{
	// rst = 0
	//VE_BLK_WRAP, VE_LGCY_P2, VE_LGCY, VDE, VE_P2, VE, VCPU
	rtd_outl(SYS_REG_SYS_SRST0_reg, _BIT9 | _BIT8 | _BIT6 | _BIT5 | _BIT4 | _BIT3);
	rtd_outl(SYS_REG_SYS_SRST1_reg, _BIT4);

	// clk = 1
	// VE_LGCY_P2, VE_LGCY, VE_P2, VE, VCPU
	rtd_outl(SYS_REG_SYS_CLKEN0_reg, _BIT8	 | _BIT7  | _BIT6 | _BIT5 | _BIT4 | _BIT3 | _BIT0);

	// clk = 0
	rtd_outl(SYS_REG_SYS_CLKEN0_reg, _BIT8	| _BIT7 | _BIT6 | _BIT5 | _BIT4 | _BIT3);

	// rst = 0
	rtd_outl(SYS_REG_SYS_SRST0_reg, _BIT9 | _BIT8 | _BIT6 | _BIT5 | _BIT4 | _BIT3);
	rtd_outl(SYS_REG_SYS_SRST1_reg, _BIT4);
	udelay(10);

	// rst = 1
	rtd_outl(SYS_REG_SYS_SRST0_reg, _BIT9 | _BIT8 | _BIT6 | _BIT5 | _BIT4 | _BIT3 | _BIT0);
	rtd_outl(SYS_REG_SYS_SRST1_reg, _BIT4 |  _BIT0);
	udelay(10);

	// clk = 1
	rtd_outl(SYS_REG_SYS_CLKEN0_reg, _BIT8 | _BIT7 | _BIT6 | _BIT5 | _BIT4 | _BIT3 | _BIT0);

}

/*
 * enable video2 related clock
 */
void enable_video2_clock(void)
{
#if 0
	// rst = 0
	// VDE2, VE2_P2, VE2, VCPU2
	rtd_clearbits(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);

	// clk = 1
	// VE2_P2, VE2, VCPU2
	rtd_setbits(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);

	// clk = 0
	rtd_clearbits(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);

	// rst = 0
	rtd_clearbits(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);
	udelay(10);

	// rst = 1
	rtd_setbits(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);
	udelay(10);

	// clk = 1
	rtd_setbits(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);
#else
/*
	// clk = 0
	rtd_clearbits(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);
	udelay(10);

	// rst = 0
	// VDE2, VE2_P2, VE2, VCPU2
	rtd_clearbits(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);
	udelay(10);

	// clk = 1
	// VE2_P2, VE2, VCPU2
	rtd_setbits(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);
        udelay(10);

	// clk = 0
	rtd_clearbits(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);
	udelay(10);

	// rst = 1
	rtd_setbits(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);
	udelay(10);

	// clk = 1
	rtd_setbits(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);
	udelay(10);
*/

	// rst = 0
	// VDE2, VE2_P2, VE2, VCPU2
	rtd_outl(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);

	// clk = 1
	// VE2_P2, VE2, VCPU2
	rtd_outl(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14 | _BIT0);

	// clk = 0
	rtd_outl(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);

	// rst = 0
	rtd_outl(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);
	udelay(10);

	// rst = 1
	rtd_outl(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14 | _BIT0);
	udelay(10);

	// clk = 1
	rtd_outl(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14 | _BIT0);
#endif
}

static void release_hw_semaphore(void)
{
//	printf("## Release HW semaphore\n");
	rtd_outl(SB2_HD_SEM_reg, 0x0);
}

__attribute__ ((unused)) static void hold_hw_semaphore(void)
{
	//printf("## Hold HW semaphore\n");
	while(!(rtd_inl(SB2_HD_SEM_reg)))
	{
		udelay(100);
	}
}

static unsigned long do_go_video_fw(int vcpu_no)
{
	int ret = 0;

	switch (vcpu_no) {
		case 0:
			printf("Start Video Firmware 12...\n");
			*VideoFW_entry_qc_ptr_V1 = SWAPEND32(MIPS_VIDEO_VDE_1_PATTERN_ENTRY_ADDR | MIPS_KSEG1BASE);
			*VideoFW_entry_qc_ptr_V2 = SWAPEND32(MIPS_VIDEO_VDE_2_PATTERN_ENTRY_ADDR | MIPS_KSEG1BASE);
			break;

		case 1:
			*VideoFW_entry_qc_ptr_V1 = SWAPEND32(MIPS_VIDEO_VDE_1_PATTERN_ENTRY_ADDR| MIPS_KSEG1BASE);
			printf("%######\n");
			break;

		case 2:
			//*VideoFW_entry_qc_ptr_V2 = SWAPEND32(MIPS_VIDEO_VDE_2_PATTERN_ENTRY_ADDR| MIPS_KSEG1BASE);
			*VideoFW_entry_qc_ptr_V2  = SWAPEND32(MIPS_VIDEO_VDE_1_PATTERN_ENTRY_ADDR| MIPS_KSEG1BASE);

		printf("%######\n");
			break;

		default:
			printf("## Unknown option %d\n", vcpu_no);
			ret = 1;
			break;
	}

	if (ret == 0)
	{
		flush_dcache_all();

		mdelay(10);
		release_hw_semaphore();
		printf("%s %d hold_hw_semaphore\n", __FUNCTION__, __LINE__);
		hold_hw_semaphore();

		/* Enable VCPU1 */
		if (vcpu_no == 0 || vcpu_no == 1)
		{
			enable_video_clock();
			do_vcpu_bisr(_BIT0);
		}

		/* Enable VCPU2 */
		if (vcpu_no == 0 || vcpu_no == 2)
		{
			enable_video2_clock();
			do_vcpu_bisr(_BIT1);
		}

		release_hw_semaphore();
	}

	return ret;
}

#if 0 //ve1_run ve2_run ve1_check  ve2_check
// wait VE pattern exc out
int ddr_debug_01 (void)
{
	debug("\n%s\n", __FUNCTION__);

	volatile unsigned int check_status_vde1 = 0;
	volatile unsigned int check_status_vde2 = 0;
	volatile unsigned int check_end_vde1 = 0; /* VDE1 end flag */
	volatile unsigned int check_end_vde2 = 0; /* VDE2 end flag */
	unsigned int timeout = 0x00001000;
	unsigned int error = 0;


#if !defined(CONFIG_QC_VDE1)
	check_end_vde1 = 1;
#endif

#if !defined(CONFIG_QC_VDE2)
	check_end_vde2 = 1;
#endif

	while(--timeout)
	{
#if defined(CONFIG_QC_VDE1)
		invalidate_dcache_range(0x01fff000, 0x01fff000+4);
		check_status_vde1 = rtd_inl(0x01fff000); /* VDE1 run status addr */
		if (check_status_vde1 == 0)
		{
			debug("vde1_idle       %x\n", check_status_vde1);
		}
		else if(check_status_vde1 == 0x01000000)
		{
			debug("vde_busy?       %x\n", check_status_vde1);
			udelay(200);
		}
		else if(check_status_vde1 == 0x02000000)
		{
			debug("vde1_done?       %x\n", check_status_vde1);
			check_end_vde1 = 1;
		}
		else if (check_status_vde1 == 0x03000000)
		{
			debug("vde1_crc_err?       %x\n", check_status_vde1);
			check_end_vde1 = 1;
			error = 1;
		}
		else
		{
			//debug("VDE other ?       %x\n", check_status_vde1);
		}
#endif /* CONFIG_QC_VDE1 */

#if defined(CONFIG_QC_VDE2)
		invalidate_dcache_range(0x04fff000, 0x04fff000+4);
		check_status_vde2 = rtd_inl(0x04fff000);

		if (check_status_vde2 == 0)
		{
			debug("vde2_idle       %x\n", check_status_vde2);
		}
		else if(check_status_vde2 == 0x01000000)
		{
			//debug("vde2_busy?       %x\n", check_status_vde2);
			udelay(200);
		}
		else if(check_status_vde2 == 0x02000000)
		{
			debug("vde2_done?       %x\n", check_status_vde2);
			check_end_vde2 = 1;
		}
		else if (check_status_vde2 == 0x03000000)
		{
			debug("vde2_crc_err?       %x\n", check_status_vde2);
			check_end_vde2 = 1;
			error = 1;
		}
		else
		{
			debug("VDE other ?       %x\n", check_status_vde2);
		}
#endif /* CONFIG_QC_VDE2 */


		if((check_end_vde1==1) && (check_end_vde2==1))
		{
			debug("vde1 and vde2 run end\n");
			break;
		}

	} /* while(--timeout) */

	if (timeout){
#if defined(CONFIG_QC_VDE1)
		debug("check_status_vde1:0x%08x\n",check_status_vde1);
#endif

#if defined(CONFIG_QC_VDE2)
		debug("check_status_vde2:0x%08x\n",check_status_vde2);
#endif

		debug("time not out~!\n");
		return 0;
	}
	else if (error) {

#if defined(CONFIG_QC_VDE1)
		debug("check_status_vde1:0x%08x\n",check_status_vde1);
#endif

#if defined(CONFIG_QC_VDE2)
		debug("check_status_vde2:0x%08x\n",check_status_vde2);
#endif

		printf("crc error~!\n");
		return -1;
	}
	else {

#if defined(CONFIG_QC_VDE1)
		debug("check_status_vde1:0x%08x\n",check_status_vde1);
#endif
#if defined(CONFIG_QC_VDE2)
		debug("check_status_vde2:0x%08x\n",check_status_vde2);
#endif
		printf("timeout~!\n");
		return -2;
	}
}

int run_hex (struct hex_data * hex_addr, int len)
{
	debug("%s\n", __FUNCTION__);

	int ret = 0;
	unsigned int result_vde1,result_vde2;
	unsigned int debug_value = 0;

	prepare_binary(hex_addr, len);
	prepare_ventry();
	flush_dcache_all();

#if defined(CONFIG_QC_VDE1)
	do_go_video_fw(1);
#endif

	mdelay(20);

#if defined(CONFIG_QC_VDE2)
	do_go_video_fw(2);
#endif


	ret = ddr_debug_01();


#if 0  // debug
	printf("debug\n");
	debug_value = SWAPEND32(rtd_inl(0x1800e400));
	printf("0xb800e400 : = 0x%x\n", debug_value);

	debug_value = SWAPEND32(rtd_inl(0x1800e404));
	printf("0xb800e404 : = 0x%x\n", debug_value);

	debug_value = SWAPEND32(rtd_inl(0x82100000 ));
	printf("0x82100000  : = 0x%x\n", debug_value);

	debug_value = SWAPEND32(rtd_inl(0xa2100000 ));
	printf("0xa2100000  : = 0x%x\n", debug_value);

#endif

#if defined(CONFIG_QC_VDE1)
	rtd_outl(VDE1_CMD_ADDR, VDE1_CMD_STOP); //stop VDE1
#if 0
	rtd_outl(SB2_HD_SEM_reg, 0x1);
#endif

	disable_video_clock(1);
#if 0
        rtd_outl(SB2_HD_SEM_reg, 0x0);
#endif

#endif

#if defined(CONFIG_QC_VDE2)
	rtd_outl(VDE2_CMD_ADDR, VDE2_CMD_STOP); //stop VDE2
#if 0
	rtd_outl(SB2_HD_SEM_reg, 0x1);
#endif

	disable_video_clock(2);
#if 0
        rtd_outl(SB2_HD_SEM_reg, 0x0);
#endif

#endif


#if defined(CONFIG_QC_VDE1)
	invalidate_dcache_range(VDE1_TEST_FLAG_ADDR, VDE1_TEST_FLAG_ADDR+4);
	result_vde1 = SWAPEND32(rtd_inl(VDE1_TEST_FLAG_ADDR));  // VDE1 test pass|fail flag
	printf("result_vde1 = 0x%x\n", result_vde1);
#endif

#if defined(CONFIG_QC_VDE2)
	invalidate_dcache_range(VDE2_TEST_FLAG_ADDR, VDE2_TEST_FLAG_ADDR+4);
	result_vde2 = SWAPEND32(rtd_inl(VDE2_TEST_FLAG_ADDR));  // VDE2 test pass|fail flag
	printf("result_vde2 = 0x%x\n", result_vde2);
#endif

// judge pass or fail
// VDE1 and VDE2
#if (defined(CONFIG_QC_VDE1) && defined(CONFIG_QC_VDE2))
	if (0xbabebabe != result_vde1 || 0xbabebabe != result_vde2 || ret <0)
	{
		if(-2 == ret)
		{
			printf("VDE1 and VDE2 FAIL\n");
		}
		else
		{
			if (0xbabebabe != result_vde1)
			{
				printf("VDE1 FAIL\n");
			}
			else
			{
				printf("VDE1 PASS\n");
			}
			if (0xbabebabe != result_vde2)
			{
				printf("VDE2 FAIL\n");
			}
			else
			{
				printf("VDE2 PASS\n");
			}

		}

		return -1;
	}
	else
	{
		printf("VDE1 and VDE2  PASS\n");
		return 0;
	}

#else // VDE1 or VDE2

#if defined(CONFIG_QC_VDE1) // only VDE1
	if (0xbabebabe != result_vde1 || ret < 0)
	{
		printf("VDE1 FAIL\n");
		return -1;
	}
	else
	{
		printf("VDE1 PASS\n");
		return 0;
	}
#endif // #if defined(CONFIG_QC_VDE1)

#if defined(CONFIG_QC_VDE2) // only VDE2
	if (0xbabebabe != result_vde2 || ret < 0)
	{
		printf("VDE2 FAIL\n");
		return -1;
	}
	else
	{
		printf("VDE2 PASS\n");
		return 0;
	}
#endif // #if defined(CONFIG_QC_VDE2)

#endif // #if (defined(CONFIG_QC_VDE1) && defined(CONFIG_QC_VDE2))
}

#else // vde1_run vde1_check ve2_run vde2_check
extern void invalidate_dcache_range(unsigned long start, unsigned long stop);
int ddr_debug_01 (int VDE_num)
{
	debug("\n%s\n", __FUNCTION__);

	volatile unsigned int check_status_vde1 = 0;
	volatile unsigned int check_status_vde2 = 0;
	unsigned int timeout = 5000;
	unsigned int error = 0;

	while(--timeout)
	{
		if(VDE_num == 1)
		{
			invalidate_dcache_range(0x01fff000, 0x01fff000+4);
			check_status_vde1 = rtd_inl(0x01fff000); /* VDE1 run status addr */
			if (check_status_vde1 == 0)
			{
				debug("vde1_idle       %x\n", check_status_vde1);
			}
			else if(check_status_vde1 == 0x01000000)
			{
				debug("vde_busy?       %x\n", check_status_vde1);
				udelay(100);
			}
			else if(check_status_vde1 == 0x02000000)
			{
				debug("vde1_done?       %x\n", check_status_vde1);
				break;
			}
			else if (check_status_vde1 == 0x03000000)
			{
				debug("vde1_crc_err?       %x\n", check_status_vde1);
				error = 1;
				break;
			}
			else
			{
				//debug("VDE other ?       %x\n", check_status_vde1);
			}
		} // if(is_VDE1 == 1)

		if(VDE_num == 2)
		{
			invalidate_dcache_range(0x01fff000, 0x01fff000+4);
			check_status_vde2 = rtd_inl(0x01fff000);

			if (check_status_vde2 == 0)
			{
				debug("vde2_idle       %x\n", check_status_vde2);
			}
			else if(check_status_vde2 == 0x01000000)
			{
				debug("vde2_busy?       %x\n", check_status_vde2);
				udelay(100);
			}
			else if(check_status_vde2 == 0x02000000)
			{
				debug("vde2_done?       %x\n", check_status_vde2);
				break;
			}
			else if (check_status_vde2 == 0x03000000)
			{
				debug("vde2_crc_err?       %x\n", check_status_vde2);
				error = 1;
				break;
			}
			else
			{
				debug("VDE other ?       %x\n", check_status_vde2);
			}
		}

	} /* while(--timeout) */

	if(VDE_num == 1)
	{
		if (error) {
			debug("check_status_vde1:0x%08x\n",check_status_vde1);
			printf("crc error~!\n");
			return -1;
		}
		else if (timeout){
			debug("check_status_vde1:0x%08x\n",check_status_vde1);
			debug("time not out~!\n");
			return 0;
		}
		else {
			debug("check_status_vde1:0x%08x\n",check_status_vde1);
			printf("timeout~!\n");
			return -2;
		}
	}// if(VDE_num == 1)
	else if(VDE_num == 2)
	{
		if (error) {
			debug("check_status_vde2:0x%08x\n",check_status_vde2);
			printf("crc error~!\n");
			return -1;
		}
		else if (timeout){
			debug("check_status_vde2:0x%08x\n",check_status_vde2);
			debug("time not out~!\n");
			return 0;
		}
		else {
			debug("check_status_vde2:0x%08x\n",check_status_vde2);
			printf("timeout~!\n");
			return -2;
		}
	} // else if(VDE_num == 2)
	return 0;
}

unsigned char * backup_buf[4000] = {0};

__attribute__ ((unused)) static void mem_backup(unsigned char * start,unsigned int len)
{
    debug("memory backup\n");
    memcpy(backup_buf,start,len);
}

__attribute__ ((unused)) static void mem_restore(unsigned char * start,unsigned int len)
{
    debug("memory restore\n");
    memcpy(start,backup_buf,len);
}

int run_hex ()
{
	debug("%s\n", __FUNCTION__);

	int ret10 = 0;
	int ret11 = 0;
	int ret20 = 0;
	int ret21 = 0;
	unsigned int result_vde10,result_vde11,result_vde20,result_vde21;
	//unsigned int debug_value = 0;

#if defined(CONFIG_QC_VDE1)
    //mem_backup((unsigned char *)0x2000, 0x2000);
    prepare_binary(hex_vde1_dir0_addr, hex_vde1_dir0_len);//load vde1 pattern
    prepare_ventry();
    flush_dcache_all();
	disable_video_clock(1);
	do_go_video_fw(1);
	rtd_outl(VDE1_CMD_ADDR, VDE1_CMD_START);
	ret10 = ddr_debug_01(1);
	rtd_outl(VDE1_CMD_ADDR, VDE1_CMD_STOP); //stop VDE1
	//printf("0xb800e400:%08x\n", rtd_inl(0xb800e400));
    //printf("0xb800e4f8:%08x\n", rtd_inl(0xb800e4f8));
    //printf("0xb8007004:%08x\n", rtd_inl(0xb8007004));
    //printf("0xb800e404:%08x\n", rtd_inl(0xb800e404));
	disable_video_clock(1);
	invalidate_dcache_range(VDE1_TEST_FLAG_ADDR, VDE1_TEST_FLAG_ADDR+4);
	result_vde10 = SWAPEND32(rtd_inl(VDE1_TEST_FLAG_ADDR));  // VDE1 test pass|fail flag
	printf("result_VDE10 = 0x%x\n", result_vde10);
    //mem_restore((unsigned char *)0x2000, 0x2000);

    //mem_backup((unsigned char *)0x2000, 0x2000);
    prepare_binary(hex_vde1_dir1_addr, hex_vde1_dir1_len);//load vde1 pattern
    prepare_ventry();
    flush_dcache_all();
	do_go_video_fw(1);
	rtd_outl(VDE1_CMD_ADDR, VDE1_CMD_START);
	ret11 = ddr_debug_01(1);
	rtd_outl(VDE1_CMD_ADDR, VDE1_CMD_STOP); //stop VDE1
	//printf("0xb800e400:%08x\n", rtd_inl(0xb800e400));
    //printf("0xb800e4f8:%08x\n", rtd_inl(0xb800e4f8));
    //printf("0xb8007004:%08x\n", rtd_inl(0xb8007004));
    //printf("0xb800e404:%08x\n", rtd_inl(0xb800e404));
	disable_video_clock(1);
	invalidate_dcache_range(VDE1_TEST_FLAG_ADDR, VDE1_TEST_FLAG_ADDR+4);
	result_vde11 = SWAPEND32(rtd_inl(VDE1_TEST_FLAG_ADDR));  // VDE1 test pass|fail flag

	printf("result_VDE11 = 0x%x\n", result_vde11);
    //mem_restore((unsigned char *)0x2000, 0x2000);
#endif

#if defined(CONFIG_QC_VDE2)
    prepare_binary(hex_vde2_dir0_addr, hex_vde2_dir0_len);//load vde2 pattern
    prepare_ventry();
    flush_dcache_all();
	disable_video_clock(2);
	do_go_video_fw(2);
	ret20 = ddr_debug_01(2);
	rtd_outl(VDE2_CMD_ADDR, VDE2_CMD_STOP); //stop VDE2
	printf("0xb8035400:%08x\n", rtd_inl(0xb8035400));
    printf("0xb80354f8:%08x\n", rtd_inl(0xb80354f8));
    printf("0xb8007004:%08x\n", rtd_inl(0xb8007004));
	disable_video_clock(2);
	invalidate_dcache_range(VDE2_TEST_FLAG_ADDR, VDE2_TEST_FLAG_ADDR+4);
	result_vde20 = SWAPEND32(rtd_inl(VDE2_TEST_FLAG_ADDR));  // VDE2 test pass|fail flag
	printf("result_VDE20 = 0x%x\n", result_vde20);

    prepare_binary(hex_vde2_dir1_addr, hex_vde2_dir1_len);//load vde2 pattern
    prepare_ventry();
    flush_dcache_all();
	do_go_video_fw(2);
	ret21 = ddr_debug_01(2);
	rtd_outl(VDE2_CMD_ADDR, VDE2_CMD_STOP); //stop VDE2
	printf("0xb8035400:%08x\n", rtd_inl(0xb8035400));
    printf("0xb80354f8:%08x\n", rtd_inl(0xb80354f8));
    printf("0xb8007004:%08x\n", rtd_inl(0xb8007004));
	disable_video_clock(2);
	invalidate_dcache_range(VDE2_TEST_FLAG_ADDR, VDE2_TEST_FLAG_ADDR+4);
	result_vde21 = SWAPEND32(rtd_inl(VDE2_TEST_FLAG_ADDR));  // VDE2 test pass|fail flag
	printf("result_VDE21 = 0x%x\n", result_vde21);
#endif

// judge pass or fail
#if (defined(CONFIG_QC_VDE1) && defined(CONFIG_QC_VDE2))
	if ((0xbabebabe != result_vde10) || (0xbabebabe != result_vde11) ||  (0xbabebabe != result_vde20)
                       || (0xbabebabe != result_vde21) || (ret10 <0) || (ret11<0) ||  (ret20 <0) || (ret21<0))
	{
		if((ret10 <0) || (0xbabebabe != result_vde10) || (ret11 <0) || (0xbabebabe != result_vde11))
		{
			printf("VDE1 FAIL\n");
		}
		else
		{
			printf("VDE1 PASS\n");
		}

		if((ret20<0) || 0xbabebabe != result_vde20 || (ret21<0) || 0xbabebabe != result_vde21)
		{
			printf("VDE2 FAIL\n");
		}
		else
		{
			printf("VDE2 PASS\n");
		}
		return -1;
	}
	else
	{
		printf("VDE1 and VDE2  PASS\n");
		return 0;
	}

#else // VDE1 or VDE2

#if defined(CONFIG_QC_VDE1) // only VDE1
	if((ret10 <0) || (0xbabebabe != result_vde10) || (ret11 <0) || (0xbabebabe != result_vde11))
	{
		printf("VDE1 FAIL\n");
		return -1;
	}
	else
	{
		printf("VDE1 PASS\n");
		return 0;
	}
#endif // #if defined(CONFIG_QC_VDE1)

#if defined(CONFIG_QC_VDE2) // only VDE2
	if((ret20<0) || 0xbabebabe != result_vde20 || (ret21<0) || 0xbabebabe != result_vde21)
	{
		printf("VDE2 FAIL\n");
		return -1;
	}
	else
	{
		printf("VDE2 PASS\n");
		return 0;
	}
#endif // #if defined(CONFIG_QC_VDE2)

#endif // #if (defined(CONFIG_QC_VDE1) && defined(CONFIG_QC_VDE2))
}

#endif

int vde_qc_bin()
{
	int ret = SUCCESS;
	printf("%s\n", __FUNCTION__);
	ret = run_hex();
	return (ret == 0) ? SUCCESS:FAILED;
}


