
/************************************************************************
 *
 *  pinmux.c
 *
 *  pinmux driver for magellan2
 *
 *
 * Copyright (c) 2013 Realtek Semiconductor Corp. All rights reserved.
 *
 ************************************************************************/

/************************************************************************
 *  Include files
 ************************************************************************/
#include "util.h"
#include "mcp.h"
#include "error_type.h"

#include "otp_reg.h"
#include "mis_reg.h"
#include "sys_reg.h"
#include "iso_reg.h"
#include "pin_share_reg.h"

/************************************************************************
 *  Definitions
 ************************************************************************/
//#define ROM_DEBUG

//this is used for realchip simulation
#ifdef FPGA
#undef FPGA
#endif

//this is used for realchip simulation
#ifdef CONFIG_ENV_FPGA
#undef CONFIG_ENV_FPGA
#endif
/************************************************************************
 *  External function prototypes
 ************************************************************************/
extern void enable_uart_tx(void);

/************************************************************************
 *  Function body
 ************************************************************************/

static void set_nand_pin_mux(int type_select)
{
#if defined(ROM_DEBUG) || defined(CONFIG_DEBUG_UART_PRINT)
	prints("\n");
	prints(__FUNCTION__);
#endif

#if !defined(FPGA) && !defined(CONFIG_ENV_FPGA)
	// sync from cckao
	//PAD_GPIO_3		NF_D7		0xB800_0800[7:4]=0x5
	mask_register(0x18000800, 0xffffff0f, 0x00000050);

	//PAD_GPIO_7		NF_D0		0xB800_0804[7:4]=0x5
	//PAD_GPIO_6		NF_ALE		0xB800_0804[15:12]=0x5
	//PAD_GPIO_5		NF_CLE		0xB800_0804[23:20]=0x5
	//PAD_GPIO_4		NF_D6		0xB800_0804[31:28]=0x5
	mask_register(0x18000804, 0x0f0f0f0f, 0x50505050);

	//PAD_GPIO_11		NF_D2		0xB800_0808[7:4]=0x5
	//PAD_GPIO_10		NF_D1		0xB800_0808[15:12]=0x5
	//PAD_GPIO_9			NF_D3		0xB800_0808[23:20]=0x5
	mask_register(0x18000808, 0xff0f0f0f, 0x00505050);

	//PAD_GPIO_15		NF_RD		0xB800_080C[7:4]=0x5
	//PAD_GPIO_14		NF_RDY		0xB800_080C[15:12]=0x5
	//PAD_GPIO_13		NF_D4		0xB800_080C[23:20]=0x5
	//PAD_GPIO_12		NF_D5		0xB800_080C[31:28]=0x5
	mask_register(0x1800080c, 0x0f0f0f0f, 0x50505050);

	//PAD_GPIO_17		NF_CE0		0xB800_0810[23:20]=0x5
	//PAD_GPIO_16		NF_WR		0xB800_0810[31:28]=0x5
	mask_register(0x18000810, 0x0f0fffff, 0x50500000);	
#endif /* end of !defined(FPGA) && !defined(CONFIG_ENV_FPGA) */
}

static void set_spi_pin_mux(void)
{
#if defined(ROM_DEBUG) || defined(CONFIG_DEBUG_UART_PRINT)
	prints("\n");
	prints(__FUNCTION__);
#endif

#if !defined(FPGA) && !defined(CONFIG_ENV_FPGA)
	prints(__FUNCTION__);

	// Sync from cckao
	// SPI_CS_N, PAD_GPIO_87
	// 0xb800_0854[7:4]= 0x01 (spi/spi_nand) 
	mask_register(0x18000854, 0xffffff0f, (1 << 4));
	prints("1A");

	// because of cckao [Generally not use SPI_WP & SPI_HOLD

	//SPI_SCLK, PAD_GPIO_85
	//0xb800_0854[23:20]=0x01 (spi/spi_nand)
	mask_register(0x18000854, 0xff0fffff, (1 << 20));
	prints("2B");

	//SPI_DO, PAD_GPIO_88
	//0xb800_0858[31:28]=0x01 (spi/spi_nand)
	mask_register(0x18000858, 0x0fffffff, (1 << 28));
	prints("3C");

	//SPI_DI, PAD_GPIO_86
	//0xb800_0854[15:12]=0x01 (spi/spi_nand) 
	mask_register(0x18000854, 0xffff0fff, (1 << 12));
	prints("4D");	
#endif /* end of !defined(FPGA) && !defined(CONFIG_ENV_FPGA) */
}

static void set_emmc_pin_mux(int type_select)
{
#if defined(ROM_DEBUG) || defined(CONFIG_DEBUG_UART_PRINT)
	prints("\n");
	prints(__FUNCTION__);
#endif

#if !defined(FPGA) && !defined(CONFIG_ENV_FPGA)
	type_select = type_select;

#if 0	// not used
	unsigned int inverse;
	unsigned int assert_bits;
	unsigned int clk_workaround;

	//printf("\n%s type_select = %d\n", __FUNCTION__, type_select);


	// see Mantis 55208 for this workaround
	clk_workaround = REG32(SYS_GROUP1_CK_EN);
	REG32(SYS_GROUP1_CK_EN) |= (0x1 << 29);

	if (type_select == EMMC_TV_BGA_CHINA) {
		// set GPIO_184_ps to eMMC_D0 , 0x1800_0888[22:18] = 0x02 (eMMC)
		// set GPIO_185_ps to eMMC_D1 , 0x1800_0888[13:9 ] = 0x02 (eMMC)
		inverse = (0x1f << 18) | (0x1f << 9);
		assert_bits = (0x2 << 18) | (0x2 << 9);
		mask_register(GPIO_MUXCFG_34, ~inverse, assert_bits);

		// set GPIO_181_ps to eMMC_D2 , 0x1800_0884[22:18] = 0x02 (eMMC)
		// set GPIO_182_ps to eMMC_D7 , 0x1800_0884[13:9 ] = 0x02 (eMMC)
		mask_register(GPIO_MUXCFG_33, ~inverse, assert_bits);

		// set GPIO_153_ps to eMMC_D3 , 0x1800_08a0[22:18] = 0x02 (eMMC)
		// set GPIO_152_ps to eMMC_D4 , 0x1800_08a0[31:27] = 0x02 (eMMC)
		inverse = (0x1f << 27) | (0x1f << 18);
		assert_bits = (0x2 << 27) | (0x2 << 18);
		mask_register(GPIO_MUXCFG_40, ~inverse, assert_bits);

		// set GPIO_188_ps to eMMC_D5 , 0x1800_088c[13:9 ] = 0x02 (eMMC)
		inverse = (0x1f << 9);
		assert_bits = (0x2 << 9);
		mask_register(GPIO_MUXCFG_35, ~inverse, assert_bits);

		// set GPIO_169_ps to eMMC_D6 , 0x1800_0874[22:18] = 0x02 (eMMC)
		inverse = (0x1f << 18);
		assert_bits = (0x2 << 18);
		mask_register(GPIO_MUXCFG_29, ~inverse, assert_bits);
	}
	else {
		// set GPIO_44_ps to eMMC_D0 , 0x1800_0834[13:9 ] = 0x02 (eMMC)
		inverse = (0x1f << 9);
		assert_bits = (0x2 << 9);
		mask_register(GPIO_MUXCFG_13, ~inverse, assert_bits);

		// set GPIO_45_ps to eMMC_D1 , 0x1800_0838[31:27] = 0x02 (eMMC)
		// set GPIO_46_ps to eMMC_D2 , 0x1800_0838[22:18] = 0x02 (eMMC)
		// set GPIO_47_ps to eMMC_D3 , 0x1800_0838[13:9 ] = 0x02 (eMMC)
		inverse = (0x1f << 27) | (0x1f << 18) | (0x1f << 9);
		assert_bits = (0x2 << 27) | (0x2 << 18) | (0x2 << 9);
		mask_register(GPIO_MUXCFG_14, ~inverse, assert_bits);

		// set GPIO_48_ps to eMMC_D4 , 0x1800_083c[31:27] = 0x02 (eMMC)
		// set GPIO_49_ps to eMMC_D5 , 0x1800_083c[22:18] = 0x02 (eMMC)
		// set GPIO_50_ps to eMMC_D6 , 0x1800_083c[13:9 ] = 0x02 (eMMC)
		mask_register(GPIO_MUXCFG_15, ~inverse, assert_bits);

		// set GPIO_51_ps to eMMC_D7 , 0x1800_0840[31:27] = 0x02 (eMMC)
		inverse = (0x1f << 27);
		assert_bits = (0x2 << 27);
		mask_register(GPIO_MUXCFG_16, ~inverse, assert_bits);
	}

	if (type_select == EMMC_TV_BGA_CHINA) {
		// set GPIO_150_ps to eMMC_CMD, 0x1800_0958[22:18] = 0x02 (eMMC)
		inverse = (0x1f << 18);
		assert_bits = (0x2 << 18);
		mask_register(GPIO_MUXCFG_50, ~inverse, assert_bits);
	}
	else if (type_select == EMMC_BOX176_LQFP_CASE_8) {
		// set ST_GPIO_42_ps to eMMC_CMD, 0x1806_0228[13:9 ] = 0x01 (eMMC)
		inverse = (0x1f << 9);
		assert_bits = (0x1 << 9);
		mask_register(GPIO_ST_MUXCFG_9, ~inverse, assert_bits);
	}
	else {
		// set GPIO_56_ps to eMMC_CMD, 0x1800_0844[13:9 ] = 0x02 (eMMC)
		inverse = (0x1f << 9);
		assert_bits = (0x2 << 9);
		mask_register(GPIO_MUXCFG_17, ~inverse, assert_bits);
	}

	switch(type_select) {
		case EMMC_TV176_LQFP:	// same as EMMC_TV256_LQFP_CASE_2_3_4_5, EMMC_TV_BGA
			// set ST_GPIO_12_ps to eMMC_CLK, 0x1806_0248[31:27] = 0x01 (eMMC)
			inverse = (0x1f << 27);
			assert_bits = (0x1 << 27);
			mask_register(GPIO_ST_MUXCFG_17, ~inverse, assert_bits);

			// set BB_DIO_3_ps to eMMC_RST_N, 0x1800_08f8[31:27] = 0x1f (gpio)
			inverse = (0x1f << 27);
			assert_bits = (0x1f << 27);
			mask_register(BB_USB_CFG_1, ~inverse, assert_bits);
			break;

		case EMMC_TV256_LQFP_CASE_1:
			// set GPIO_52_ps to eMMC_CLK, 0x1800_0840[22:18] = 0x02 (eMMC)
			inverse = (0x1f << 18);
			assert_bits = (0x2 << 18);
			mask_register(GPIO_MUXCFG_16, ~inverse, assert_bits);

			// set GPIO_55_ps to eMMC_RST_N, 0x1800_0844[22:18] = 0x02 (gpio)
			mask_register(GPIO_MUXCFG_17, ~inverse, assert_bits);
			break;

		case EMMC_BOX176_LQFP_CASE_2:
			// set GPIO_143_ps to eMMC_CLK, 0x1800_0848[31:27] = 0x02 (eMMC)
			inverse = (0x1f << 27);
			assert_bits = (0x2 << 27);
			mask_register(GPIO_MUXCFG_18, ~inverse, assert_bits);

			// set GPIO_40_ps to eMMC_RST_N, 0x1800_0830[22:18] = 0x1f (gpio)
			inverse = (0x1f << 18);
			assert_bits = (0x1f << 18);
			mask_register(GPIO_MUXCFG_12, ~inverse, assert_bits);
			break;

		case EMMC_BOX176_LQFP_CASE_4_5:
			// set ST_GPIO_12_ps to eMMC_CLK, 0x1806_0248[31:27] = 0x01 (eMMC)
			inverse = (0x1f << 27);
			assert_bits = (0x1 << 27);
			mask_register(GPIO_ST_MUXCFG_17, ~inverse, assert_bits);

			// set GPIO_97_ps to eMMC_RST_N, 0x1800_0860[13:9 ] = 0x1f (gpio)
			inverse = (0x1f << 9);
			assert_bits = (0x1f << 9);
			mask_register(GPIO_MUXCFG_24, ~inverse, assert_bits);
			break;

		case EMMC_BOX176_LQFP_CASE_7:
			// set GPIO_39_ps to eMMC_CLK, 0x1800_0830[31:27] = 0x02 (eMMC)
			inverse = (0x1f << 27);
			assert_bits = (0x2 << 27);
			mask_register(GPIO_MUXCFG_12, ~inverse, assert_bits);

			// set GPIO_42_ps to eMMC_RST_N, 0x1800_0834[31:27] = 0x1f (gpio)
			assert_bits = (0x1f << 27);
			mask_register(GPIO_MUXCFG_13, ~inverse, assert_bits);
			break;

		case EMMC_BOX176_LQFP_CASE_8:
			// set ST_GPIO_43_ps to eMMC_CLK, 0x1806_0228[22:18] = 0x01 (eMMC)
			// set ST_GPIO_44_ps to eMMC_RST_N, 0x1806_0228[31:27] = 0x1f (gpio)
			inverse = (0x1f << 27) | (0x1f << 18);
			assert_bits = (0x1f << 27) | (0x1 << 18);
			mask_register(GPIO_ST_MUXCFG_9, ~inverse, assert_bits);
			break;

		case EMMC_TV_BGA_LGE:
			// set GPIO_85_ps to eMMC_CLK, 0x1800_0850[13:9 ] = 0x02 (eMMC)
			inverse = (0x1f << 9);
			assert_bits = (0x2 << 9);
			mask_register(GPIO_MUXCFG_20, ~inverse, assert_bits);

			// set BB_DIO_3_ps to eMMC_RST_N, 0x1800_08f8[31:27] = 0x1f (eMMC_RST)
			inverse = (0x1f << 27);
			assert_bits = (0x1f << 27);
			mask_register(BB_USB_CFG_1, ~inverse, assert_bits);
			break;

		case EMMC_TV_BGA_CHINA:
			// set GPIO_157_ps to eMMC_CLK, 0x1800_08a4[13:9 ] = 0x02 (eMMC)
			inverse = (0x1f << 9);
			assert_bits = (0x2 << 9);
			mask_register(GPIO_MUXCFG_41, ~inverse, assert_bits);

			// set GPIO_151_ps to eMMC_RST_N, 0x1800_089c[13:9 ] = 0x1f (eMMC)
			inverse = (0x1f << 9);
			assert_bits = (0x1f << 9);
			mask_register(GPIO_MUXCFG_39, ~inverse, assert_bits);
			break;

		default:	// should not happened
			break;
	}

	// set input mux in 0x1800_0914
	switch(type_select) {
		case EMMC_TV176_LQFP:
		case EMMC_TV256_LQFP_CASE_1:
		case EMMC_TV_BGA_LGE:
		case EMMC_BOX176_LQFP_CASE_2:
		case EMMC_BOX176_LQFP_CASE_4_5:
		case EMMC_BOX176_LQFP_CASE_7:
			// [30:28] = 0x1
			// [27:26], [25:24], [23:22], [21:20], [19:18], [16:15], [14:13], [12:11] = 0x0
			assert_bits = 0x10000000;
			break;
		case EMMC_BOX176_LQFP_CASE_8:
			// [30:28] = 0x2
			// [27:26], [25:24], [23:22], [21:20], [19:18], [16:15], [14:13], [12:11] = 0x0
			assert_bits = 0x20000000;
			break;
		case EMMC_TV_BGA_CHINA:
			// [30:28] = 0x6
			// [27:26], [25:24], [23:22], [21:20], [19:18], [16:15], [14:13], [12:11] = 0x2
			assert_bits = 0x6aa95000;
			break;
		default:
			assert_bits = 0;
			break;
	}
	inverse = 0x7ffdf800;
	mask_register(PIN_MUX_CTRL6, ~inverse, assert_bits);

	REG32(SYS_GROUP1_CK_EN) = clk_workaround;
#endif	
#endif /* end of !defined(FPGA) && !defined(CONFIG_ENV_FPGA) */
}

/*
 * TV176/others check from GPIO
 *	0: TV176_LQFP
 *	1: others package
 */
int get_tv176_others_check_value(void)
{
	int res = 0;
	unsigned int pin_mux;
	unsigned int gpio_dir;

	if ((REG32(0x18060000) & 0xffff) <= 0x1) {
		pin_mux = REG32(GPIO_MUXCFG_45);	// save pin mux
		gpio_dir = REG32(MIS_GP5DIR);		// save GPIO direction
		REG32(GPIO_MUXCFG_45) |= (0x1f << 27);	// set gpio_167_ps = 0x1f
		REG32(MIS_GP5DIR) &= ~(0x1 << 7);	// set GPIO 167 as input pin, [7] = 0
		res = (REG32(MIS_GP5DATI) >> 7) & 0x1;	// get from GPIO 167
		REG32(MIS_GP5DIR) = gpio_dir;		// restore GPIO direction
		REG32(GPIO_MUXCFG_45) = pin_mux;	// restore pin mux
		//printf("\n%s res = %d\n", __FUNCTION__, res);
		return res;
		}
	else {
		return ((REG32(0x180171a0)>>1)&1);
	}
}

/*
 * get different NAND/eMMC type from GPIO
 */
int get_nand_emmc_type_select(const int flash_type)
{
#if defined(ROM_DEBUG) || defined(CONFIG_DEBUG_UART_PRINT)
	prints("\n");
	prints(__FUNCTION__);
#endif

	int res;
	unsigned int pin_mux;
	unsigned int gpio_dir;

	if (get_tv176_others_check_value() == 0) {
		if (flash_type == NAND_FLASH_BOOT)
			return NAND_TV176_LQFP;
		else
			return EMMC_TV176_LQFP;
	}

	/*
	 * from "Sirius ROM CODE.ppt"
	 *-----------------------------------------
	 * BOOT_TYPE_bit0 from PAD_GPIO_20 as bit0
	 * BOOT_TYPE_bit1 from PAD_GPIO_21 as bit1
	 * BOOT_TYPE_bit2 from PAD_GPIO_22 as bit2
	 *-----------------------------------------
	 * 000 -> EMMC: TV256_LQFP Case 2,3,4,5
	 *              TV_BGA
	 * 001 -> NAND: TV256_LQFP Case 2,3,5
	 *              TV_BGA
	 *        EMMC: TV256_LQFP Case 1
	 * 010 -> NAND: BOX176_LQFP Case 1,4,5,6,8
	 *        EMMC: BOX176_LQFP Case 2
	 * 011 -> EMMC: BOX176_LQFP Case 4,5
	 * 100 -> EMMC: BOX176_LQFP Case 7
	 * 101 -> EMMC: BOX176_LQFP Case 8
	 * 110 -> EMMC: TV_BGA_LGE
	 * 111 -> EMMC: TV_BGA_CHINA
	 */

	/* get boot_type (from GPIO) */
	pin_mux = REG32(GPIO_MUXCFG_7);	// save pin mux
	gpio_dir = REG32(MIS_GP0DIR);	// save GPIO direction
	REG32(GPIO_MUXCFG_7) = pin_mux | (0x1f << 27) | (0x1f << 18) | (0x1f << 9);	// set gpio_20_ps = 0x1f, gpio_21_ps = 0x1f, gpio_22_ps = 0x1f
	REG32(MIS_GP0DIR) &= ~(0x7 << 22);	// set GPIO 20/21/22 as input pin, [22:20] = 0
	res = (REG32(MIS_GP0DATI) >> 20) & 0x7;	// get boot_type type from GPIO [22:20]
	REG32(MIS_GP0DIR) = gpio_dir;	// restore GPIO direction
	REG32(GPIO_MUXCFG_7) = pin_mux;	// restore pin mux

	printf("\n%s res = %d\n", __FUNCTION__, res);
	return res;
}

int set_pin_mux_force(const int flash_type)
{
	int res;

#ifdef ROM_DEBUG
	prints("\n");
	prints(__FUNCTION__);
#endif
	res = 0;

	switch (flash_type) {
		case SPI_FLASH_BOOT:
		case SPI_NAND_FLASH_BOOT:
			set_spi_pin_mux();
			break;

		case NAND_FLASH_BOOT:
			set_nand_pin_mux(get_nand_emmc_type_select(flash_type));
			printf("\nNAND boot\n");
			break;

		case EMMC_BOOT:
			printf("\nEMMC boot\n");
#if 0			
#if defined(Config_Package_type_EMMC_TV176_LQFP)
			printf("\nEMMC_TV176_LQFP\n");
			//set_emmc_pin_mux(EMMC_TV176_LQFP);
			REG32(0x18000914) = (REG32(0x18000914) & 0x000000ff) |0x10000000;	// EMMC_Source control reg(176/256 QFP)
			REG32(0x18060248) = (REG32(0x18060248) & 0x00ffffff) |0x08000000;	// EMMC_CLK st_gpio_12
			REG32(0x18000844) = (REG32(0x18000844) & 0xffffc000) |0x00000400;	// EMMC_CMD gpio_56
			REG32(0x18000834) = (REG32(0x18000834) & 0xffffc000) |0x00000400;	// EMMC_DAT0 gpio_44
			REG32(0x18000838) = (REG32(0x18000838) & 0x00000000) |0x10080400;	// EMMC_DAT1/2/3 gpio_45/46/47
			REG32(0x1800083c) = (REG32(0x1800083c) & 0x00000000) |0x10080400;	// EMMC_DAT4/5/6 gpio_48/49/50
			REG32(0x18000840) = (REG32(0x18000840) & 0x007fffff) |0x10000000;	// EMMC_DAT7 gpio_51
			REG32(0x180008F8) = (REG32(0x180008F8) & 0xffc1ffff) |0x003e0000;	//EMMC_RST_N 0x1800_08F8[21:17]=0x1F	bb_dio3_ps	GPIO196
#elif defined(Config_Package_type_EMMC_TV_BGA_CA)
			printf("\nEMMC_TV_BGA_CA\n");
			set_emmc_pin_mux(EMMC_TV_BGA_CHINA);
#else
			printf("\n[Warning] no package type defined\n");
			printf("\nchange to default EMMC_TV176_LQFP\n");
			REG32(0x18000914) = (REG32(0x18000914) & 0x000000ff) |0x10000000;	// EMMC_Source control reg(176/256 QFP)
			REG32(0x18060248) = (REG32(0x18060248) & 0x00ffffff) |0x08000000;	// EMMC_CLK st_gpio_12
			REG32(0x18000844) = (REG32(0x18000844) & 0xffffc000) |0x00000400;	// EMMC_CMD gpio_56
			REG32(0x18000834) = (REG32(0x18000834) & 0xffffc000) |0x00000400;	// EMMC_DAT0 gpio_44
			REG32(0x18000838) = (REG32(0x18000838) & 0x00000000) |0x10080400;	// EMMC_DAT1/2/3 gpio_45/46/47
			REG32(0x1800083c) = (REG32(0x1800083c) & 0x00000000) |0x10080400;	// EMMC_DAT4/5/6 gpio_48/49/50
			REG32(0x18000840) = (REG32(0x18000840) & 0x007fffff) |0x10000000;	// EMMC_DAT7 gpio_51
			REG32(0x180008F8) = (REG32(0x180008F8) & 0xffc1ffff) |0x003e0000;	//EMMC_RST_N 0x1800_08F8[21:17]=0x1F	bb_dio3_ps	GPIO196
#endif
#endif
			break;

		case UNKNOWN_BOOT:
		default:
			res = ERR_UNKNOWN_TYPE;
			break;
	}

	return res;
}

int set_pin_mux_auto(const int flash_type)
{
	int res;

#ifdef ROM_DEBUG
	prints("\n");
	prints(__FUNCTION__);
#endif
	res = 0;

	switch (flash_type) {
		case SPI_FLASH_BOOT:
		case SPI_NAND_FLASH_BOOT:
			set_spi_pin_mux();
			break;
		case NAND_FLASH_BOOT:
			set_nand_pin_mux(get_nand_emmc_type_select(flash_type));
			break;

		case EMMC_BOOT:
			set_emmc_pin_mux(get_nand_emmc_type_select(flash_type));
			break;

		case UNKNOWN_BOOT:
		default:
			res = ERR_UNKNOWN_TYPE;
			break;
	}

	return res;
}

int set_pin_mux(const int flash_type)
{
	int res = 0;

#ifdef ROM_DEBUG
	prints("\n");
	prints(__FUNCTION__);
#endif

#if !defined(FPGA) && !defined(CONFIG_ENV_FPGA)
	unsigned int chip_id;
//	chip_id = REG32(0x18060000);
	chip_id = 0;

	// check chip version
	switch (chip_id & 0xffff) {
		case 0x0001:
			printf("\npin mux auto\n");
			res = set_pin_mux_auto(flash_type);
			break;
		case 0x0000:
		default:
			printf("\npin mux force\n");
			res = set_pin_mux_force(flash_type);
			break;
	}
#endif

	return res;
}

void set_uart_rx_pin_mux(void)
{
	//(see Main_Sirius_PinMux.doc, PMM_Sirius_PinMux.doc)
#if !defined(FPGA) && !defined(CONFIG_ENV_FPGA)
	/* set UART0_rxdi_in_sel pin mux */
	// PAD_ST_GPIO_12
	// uart0_rxdi_src7,<I>
	// 0xB806_0230[14:12]=0x7(switch to src7) 
	// 0xB806_020C[31:28]=0x9 (for UART0_RX)
	mask_register(0x18060230, ~(0x7 << 12), (0x7 << 12));
	mask_register(0x1806020C, ~(0xf << 28), (0x9 << 28));
	
	/* set uart1_rxdi_in_sel pin mux */
	// PAD_ST_GPIO_27 
	// uart1_rxdi_src5,<I>
	// 0xB806_0230[10:8]=0x5 (switch to src5 ) 
	// 0xB806_0218[7:4]=0x7 (for UART1_RX)
	mask_register(0x18060230, ~(0x7 << 8), (0x5 << 8));
	mask_register(0x18060218, ~(0xf << 7), (0x7 << 4));

#if defined(ROM_DEBUG) || defined(CONFIG_DEBUG_UART_PRINT)
	//enable_uart_tx();	// for debug use
#endif

#endif /* end of if !defined(FPGA) && !defined(#ifndef CONFIG_ENV_FPGA) */
}

/*
 * This function switch UART TX to GPI,
 * main purpose is to detect UART TX toggled by VGA DDC in check_recovery() function
 *
 */
void set_recovery_uart(void)
{
#if !defined(FPGA) && !defined(CONFIG_ENV_FPGA)
	// UART0 TX: PAD_ST_GPIO_13 
	//	0xB806_020C[23:20]=0xF (for GPIO)
	//	0xB806_1100[13]=0x0/1 (direction:input/output)
	//	0xB806_1108[13] (output data)
	//	0xB806_1110[13] (input data)
	REG32(0x1806020c) |= (0xf << 20);
	REG32(0x18061100) &= ~(1 << 13);

	// UART1 TX: PAD_ST_GPIO_28 
	//	0xB806_021C[31:28]=0xF (for GPIO)
	//	0xB806_1100[28]=0x0/1 (direction:input/output)
	//	0xB806_1108[28] (output data)
	//	0xB806_1110[28] (input data)
	REG32(0x1806021c) |= (0xf << 28);
	REG32(0x18061100) &= ~(1 << 28);
	
	/* clear GPIO ASSERT & DEASSERT Pending BIT (write 1 to clear) */
	REG32(0x18061130) = (1 << 14) | (1 << 29);
	REG32(0x18061138) = (1 << 14) | (1 << 29);

#endif /* end of #if !defined(FPGA) && !defined(CONFIG_ENV_FPGA) */

	REG32(U0IIR_FCR) = 0x1;
	REG32(U0IIR_FCR) = 0x7;	//set reciver tigger level to 0
#if !defined(FPGA) && !defined(CONFIG_ENV_FPGA)
	REG32(U1IIR_FCR) = 0x1;
	REG32(U1IIR_FCR) = 0x7; //set reciver tigger level to 0
#endif
}
