#include "hdmi_common.h"
#if 0
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <asm/uaccess.h>
#include <linux/sched.h>
#include <linux/kthread.h>
#include <linux/string.h>
#endif
#include "hdmi_reg.h"
#include "hdmi_audio.h"

#if 0
#include "hdmi_info_packet.h"
#endif
/**********************************************************************************************
*
*   Variables
*
**********************************************************************************************/
extern HDMI_PORT_INFO_T hdmi_rx[HDMI_PORT_TOTAL_NUM];
extern unsigned char current_port;

static unsigned char audio_thread_run[HDMI_PORT_TOTAL_NUM] = {0};

#if !defined(__QC_VERIFY_COMMON_H__)
static bool audio_thread_flag = false;
static struct task_struct *audio_task;
#endif
/**********************************************************************************************
*
*   Audio Thread
*
**********************************************************************************************/

#if !defined(__QC_VERIFY_COMMON_H__)
static int hdmi_audio_thread(void* arg)
{
    while (!kthread_should_stop() && audio_thread_flag == true)
    {
        newbase_hdmi_audio_detect();
        msleep(10); // sleep 10 ms
    }

    return 0;
}

void newbase_hdmi_audio_thread_enable(unsigned char create)
{
    if (create)
    {
        if (audio_thread_flag!=true)
        {
            audio_task = kthread_create(hdmi_audio_thread, NULL, "audio_task");

            newbase_hdmi_audio_reset_status();

            if (audio_task)
            {
                audio_thread_flag = true;
                wake_up_process(audio_task);
                HDMI_PRINTF("hdmi audio thread started\n");
            }
            else
            {
                HDMI_EMG("unable to start audio thread \n");
            }
        }
    }
    else
    {
        if (audio_thread_flag == true)
        {
            if (kthread_stop(audio_task)==0)
                HDMI_PRINTF("hdmi audio thread stopped\n");
            else
                HDMI_EMG("hdmi audio thread stopped error\n");

            audio_thread_flag = false;
        }
    }
}

#endif
unsigned char lib_hdmi_audio_get_thread_run(unsigned char nport)
{
    return audio_thread_run[nport];
}


/**********************************************************************************************
*
*   Internal Functin
*
**********************************************************************************************/

void lib_hdmi_audio_set_thread_run(unsigned char nport, unsigned char uc_run)
{
//  HDMI_PRINTF("lib_hdmi_audio_set_thread_run[%d]=%d\n", nport, uc_run);
    audio_thread_run[nport] = uc_run;
}


void newbase_hdmi_audio_var_init(unsigned char nport)
{
    hdmi_rx[nport].audio_t.coding_type= 0;
    hdmi_rx[nport].audio_t.sr_mismatch_cnt = 0;
    hdmi_rx[nport].audio_t.track_mode_tgl = 0;
}



unsigned char newbase_hdmi_audio_detect(void)
{
    unsigned char i;
    unsigned char port = current_port;

    if (hdmi_in(DBG_FW_ADDR)&DEBUG_29_BYPASS_AUDIO_DET)
        return FALSE;    

    if(_FALSE == lib_hdmi_audio_get_thread_run(port))
        return FALSE;

    if (((GET_H_VIDEO_FSM(port) != MAIN_FSM_HDMI_DISPLAY_ON) && 
        (GET_H_VIDEO_FSM(port) != MAIN_FSM_HDMI_VIDEO_READY)) ||
        (lib_hdmi_get_avmute(port) && !lib_hdmi_get_bch_2bit_error(port)) ||
        (hdmi_rx[port].timing_t.tmds_clk_b < VALID_MIN_CLOCK))
    {
        lib_hdmi_audio_output(port, 0);
        SET_H_AUDIO_FSM(port, AUDIO_FSM_AUDIO_START);
        return FALSE;
    }

    switch (GET_H_AUDIO_FSM(port))
    {
    case AUDIO_FSM_AUDIO_START:
        //HDMI_PRINTF("AUDIO_FSM_AUDIO_START\n");
        for (i=0; i<HDMI_PORT_TOTAL_NUM; i++)
        {
            SET_H_AUDIO_FSM(i, AUDIO_FSM_AUDIO_START);
            lib_hdmi_audio_init(i);
        }

        hdmi_rx[port].audio_t.sr_mismatch_cnt = 0;

        if (!newbase_hdmi_audio_wait_sample(port)) {
            //HDMI_PRINTF("No audio sample(p%d)\n", port);
            break; //need to wait for audio sample packet
        }
        
        SET_H_AUDIO_FSM(port, AUDIO_FSM_AUDIO_START_OUT);
        break;            
        
    case AUDIO_FSM_AUDIO_START_OUT:
        HDMI_PRINTF("AUDIO_FSM_AUDIO_START_OUT[%d]\n", port);        
        lib_hdmi_audio_output(port, 1);
        SET_H_AUDIO_FSM(port, AUDIO_FSM_AUDIO_CHECK);
        break;
        
    case AUDIO_FSM_AUDIO_CHECK:
        
        if (lib_hdmi_get_avmute(port))
        {
            // Humm... do we have to stop audio here ???
            lib_hdmi_audio_output(port, 0);
            SET_H_AUDIO_FSM(port, AUDIO_FSM_AUDIO_START);
            //HDMI_PRINTF("Audio Output Disable cause by AVMCR output disable:%x\n",hdmi_in(HDMI_HDMI_AVMCR_reg));
        }

        break;
        
    default:
        SET_H_AUDIO_FSM(port, AUDIO_FSM_AUDIO_START);
        return FALSE;;
    }

    return TRUE;
}

void newbase_hdmi_audio_close(unsigned char port)
{
    lib_hdmi_audio_output(port, 0);
    lib_hdmi_audio_set_thread_run(port, _FALSE);
    SET_H_AUDIO_FSM(port, AUDIO_FSM_AUDIO_START);
}

void newbase_hdmi_audio_reset_status(void)
{
    unsigned char i;
    for (i=0; i<HDMI_PORT_TOTAL_NUM; i++)
        SET_H_AUDIO_FSM(i, AUDIO_FSM_AUDIO_START);
}

unsigned char newbase_hdmi_audio_get_current_status(void)
{
    return GET_H_AUDIO_FSM(current_port);
}

#if 0
unsigned char newbase_hdmi_audio_wait_acr(unsigned char port)
{   //Audio Clock Regeneration (N/CTS)
    unsigned char i, bch_err;

    lib_hdmi_set_rsv_packet_type(port, 0, 0x01);
    lib_hdmi_clear_rsv_packet0_status(port);

    for (i=0; i<5; i++) 
    {
        if (lib_hdmi_read_rsv0_packet(port, &bch_err, 1)==0) 
        {
//          HDMI_PRINTF("bch status:%x\n",bch_err);
            return ((bch_err & 0x1f) == 0); //acr no checksum bit, do not check
        }
        msleep(10);
    }

    return FALSE;
}

unsigned char newbase_hdmi_audio_wait_sample(unsigned char port)
{   //Audio Sample (L-PCM and IEC 61937 compressed formats)
    unsigned char i, buf[2];

    lib_hdmi_set_rsv_packet_type(port, 0, 0x02);
    lib_hdmi_clear_rsv_packet0_status(port);

    for (i=0; i<5; i++)
    {
        if (lib_hdmi_read_rsv0_packet(port, buf, 2)==0)  // got audio sample
            return TRUE;

        msleep(10);
    }

    return FALSE;
}
#endif


void lib_hdmi_audio_init(unsigned char nport)
{
    lib_hdmi_audio_output(nport, 0);
}


void lib_hdmi_audio_cts_bound(unsigned char nport, unsigned int clk)
{
    unsigned long cts_up, cts_low;

    if (clk > 5600) { //6G timing
        cts_up = 0xfffff ;//990000;
        cts_low = 445500 *7/8 ;
    } else if (clk > 2750) { //   3G timing
        cts_up = 421875 * 9/8 ; //990000;
        cts_low = 222750 *7/8 ;
    } else if (clk > 1380) { //148M
        cts_up = 421875 * 9/8  ;//990000;
        cts_low = 140625 *7/8 ;
    } else if (clk > 660) {
        cts_up = 234375 * 9/8  ;//990000;
        cts_low = 74250 *7/8 ;
    } else {
        cts_up = 60060 * 9/8  ;//990000;
        cts_low = 25200 *7/8 ;
    }

    hdmi_mask(HDMI_AUDIO_CTS_UP_BOUND_reg,~(HDMI_AUDIO_CTS_UP_BOUND_cts_up_bound_mask),HDMI_AUDIO_CTS_UP_BOUND_cts_up_bound(cts_up));
    hdmi_mask(HDMI_AUDIO_CTS_LOW_BOUND_reg,~HDMI_AUDIO_CTS_LOW_BOUND_cts_low_bound_mask,HDMI_AUDIO_CTS_LOW_BOUND_cts_low_bound(cts_low));     //CTS low boundary set 20000
}


void lib_hdmi_audio_n_bound(unsigned char nport, unsigned int freq)
{
    int N_up, N_low;

    if (freq>190000) { //192k
        N_up = 46592 *9/8 ;//;
        N_low = 20480 *7/8 ;
    } else if (freq >170000) { //   176.4
        N_up = 71344 * 9/8 ; //990000;
        N_low = 17836 *7/8 ;
    } else if (freq >80000) { //   96 k 88.2k
        N_up = 35672 * 9/8  ;//990000;
        N_low = 8918 *7/8 ;
    } else { //32k  44.1k  48k
        N_up = 17836 * 9/8  ;//990000;
        N_low = 3072 *7/8 ;
    }

    hdmi_mask(HDMI_AUDIO_N_UP_BOUND_reg,~(HDMI_AUDIO_N_UP_BOUND_n_up_bound_mask),HDMI_AUDIO_N_UP_BOUND_n_up_bound(N_up));
    hdmi_mask(HDMI_AUDIO_N_LOW_BOUND_reg,~HDMI_AUDIO_N_LOW_BOUND_n_low_bound_mask,HDMI_AUDIO_N_LOW_BOUND_n_low_bound(N_low));     //CTS low boundary set 20000
    HDMI_PRINTF("freq = %d Audio_N_Bound H = %d , L =%d\n", freq, N_up, N_low);
}


unsigned char lib_hdmi_audio_is_fifo_overflow(unsigned char nport)
{
    return HDMI_HDMI_AUDSR_get_aud_fifof(hdmi_in(HDMI_HDMI_AUDSR_reg));
}


void lib_hdmi_audio_wclr_fifo_overflow(unsigned char nport)
{
    hdmi_out(HDMI_HDMI_AUDSR_reg, HDMI_HDMI_AUDSR_aud_fifof_mask);
}


unsigned char lib_hdmi_audio_is_output(unsigned char nport)
{
    return HDMI_HDMI_AUDCR_get_aud_en(hdmi_in(HDMI_HDMI_AUDCR_reg));
}


void lib_hdmi_audio_output(unsigned char nport, unsigned char on)
{
    if (on)
    {      
        if (!lib_hdmi_audio_is_output(nport))  
        {
            hdmi_out(HDMI_HDMI_AUDCR_reg, HDMI_HDMI_AUDCR_aud_en(1));
            hdmi_out(HDMI_HDMI_AUDSR_reg, HDMI_HDMI_AUDSR_aud_fifof_mask);  // clear audio fifo over flow status
            HDMI_INFO("Audio output start\n");
        }
    }
    else
    {
        if (lib_hdmi_audio_is_output(nport))  
        {
            hdmi_out(HDMI_HDMI_AUDCR_reg, HDMI_HDMI_AUDCR_aud_en(0));
            hdmi_out(HDMI_HDMI_AUDSR_reg, HDMI_HDMI_AUDSR_aud_fifof_mask);  // clear audio fifo over flow status
            HDMI_INFO("Audio output stopped\n");
        }
    }    
}


unsigned char lib_hdmi_audio_get_hbr_mode(unsigned char nport)
{
    return HDMI_High_Bit_Rate_Audio_Packet_get_hbr_audio_mode(hdmi_in(HDMI_High_Bit_Rate_Audio_Packet_reg));
}


void lib_hdmi_audio_dbg_reg_dump(unsigned char nport)
{
    HDMI_EMG("###################Audio Register Start ###################\n");
    
    HDMI_EMG("ASR0=%x ASR1=%x \n",hdmi_in(HDMI_HDMI_ASR0_reg),hdmi_in(HDMI_HDMI_ASR1_reg));
    HDMI_EMG("ACRCR=%x ACRSR0=%x ACRSR1=%x \n",hdmi_in(HDMI_HDMI_ACRCR_reg),hdmi_in(HDMI_HDMI_ACRSR0_reg),hdmi_in(HDMI_HDMI_ACRSR1_reg));
    HDMI_EMG("CTS_UP_BOUND=%x  CTS_LOW_BOUND=%x \n",hdmi_in(HDMI_AUDIO_CTS_UP_BOUND_reg),hdmi_in(HDMI_AUDIO_CTS_LOW_BOUND_reg));
    HDMI_EMG("N_UP_BOUND=%x  N_LOW_BOUND=%x \n",hdmi_in(HDMI_AUDIO_N_UP_BOUND_reg),hdmi_in(HDMI_AUDIO_N_LOW_BOUND_reg));
    HDMI_EMG("INTCR=%x SR_CHG=%x \n",hdmi_in(HDMI_HDMI_INTCR_reg),hdmi_in(HDMI_Audio_Sample_Rate_Change_IRQ_reg));

    HDMI_EMG("###################Audio Register End###################\n");
}
