#include "hdmi_common.h"
#if 0
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/uaccess.h>
#include <linux/sched.h>
#include <linux/kthread.h>
#endif
#include "hdmi_reg.h"
#include "hdmi_scdc.h"

extern HDMI_PORT_INFO_T hdmi_rx[HDMI_PORT_TOTAL_NUM];

/*---------------------------------------------------
 * Func : lib_hdmi_scdc_enable
 *
 * Desc : enable SCDC
 *
 * Para : nport : HDMI port number 
 *        enable : 0 : disabled, others : enabled
 *   
 * Retn : N/A
 *--------------------------------------------------*/
void lib_hdmi_scdc_enable(unsigned char nport, unsigned char enable)
{
    hdmi_mask(HDMI_STB_SCDC_CR_reg, ~HDMI_STB_SCDC_CR_scdc_en_mask, (enable) ? HDMI_STB_SCDC_CR_scdc_en_mask : 0);
}


/*---------------------------------------------------
 * Func : lib_hdmi_scdc_reset
 *
 * Desc : set reset bit of scdc hw
 *
 * Para : nport : HDMI port number 
 *        rst : reset scdc
 *   
 * Retn : N/A
 *--------------------------------------------------*/
void lib_hdmi_scdc_reset(unsigned char nport, unsigned char rst)
{
    hdmi_mask(HDMI_STB_SCDC_CR_reg, ~HDMI_STB_SCDC_CR_scdc_reset_mask, (rst) ? HDMI_STB_SCDC_CR_scdc_reset_mask : 0);
}


/*---------------------------------------------------
 * Func : lib_hdmi_scdc_reset_toggle
 *
 * Desc : toggle reset bit to reset all SCDC register
 *
 * Para : nport : HDMI port number 
 *   
 * Retn : N/A
 *--------------------------------------------------*/
void lib_hdmi_scdc_reset_toggle(unsigned char nport)
{
    lib_hdmi_scdc_reset(nport, 1);
    lib_hdmi_scdc_reset(nport, 0);
    HDMI_PRINTF("HDMI SCDC: clear(p%d)\n", nport);
}


/*---------------------------------------------------
 * Func : lib_hdmi_scdc_ced_enable
 *
 * Desc : enable/disable SCDC CED
 *
 * Para : nport : HDMI port number
 *        enable : 
 *
 * Retn : N/A
 *--------------------------------------------------*/
void lib_hdmi_scdc_ced_enable(
    unsigned char           nport, 
    unsigned char           enable
    )
{
    if (hdmi_rx[nport].hdmi_2p1_en)
    {
        hdmi_mask(HD21_cercr_reg, ~HDMI21_P0_hd21_cercr_scdc_ced_en_mask, 
                  (enable) ? HDMI21_P0_hd21_cercr_scdc_ced_en_mask : 0);
    }
    else
    {
    	hdmi_mask(HDMI_CERCR_reg, ~HDMI_CERCR_scdc_ced_en_mask, 
                  (enable) ? HDMI_CERCR_scdc_ced_en_mask : 0);
    }
}


/*---------------------------------------------------
 * Func : lib_hdmi_scdc_data_sel
 *
 * Desc : Set source of SCDC
 *
 * Para : nport : HDMI port number
 *        hdmi2p1 : 1 : link to hdmi 2.1           
 *                  0 : link to hdmi 2.0     
 *   
 * Retn : N/A
 *--------------------------------------------------*/
void lib_hdmi_scdc_data_sel(
    unsigned char           nport, 
    unsigned char           hdmi2p1
    )
{
    if (hdmi2p1)
        hdmi2p1 = 1;

    if (HDMI_STB_SCDC_CR_get_scdc_sel(hdmi_in(HDMI_STB_SCDC_CR_reg))==hdmi2p1)
        return;

    hdmi_mask(HDMI_STB_SCDC_CR_reg, ~HDMI_STB_SCDC_CR_scdc_sel_mask, 
                    HDMI_STB_SCDC_CR_scdc_sel(hdmi2p1));
    
    HDMI_SCDC_INFO("scdc src = %s\n", (hdmi2p1) ? "hdmi 2.1" : "hdmi 2.0");
}


/*---------------------------------------------------
 * Func : lib_hdmi_scdc_is_sink_config_31_updated
 *
 * Desc : check the sink config (byte31) has been updated
 *
 * Para : nport : HDMI port number 
 *   
 * Retn : 0 : no updated, others : updated
 *--------------------------------------------------*/ 
unsigned char lib_hdmi_scdc_is_sink_config_31_updated(
    unsigned char           nport
    )
{
    return HDMI_STB_SCDC_CR_get_write31_det_irq_flag(hdmi_in(HDMI_STB_SCDC_CR_reg));    
}



/*---------------------------------------------------
 * Func : lib_hdmi_scdc_clr_sink_config_31_update_status
 *
 * Desc : clear sink config (byte31) updated status
 *
 * Para : nport : HDMI port number 
 *   
 * Retn : N/A
 *--------------------------------------------------*/ 
void lib_hdmi_scdc_clr_sink_config_31_update_status(
    unsigned char           nport
    )
{
    hdmi_mask(HDMI_STB_SCDC_CR_reg, ~HDMI_STB_SCDC_CR_write31_det_irq_flag_mask, 
                                HDMI_STB_SCDC_CR_write31_det_irq_flag_mask);    
}




/*---------------------------------------------------
 * Func : lib_hdmi_scdc_read
 *
 * Desc : read SCDC register
 *
 * Para : nport : HDMI port number
 *        addr :  scdc register address 
 *   
 * Retn : SCDC register value
 *--------------------------------------------------*/
unsigned char lib_hdmi_scdc_read(
    unsigned char           nport, 
    unsigned char           addr
    )
{
    hdmi_out(HDMI_STB_SCDC_AP_reg, addr);
    return HDMI_STB_SCDC_DP_get_dp1(hdmi_in(HDMI_STB_SCDC_DP_reg));
}


/*---------------------------------------------------
 * Func : lib_hdmi_scdc_write
 *
 * Desc : write SCDC register
 *
 * Para : nport : HDMI port number
 *        addr  : scdc register address 
 *        value : value to be written
 *   
 * Retn : N/A
 *--------------------------------------------------*/
void lib_hdmi_scdc_write(
    unsigned char           nport, 
    unsigned char           addr,
    unsigned char           value
    )
{
    hdmi_out(HDMI_STB_SCDC_AP_reg, addr);
    hdmi_out(HDMI_STB_SCDC_DP_reg, value);
}


/*---------------------------------------------------
 * Func : lib_hdmi_scdc_write_mask
 *
 * Desc : write SCDC register
 *
 * Para : nport : HDMI port number
 *        addr  : scdc register address 
 *        mask  : the masked bit won't be updated
 *        value : value to be written
 *   
 * Retn : N/A
 *--------------------------------------------------*/
void lib_hdmi_scdc_write_mask(
    unsigned char           nport, 
    unsigned char           addr,
    unsigned char           mask,
    unsigned char           value
    )
{
    hdmi_out(HDMI_STB_SCDC_AP_reg, addr);
    hdmi_out(HDMI_STB_SCDC_DP_reg, HDMI_STB_SCDC_DP_dp1_rwmask(mask) |
                               HDMI_STB_SCDC_DP_dp1(value));
}


/*---------------------------------------------------
 * Func : lib_hdmi_scdc_set_flt_update
 *
 * Desc : set flt_update status
 *
 * Para : nport : HDMI port number
 *
 * Retn : N/A
 *--------------------------------------------------*/
void lib_hdmi_scdc_set_flt_update(unsigned char nport)
{    
    lib_hdmi_scdc_write_mask(nport,
        SCDC_UPDATE_FLAGS, ~SCDC_UPDATE_FLAGS_FLT_UPDATE,
        SCDC_UPDATE_FLAGS_FLT_UPDATE);
}

/*---------------------------------------------------
 * Func : lib_hdmi_scdc_set_flt_update
 *
 * Desc : get flt_update status
 *
 * Para : nport : HDMI port number
 *
 * Retn : flt update status
 *--------------------------------------------------*/
unsigned char lib_hdmi_scdc_get_flt_update(unsigned char nport)
{
    return (lib_hdmi_scdc_read(nport, SCDC_UPDATE_FLAGS) & SCDC_UPDATE_FLAGS_FLT_UPDATE) ? 1 : 0;
}

