/*
 *  This is a driver for the eMMC controller found in Realtek
 *  SoCs.
 *
 *  Copyright (C) 2013 Realtek Semiconductors, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <common.h>
#include <drivers/mmc/rtkmmc.h>
#include <drivers/mmc/rtkmmc_generic.h>

#if defined(Config_Boot_ATF_TRUE)
#include <system.h>
#endif
#define __RTKMMC_GENERIC_C__

#ifdef CONFIG_GENERIC_MMC

/************************************************************************
 *  Local define
 ************************************************************************/
#ifdef Config_emmc_timing_is_HS400
#define EN_EMMC_HS400
#define EN_EMMC_HS200
#endif

#ifdef Config_emmc_timing_is_HS200
#define EN_EMMC_HS200
#endif

#define CONFIG_MMC_MODE_8BIT
/************************************************************************
 *  function protal type
 ************************************************************************/
extern unsigned int rtkmmc_debug_msg;
extern unsigned int rtkmmc_off_error_msg_in_init_flow;

extern int mmc_initial( int reset_only );
extern int dw_em_init_setup(void);

extern int mmccr_SendCmd( unsigned int cmd_idx,
			unsigned int cmd_arg,
			unsigned int rsp_para,
			unsigned int * rsp_buffer );
extern int mmccr_Stream_Only( unsigned int address,
			unsigned int blk_cnt,
			unsigned int xfer_flag,
			unsigned int * data_buffer );

extern int dw_em_Normal( ushort cmdidx,
			unsigned int    cmdarg,
			unsigned int    xfer_flag,
			unsigned int    byte_cnt,
			unsigned char * data_buffer );

extern int dw_em_Stream(
	struct mmc_cmd * mcmd,
	struct mmc_data * data,
	unsigned int xfer_flag);

extern int dw_em_SendCmd( unsigned char  cmd_idx,
			unsigned int   cmd_arg,
			unsigned int   ctl,
			unsigned int * rsp);


extern void mmccr_set_div( unsigned int set_div, unsigned int show_msg );
extern int dw_em_send_stop(e_device_type * card);
extern int rtk_eMMC_xfer_counter(e_device_type * card,int byte_cnt);

extern int dw_em_set_DDR(u32 ddr_mode);
extern int dw_em_set_DDR400(u32 ddr_mode);
extern int dw_em_execute_tuning(u32 opcode);
extern int dw_em_dis_sd_tune(void);
extern void dw_em_set_IO_driving(unsigned int type_sel);
extern void eMMC_show_speed(void);

/************************************************************************
 *  Local veriable
 ************************************************************************/
int mmc_had_been_initialized;
int mmc_ready_to_use;
static struct mmc mmc_dev[2];

char * non_cache_ext_csd;
char * non_cache_ext_csd2;
char * non_cache_emmc_tmp_buffer;
int allocate_ext_csd_magic_number;

/************************************************************************
 *  function body
 ************************************************************************/
static
int rtkmmc_send_cmd(
	struct mmc * mmc,
	struct mmc_cmd * cmd,
	struct mmc_data * data )
{
	int ret_err;

	mmcinfo("\n*** cmdidx=0x%02x(%d),cmd->cmdarg=0x%08x,resp_type=0x%08x -------\n",
			cmd->cmdidx, cmd->cmdidx, cmd->cmdarg, cmd->resp_type);
	if( data ) {
		mmcinfo("*** flags=0x%08x(%s), blks=%d, blksize=%d, buff=0x%08x\n",
			data->flags,
			(data->flags&MMC_DATA_READ) ? "R" : "W",
			data->blocks, data->blocksize,(unsigned int)data->dest);
	}
	/*
	 * CMDREG
	 * CMDIDX[13:8] : Command index
	 * DATAPRNT[5]  : Data Present Select
	 * ENCMDIDX[4]  : Command Index Check Enable
	 * ENCMDCRC[3]  : Command CRC Check Enable
	 * RSPTYP[1:0]
	 *  00 = No Response
	 *  01 = Length 136
	 *  10 = Length 48
	 *  11 = Length 48 Check busy after response
	 */
	/* Delay added before checking the status of frq change
	 * retry not supported by mmc.c(core file)
	 */
	ret_err = -1;

	if(data){
		unsigned int xfer_flag;
		if(data->flags & MMC_DATA_READ)
			xfer_flag = RTK_MMC_DATA_READ;
		else
			xfer_flag = RTK_MMC_DATA_WRITE;

		ret_err = dw_em_Stream( cmd,data,xfer_flag);

		rtk_eMMC_xfer_counter(&emmc_card,(data->blocks*512));

	} else {
		ret_err = dw_em_SendCmd( cmd->cmdidx, cmd->cmdarg,
					cmd->resp_type, cmd->response );
	}

	return ret_err;
}

#define HS_52_DDR   (0x01)
#define HS_200_DDR  (0x01<<16)

typedef int (*set_ddr_func_t)(u32 ddr_mode);
static void rtkmmc_set_ios(struct mmc * mmc)
{
	//e_device_type* sdport = mmc->priv;
	set_ddr_func_t dw_em_ddr_func = NULL;

	unsigned int tmp_clock = 0;
	unsigned int tmp_bits = 0;
	unsigned int ctrl_DDR = 0;
	unsigned int bus_width = 0;
	unsigned int timing = 0;
	unsigned int driving = 0;

	timing    = mmc->timing;
	tmp_clock = mmc->clock;
	bus_width = mmc->bus_width;

	dw_em_ddr_func = dw_em_set_DDR;

	if (timing == MMC_TIMING_MMC_HS400) {
		ctrl_DDR = 1;
		if (tmp_clock > CLK_200Mhz) {
			tmp_clock = CLK_200Mhz;
		}
		dw_em_ddr_func = dw_em_set_DDR400;
		driving = DRIVING_HS400;
	} else if (timing == MMC_TIMING_MMC_HS200) {
		ctrl_DDR = 0;
		if (tmp_clock > CLK_200Mhz) {
			tmp_clock = CLK_200Mhz;
		}
		driving = DRIVING_HS200;
	} else if (timing == MMC_TIMING_UHS_DDR50) {
		dw_em_dis_sd_tune();
		ctrl_DDR = 1;
		if(tmp_clock > CLK_50Mhz){
			tmp_clock = CLK_50Mhz;
		}
		driving = DRIVING_COMMON;
	} else if ((timing == MMC_TIMING_SD_HS)
		 ||(timing == MMC_TIMING_MMC_HS)) {
		dw_em_dis_sd_tune();
		ctrl_DDR = 0;
		if(tmp_clock > CLK_50Mhz){
			tmp_clock = CLK_50Mhz;
		}
		driving = DRIVING_COMMON;
	} else {
		dw_em_dis_sd_tune();
		ctrl_DDR = 0;
		if(tmp_clock>CLK_25Mhz){
			tmp_clock = CLK_25Mhz;
		}
		driving = DRIVING_COMMON;
	}

	/*
	 Here, need to check timing mode to enable or disable dw_em_dis_sd_tune(sdport);
	*/
	if (bus_width == 8){
		tmp_bits = BUS_WIDTH_8;
	}else if (bus_width == 4){
		tmp_bits = BUS_WIDTH_4;
	}else{
		tmp_bits = BUS_WIDTH_1;
	}

	dw_em_set_IO_driving(driving);
	dw_em_ddr_func(ctrl_DDR);
	dw_em_set_bits(tmp_bits);
	dw_em_set_clk(tmp_clock);

	}

static int rtkmmc_execute_tuning(struct mmc* mmc,uint opcode)
{
	int ret_err;

	ret_err = dw_em_execute_tuning(opcode);

	return ret_err;
}

static int rtkmmc_init_setup(struct mmc * mmc)
{
	mmcinfo("\n");
	dw_em_init_setup();
	return 0;
}

int board_mmc_init(bd_t * bis)
{
	int ret_err;
	struct mmc * pmmc;

	#ifdef MMC_ALLOCATE_BUFFER_FROM_NONCACHE_AREA
		if( allocate_ext_csd_magic_number != ALLOCATE_EXT_CSD_MAGIC_NUMBER ) {
			allocate_ext_csd_magic_number = ALLOCATE_EXT_CSD_MAGIC_NUMBER;
			UPRINTF("\n");
			non_cache_ext_csd = malloc_noncache_align( 512, 64);
			non_cache_ext_csd2 = malloc_noncache_align( 512, 64);
			non_cache_emmc_tmp_buffer = malloc_noncache_align( 512, 64);
			UPRINTF("MMC: non-cache ext_csd buffer  0x%08x\n", (unsigned int)non_cache_ext_csd);
			UPRINTF("MMC: non-cache ext_csd buffer2 0x%08x\n", (unsigned int)non_cache_ext_csd2);
			UPRINTF("MMC: non-cache temp buffer     0x%08x\n", (unsigned int)non_cache_emmc_tmp_buffer);
		}
	#else
		if( allocate_ext_csd_magic_number != ALLOCATE_EXT_CSD_MAGIC_NUMBER ) {
			allocate_ext_csd_magic_number = ALLOCATE_EXT_CSD_MAGIC_NUMBER;

#ifndef CONFIG_LESS_CONSOLE
			UPRINTF("MMC: NOT allocate buffer from non-cache area\n");
#endif
			non_cache_ext_csd = NULL;
			non_cache_ext_csd2 = NULL;
			non_cache_emmc_tmp_buffer = NULL;
		}
	#endif

	mmc_had_been_initialized = 0;
	mmc_ready_to_use = 0;

#if 0 // directly check eMMC device by customized init flow ( only for eMMC )
	rtkmmc_debug_msg = 1; // 1: enable debug message
	rtkmmc_off_error_msg_in_init_flow = 0; // 1: not show error message in initial stage
	ret_err = rtk_eMMC_init();
#else
	rtkmmc_debug_msg = 1; // 1: enable debug message
	rtkmmc_off_error_msg_in_init_flow = 1; // 1: not show error message in initial stage
	mmc_initial(1);
	ret_err = 0;
#endif
	if( !ret_err ) {

#define IC_NICKNAME "RTK"

		pmmc = &mmc_dev[0];
		sprintf(pmmc->name, "%s eMMC",IC_NICKNAME);
		pmmc->send_cmd = rtkmmc_send_cmd;
		pmmc->set_ios = rtkmmc_set_ios;
		pmmc->init = rtkmmc_init_setup;
		pmmc->getcd = NULL;
		pmmc->execute_tuning = rtkmmc_execute_tuning;
		pmmc->priv = (void *)&emmc_card;

		pmmc->voltages = MMC_VDD_30_31 | MMC_VDD_31_32 |
				 MMC_VDD_32_33 | MMC_VDD_33_34 |
				 MMC_VDD_165_195;

		pmmc->host_caps = MMC_MODE_4BIT
				| MMC_MODE_HS_52MHz
				| MMC_MODE_HS
				| MMC_MODE_HC;

#ifdef CONFIG_MMC_MODE_8BIT
		pmmc->host_caps |= MMC_MODE_8BIT;
#endif

#ifdef	EN_EMMC_HS200
		pmmc->host_caps |= MMC_MODE_HS_200MHz;
#endif

#ifdef EN_EMMC_HS400
		pmmc->host_caps |= (MMC_MODE_HS_400MHz|MMC_MODE_HS_200MHz);
#endif

//#define DDR_SUPPORT //disable DDR mode
#ifdef DDR_SUPPORT
		pmmc->host_caps |= MMC_MODE_DDR;
#endif

		pmmc->f_min = 400000;

#if defined(EN_EMMC_HS400) || defined(EN_EMMC_HS200)
		pmmc->f_max = 200000000;
#else
		pmmc->f_max = 50000000;
#endif
		pmmc->b_max = EMMC_MAX_XFER_BLK; // max transfer block size

		mmc_register(pmmc);
	}

	return ret_err;
}

int bringup_mmc_driver( void )
{
	int ret_err;
	struct mmc * mmc;
	int total_device_num = -1;
	int curr_device = -1;

	total_device_num = get_mmc_num();
	if ( total_device_num < 0 ) {
		UPRINTF("no registed mmc device\n");
		return curr_device;
	}

	if( mmc_had_been_initialized ) {
		return 0; // alwasy use slot 0
	}

	curr_device = 0; // alwasy use slot 0
	rtkmmc_debug_msg = 0; // 1: enable debug message
	rtkmmc_off_error_msg_in_init_flow = 1; // 1: not show error message in initial stage
	mmc = find_mmc_device(curr_device);
	if( mmc ) {
		EXECUTE_CUSTOMIZE_FUNC(0);
		mmc_init(mmc);
		EXECUTE_CUSTOMIZE_FUNC(0);

#ifndef CONFIG_LESS_CONSOLE
		UPRINTF("Device: %s\n", mmc->name);
		UPRINTF("Manufacturer ID: %x\n", mmc->cid[0] >> 24);
		UPRINTF("OEM: %x\n", (mmc->cid[0] >> 8) & 0xffff);
		UPRINTF("Name: %c%c%c%c%c \n", (mmc->cid[0] & 0xff),
			(mmc->cid[1] >> 24) & 0xff,
			(mmc->cid[1] >> 16) & 0xff,
			(mmc->cid[1] >> 8) & 0xff,
			(mmc->cid[1] >> 0) & 0xff);
#endif

		UPRINTF("Tran Speed: %d\n", mmc->tran_speed);
		UPRINTF("Rd Block Len: %d\n", mmc->read_bl_len);
		UPRINTF("%s version %d.%d\n", IS_SD(mmc) ? "SD" : "MMC",
				(mmc->version >> 4) & 0xf, mmc->version & 0xf);
		UPRINTF("High Capacity: %s\n", mmc->high_capacity ? "Yes" : "No");
		puts("Capacity: ");
		print_size(mmc->capacity, "\n");
		UPRINTF("Bus Width: %d-bit\n", mmc->bus_width);
		eMMC_show_speed();

		ret_err = curr_device;

		mmc_had_been_initialized = 1;
		if( mmc->capacity ) {
			mmc_ready_to_use = 1;
		}
		else {
			mmc_ready_to_use = 0;
		}
	}
	else {
		UPRINTF("no mmc device at slot %x\n", curr_device);
		ret_err = -999;
	}
	rtkmmc_debug_msg = 0; // 0: off cmd message
	rtkmmc_off_error_msg_in_init_flow = 0; // 0: show error message after init done

	return ret_err;
}

int mmc_data_sync( struct mmc * mmc )
{
	emmc_card.rca = (mmc->rca)<<16;
#ifndef CONFIG_LESS_CONSOLE
	UPRINTF("RCA= 0x%x\n", emmc_card.rca);
#endif
	emmc_card.sector_addressing = mmc->high_capacity ? 1 : 0;
	emmc_card.mmc = mmc;
	emmc_card.ext_csd.sectors = (mmc->capacity>>9);
	export_rtk_emmc_chk_max_xfer_blk(mmc, 0);
	return 0;

}

#endif // end of CONFIG_GENERIC_MMC
