/*
 * rtl8168.c : U-Boot driver for Realtek Gigabit Ethernet controllers
 *             with PCI-Express interface
 *
 * Masami Komiya (mkomiya@sonare.it)
 *
 * Most part is taken from r8168.c of Linux kernel
 *
 */

/**************************************************************************
*    rtl8168.c: Etherboot device driver for Realtek Gigabit Ethernet
*    controllers with PCI-Express interface
*    Written 2013 by Realtek <nicfae@realtek.com>
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    Portions of this code based on:
*	 r8168.c: A Realtek Gigabit Ethernet controllers with PCI-Express
*    interface driver for Linux kernel 3.x.
*
***************************************************************************/
#include <common.h>
#include <malloc.h>
#include <net.h>
#include <netdev.h>
#include <asm/io.h>
#include <errno.h>
#include <linux/mii.h>
#include <asm/arch/system.h>
#include <rbus/plletn_reg.h>

//#include <asm/arch-rtk/system.h>

#undef DBG_reg
//#undef DEBUG_RTL8168
//#undef DEBUG_RTL8168_TX
//#undef DEBUG_RTL8168_RX

//#define DEBUG_RTL8168
//#define DEBUG_RTL8168_TX
//#define DEBUG_RTL8168_RX

#define drv_version "v0.1"
#define drv_date "08-29-2013"
#define REG_BASE    0x18016000

/* Condensed operations for readability. */
#define currticks()	get_timer(0)

/* MAC address length*/
#define MAC_ADDR_LEN	6

/* max supported gigabit ethernet frame size -- must be at least (dev->mtu+14+4).*/
#define MAX_ETH_FRAME_SIZE	1536

#define TX_FIFO_THRESH 256	/* In bytes */

#define Reserved2_data  7
#define RX_DMA_BURST    7   /* Maximum PCI burst, '6' is 1024 */
#define RX_DMA_BURST_128 3
#define TX_DMA_BURST_unlimited  7
#define TX_DMA_BURST_1024   6
#define TX_DMA_BURST_512    5
#define TX_DMA_BURST_256    4
#define TX_DMA_BURST_128    3
#define TX_DMA_BURST_64     2
#define TX_DMA_BURST_32     1
#define TX_DMA_BURST_16     0
#define Reserved1_data  0x3F
#define RxPacketMaxSize 0x3FE8  /* 16K - 1 - ETH_HLEN - VLAN - CRC... */
#define Jumbo_Frame_2k  (2 * 1024)
#define Jumbo_Frame_3k  (3 * 1024)
#define Jumbo_Frame_4k  (4 * 1024)
#define Jumbo_Frame_5k  (5 * 1024)
#define Jumbo_Frame_6k  (6 * 1024)
#define Jumbo_Frame_7k  (7 * 1024)
#define Jumbo_Frame_8k  (8 * 1024)
#define Jumbo_Frame_9k  (9 * 1024)
#define InterFrameGap   0x03    /* 3 means InterFrameGap = the shortest one */
#define RxEarly_off_V1 (0x07 << 11)
#define RxEarly_off_V2 (1 << 11)

#define RX_BUF_SIZE 1536
#define R8168_TX_RING_BYTES (NUM_TX_DESC * sizeof(struct TxDesc))
#define R8168_RX_RING_BYTES (NUM_RX_DESC * sizeof(struct RxDesc))

#define NUM_TX_DESC	16	/* Number of Tx descriptor registers */
#define NUM_RX_DESC	16	/* Number of Rx descriptor registers */

#define TX_TIMEOUT  (6*HZ)

/* write/read MMIO register. Notice: {read,write}[wl] do the necessary swapping */
#define RTL_W8(reg, val8)	rtd_outb(ioaddr + (reg), (val8))
#define RTL_W16(reg, val16)	rtd_outw(ioaddr + (reg), (val16))
#define RTL_W32(reg, val32)	rtd_outl(ioaddr + (reg), (val32))
#define RTL_R8(reg)		rtd_inb(ioaddr + (reg))
#define RTL_R16(reg)		rtd_inw(ioaddr + (reg))
#define RTL_R32(reg)		rtd_inl(ioaddr + (reg))

#define ETH_FRAME_LEN	MAX_ETH_FRAME_SIZE
#define ETH_ALEN	MAC_ADDR_LEN
#define ETH_ZLEN	60

#define AUTONEG_ENABLE		0x01
#define DUPLEX_HALF		0x00
#define DUPLEX_FULL		0x01
#define SPEED_10		10
#define SPEED_100		100
#define SPEED_1000		1000

#define NIC_RAMCODE_VERSION 0x0001

#define _RXDESC_PER_CACHELINE (ARCH_ALIGN/sizeof(struct RxDesc)) 
#define ARCH_ALIGN 64

/* Clock & Reset Registers */
#define REG_STB_CTRL0			STB_ST_CTRL0_reg
#define REG_STB_CLKEN0			STB_ST_CLKEN0_reg
#define REG_STB_SRST0			STB_ST_SRST0_reg
#define REG_STB_XTAL_EN       STB_ST_CTRL0_xtal_en_status_mask
#define REG_STB_ETN_CLK_SEL   STB_ST_CTRL0_etn_clk_sel_rbus_mask
#define REG_STB_RSTN_MT         STB_ST_CTRL0_rstn_mt_mask
#define REG_STB_CTRL_ETN_LDO_EN STB_ST_CTRL0_etn_ldo_en_mask

#define CLKEN_WRITE_DATA		STB_ST_CLKEN0_write_data_mask
#define CLKEN_ETN_RBUS			STB_ST_CLKEN0_clken_etn_rbus_mask
#define CLKEN_ETN 			STB_ST_CLKEN0_clken_etn_mask

#define RSTN_WRITE_DATA			STB_ST_SRST0_write_data_mask
#define RSTN_ETN			STB_ST_SRST0_rstn_etn_mask

#define CTRL_ETN			STB_ST_CTRL0_etn_rstn_mask_mask

/* PLLETN registers */
#define REG_PLLETN_CLKEN		PLLETN_GETN_CLKEN_reg
#define REG_PLLETN_CLKEN_MAC_MASK	PLLETN_GETN_CLKEN_clken_getn_mac_mask
#define REG_PLLETN_CLKEN_SYS_MASK	PLLETN_GETN_CLKEN_clken_getn_sys_mask
#define REG_PLLETN_CLKEN_WRITE_MASK	PLLETN_GETN_CLKEN_write_data_mask

#define REG_PLLETN_RST			PLLETN_GETN_RST_reg
#define REG_PLLETN_RST_PHY_MASK		PLLETN_GETN_RST_rstn_getn_phy_mask
#define REG_PLLETN_RST_MAC_MASK		PLLETN_GETN_RST_rstn_getn_mac_mask
#define REG_PLLETN_RST_SYS_MASK		PLLETN_GETN_RST_rstn_getn_sys_mask
#define REG_PLLETN_RST_WRITE_MASK	PLLETN_GETN_RST_write_data_mask

#define REG_PLLETN_PHY_INFO                             PLLETN_ETN_phy_info_reg_reg
#define REG_PLLETN_PHY_INFO_AFE_MASK                    PLLETN_ETN_phy_info_reg_afe_pow_state_mask
#define REG_PLLETN_PHY_INFO_POR_DVTH_CEN_MASK           PLLETN_ETN_phy_info_reg_por_dvth_cen_mask
#define REG_PLLETN_PHY_INFO_POR_AVTH_CEN_MASK           PLLETN_ETN_phy_info_reg_por_avth_cen_mask
#define REG_PLLETN_PHY_INFO_POR_DVTH_CEN_DATA(data)     PLLETN_ETN_phy_info_reg_por_dvth_cen(data)
#define REG_PLLETN_PHY_INFO_POR_AVTH_CEN_DATA(data)     PLLETN_ETN_phy_info_reg_por_avth_cen(data)


#define REG_PLLETN_SYS			PLLETN_SYS_PLL_ETN1_reg
#define REG_PLLETN_SYS_POW_MASK		PLLETN_SYS_PLL_ETN1_plletn_pow_mask
#define REG_PLLETN_SYS_POWLDO_MASK	PLLETN_SYS_PLL_ETN1_plletn_powldo_mask

#define REG_PLLETN_INIT_DONE		PLLETN_INIT_AUTOLOAD_DONE_reg
#define	REG_PLLETN_INIT_DONE_MASK	PLLETN_INIT_AUTOLOAD_DONE_init_autoload_done_mask

#ifdef CONFIG_OTP_UTIL
#define ETN_OTP
#endif

#ifdef ETN_OTP
#include <drivers/otp/rtk_otp_region_api.h>

#define IS_ETN_OTP_INTERNAL_ANALOG_LDO_DONE(X)          ((X) & 0x8)
#define IS_ETN_OTP_INTERNAL_DIGITAL_LDO_DONE(X)         ((X) & 0x8)
#define ETN_OTP_INTERNAL_ANALOG_LDO_VAL(val)              (((val) & 0x7) >> 0)
#define ETN_OTP_INTERNAL_DIGITAL_LDO_VAL(val)              (((val) & 0x7) >> 0)
extern bool rtk_otp_field_read(OTP_FIELD_ID field_id, unsigned char *data, unsigned int len);
#endif


extern void enable_console(void);

enum RTL8168_registers {
    MAC0            = 0x00,     /* Ethernet hardware address. */
    MAC4            = 0x04,
    MAR0            = 0x08,     /* Multicast filter. */
    CounterAddrLow      = 0x10,
    CounterAddrHigh     = 0x14,
    CustomLED       = 0x18,
    TxDescStartAddrLow  = 0x20,
    TxDescStartAddrHigh = 0x24,
    TxHDescStartAddrLow = 0x28,
    TxHDescStartAddrHigh    = 0x2c,
    FLASH           = 0x30,
    ERSR            = 0x36,
    ChipCmd         = 0x37,
    TxPoll          = 0x38,
    IntrMask        = 0x3C,
    IntrStatus      = 0x3E,
    TxConfig        = 0x40,
    RxConfig        = 0x44,
    TCTR            = 0x48,
    Cfg9346         = 0x50,
    Config0         = 0x51,
    Config1         = 0x52,
    Config2         = 0x53,
    Config3         = 0x54,
    Config4         = 0x55,
    Config5         = 0x56,
    TDFNR           = 0x57,
    TimeIntr        = 0x58,
    PHYAR           = 0x60,
    CSIDR           = 0x64,
    CSIAR           = 0x68,
    PHYstatus       = 0x6C,
    MACDBG          = 0x6D,
    GPIO            = 0x6E,
    PMCH            = 0x6F,
    ERIDR           = 0x70,
    ERIAR           = 0x74,
    EPHY_RXER_NUM       = 0x7C,
    EPHYAR          = 0x80,
    OCPDR           = 0xB0,
    MACOCP          = 0xB0,
    OCPAR           = 0xB4,
    PHYOCP          = 0xB8,
    DBG_reg         = 0xD1,
    MCUCmd_reg      = 0xD3,
    RxMaxSize       = 0xDA,
    EFUSEAR         = 0xDC,
    CPlusCmd        = 0xE0,
    IntrMitigate        = 0xE2,
    RxDescAddrLow       = 0xE4,
    RxDescAddrHigh      = 0xE8,
    MTPS            = 0xEC,
    FuncEvent       = 0xF0,
    FuncEventMask       = 0xF4,
    FuncPresetState     = 0xF8,
    FuncForceEvent      = 0xFC,
};

enum RTL8168_register_content {
    /* InterruptStatusBits */
    SYSErr      = 0x8000,
    PCSTimeout  = 0x4000,
    SWInt       = 0x0100,
    TxDescUnavail   = 0x0080,
    RxFIFOOver  = 0x0040,
    LinkChg     = 0x0020,
    RxDescUnavail   = 0x0010,
    TxErr       = 0x0008,
    TxOK        = 0x0004,
    RxErr       = 0x0002,
    RxOK        = 0x0001,

    /* RxStatusDesc */
    RxRWT = (1 << 22),
    RxRES = (1 << 21),
    RxRUNT = (1 << 20),
    RxCRC = (1 << 19),

    /* ChipCmdBits */
    StopReq  = 0x80,
    CmdReset = 0x10,
    CmdRxEnb = 0x08,
    CmdTxEnb = 0x04,
    RxBufEmpty = 0x01,

    /* Cfg9346Bits */
    Cfg9346_Lock = 0x00,
    Cfg9346_Unlock = 0xC0,
    Cfg9346_EEDO = (1 << 0),
    Cfg9346_EEDI = (1 << 1),
    Cfg9346_EESK = (1 << 2),
    Cfg9346_EECS = (1 << 3),
    Cfg9346_EEM0 = (1 << 6),
    Cfg9346_EEM1 = (1 << 7),

    /* rx_mode_bits */
    AcceptErr = 0x20,
    AcceptRunt = 0x10,
    AcceptBroadcast = 0x08,
    AcceptMulticast = 0x04,
    AcceptMyPhys = 0x02,
    AcceptAllPhys = 0x01,

    /* Transmit Priority Polling*/
    HPQ = 0x80,
    NPQ = 0x40,
    FSWInt = 0x01,

    /* RxConfigBits */
    Reserved2_shift = 13,
    RxCfgDMAShift = 8,
    RxCfg_128_int_en = (1 << 15),
    RxCfg_fet_multi_en = (1 << 14),
    RxCfg_half_refetch = (1 << 13),
    RxCfg_9356SEL = (1 << 6),

    /* TxConfigBits */
    TxInterFrameGapShift = 24,
    TxDMAShift = 8, /* DMA burst value (0-7) is shift this many bits */
    TxMACLoopBack = (1 << 17),  /* MAC loopback */

    /* Config1 register */
    LEDS1       = (1 << 7),
    LEDS0       = (1 << 6),
    Speed_down  = (1 << 4),
    MEMMAP      = (1 << 3),
    IOMAP       = (1 << 2),
    VPD         = (1 << 1),
    PMEnable    = (1 << 0), /* Power Management Enable */

    /* Config2 register */
    PMSTS_En    = (1 << 5),

    /* Config3 register */
    Isolate_en  = (1 << 12), /* Isolate enable */
    MagicPacket = (1 << 5), /* Wake up when receives a Magic Packet */
    LinkUp      = (1 << 4), /* This bit is reserved in RTL8168B.*/
    /* Wake up when the cable connection is re-established */
    ECRCEN      = (1 << 3), /* This bit is reserved in RTL8168B*/
    Jumbo_En0   = (1 << 2), /* This bit is reserved in RTL8168B*/
    RDY_TO_L23  = (1 << 1), /* This bit is reserved in RTL8168B*/
    Beacon_en   = (1 << 0), /* This bit is reserved in RTL8168B*/

    /* Config4 register */
    Jumbo_En1   = (1 << 1), /* This bit is reserved in RTL8168B*/

    /* Config5 register */
    BWF     = (1 << 6), /* Accept Broadcast wakeup frame */
    MWF     = (1 << 5), /* Accept Multicast wakeup frame */
    UWF     = (1 << 4), /* Accept Unicast wakeup frame */
    LanWake     = (1 << 1), /* LanWake enable/disable */
    PMEStatus   = (1 << 0), /* PME status can be reset by PCI RST# */

    /* CPlusCmd */
    EnableBist  = (1 << 15),
    Macdbgo_oe  = (1 << 14),
    Normal_mode = (1 << 13),
    Force_halfdup   = (1 << 12),
    Force_rxflow_en = (1 << 11),
    Force_txflow_en = (1 << 10),
    Cxpl_dbg_sel    = (1 << 9),//This bit is reserved in RTL8168B
    ASF     = (1 << 8),//This bit is reserved in RTL8168C
    PktCntrDisable  = (1 << 7),
    RxVlan      = (1 << 6),
    RxChkSum    = (1 << 5),
    Macdbgo_sel = 0x001C,
    INTT_0      = 0x0000,
    INTT_1      = 0x0001,
    INTT_2      = 0x0002,
    INTT_3      = 0x0003,

    /* rtl8168_PHYstatus */
    PowerSaveStatus = 0x80,
    TxFlowCtrl = 0x40,
    RxFlowCtrl = 0x20,
    _1000bpsF = 0x10,
    _100bps = 0x08,
    _10bps = 0x04,
    LinkStatus = 0x02,
    FullDup = 0x01,

    /* DBG_reg */
    Fix_Nak_1 = (1 << 4),
    Fix_Nak_2 = (1 << 3),
    DBGPIN_E2 = (1 << 0),

    /* DumpCounterCommand */
    CounterDump = 0x8,

    /* PHY access */
    PHYAR_Flag = 0x80000000,
    PHYAR_Write = 0x80000000,
    PHYAR_Read = 0x00000000,
    PHYAR_Reg_Mask = 0x1f,
    PHYAR_Reg_shift = 16,
    PHYAR_Data_Mask = 0xffff,

    /* EPHY access */
    EPHYAR_Flag = 0x80000000,
    EPHYAR_Write = 0x80000000,
    EPHYAR_Read = 0x00000000,
    EPHYAR_Reg_Mask = 0x1f,
    EPHYAR_Reg_shift = 16,
    EPHYAR_Data_Mask = 0xffff,

    /* CSI access */
    CSIAR_Flag = 0x80000000,
    CSIAR_Write = 0x80000000,
    CSIAR_Read = 0x00000000,
    CSIAR_ByteEn = 0x0f,
    CSIAR_ByteEn_shift = 12,
    CSIAR_Addr_Mask = 0x0fff,

    /* ERI access */
    ERIAR_Flag = 0x80000000,
    ERIAR_Write = 0x80000000,
    ERIAR_Read = 0x00000000,
    ERIAR_Addr_Align = 4, /* ERI access register address must be 4 byte alignment */
    ERIAR_ExGMAC = 0,
    ERIAR_MSIX = 1,
    ERIAR_ASF = 2,
    ERIAR_OOB = 2,
    ERIAR_Type_shift = 16,
    ERIAR_ByteEn = 0x0f,
    ERIAR_ByteEn_shift = 12,

    /* OCP GPHY access */
    OCPDR_Write = 0x80000000,
    OCPDR_Read = 0x00000000,
    OCPDR_Reg_Mask = 0xFF,
    OCPDR_Data_Mask = 0xFFFF,
    OCPDR_GPHY_Reg_shift = 16,
    OCPAR_Flag = 0x80000000,
    OCPAR_GPHY_Write = 0x8000F060,
    OCPAR_GPHY_Read = 0x0000F060,
    OCPR_Write = 0x80000000,
    OCPR_Read = 0x00000000,
    OCPR_Addr_Reg_shift = 16,
    OCPR_Flag = 0x80000000,
    OCP_STD_PHY_BASE_PAGE = 0x0A40,

    /* MCU Command */
    Now_is_oob = (1 << 7),
    Txfifo_empty = (1 << 5),
    Rxfifo_empty = (1 << 4),

    /* E-FUSE access */
    EFUSE_WRITE = 0x80000000,
    EFUSE_WRITE_OK  = 0x00000000,
    EFUSE_READ  = 0x00000000,
    EFUSE_READ_OK   = 0x80000000,
    EFUSE_Reg_Mask  = 0x03FF,
    EFUSE_Reg_Shift = 8,
    EFUSE_Check_Cnt = 300,
    EFUSE_READ_FAIL = 0xFF,
    EFUSE_Data_Mask = 0x000000FF,

    /* GPIO */
    GPIO_en = (1 << 0),

};

enum _DescStatusBit {
    DescOwn     = (1 << 31), /* Descriptor is owned by NIC */
    RingEnd     = (1 << 30), /* End of descriptor ring */
    FirstFrag   = (1 << 29), /* First segment of a packet */
    LastFrag    = (1 << 28), /* Final segment of a packet */

    /* Tx private */
    /*------ offset 0 of tx descriptor ------*/
    LargeSend   = (1 << 27), /* TCP Large Send Offload (TSO) */
    LargeSend_DP = (1 << 16), /* TCP Large Send Offload (TSO) */
    MSSShift    = 16,        /* MSS value position */
    MSSMask     = 0x7FFU,    /* MSS value 11 bits */
    TxIPCS      = (1 << 18), /* Calculate IP checksum */
    TxUDPCS     = (1 << 17), /* Calculate UDP/IP checksum */
    TxTCPCS     = (1 << 16), /* Calculate TCP/IP checksum */
    TxVlanTag   = (1 << 17), /* Add VLAN tag */

    /*@@@@@@ offset 4 of tx descriptor => bits for RTL8168C/CP only     begin @@@@@@*/
    TxUDPCS_C   = (1 << 31), /* Calculate UDP/IP checksum */
    TxTCPCS_C   = (1 << 30), /* Calculate TCP/IP checksum */
    TxIPCS_C    = (1 << 29), /* Calculate IP checksum */
    /*@@@@@@ offset 4 of tx descriptor => bits for RTL8168C/CP only     end @@@@@@*/


    /* Rx private */
    /*------ offset 0 of rx descriptor ------*/
    PID1        = (1 << 18), /* Protocol ID bit 1/2 */
    PID0        = (1 << 17), /* Protocol ID bit 2/2 */

#define RxProtoUDP  (PID1)
#define RxProtoTCP  (PID0)
#define RxProtoIP   (PID1 | PID0)
#define RxProtoMask RxProtoIP

    RxIPF       = (1 << 16), /* IP checksum failed */
    RxUDPF      = (1 << 15), /* UDP/IP checksum failed */
    RxTCPF      = (1 << 14), /* TCP/IP checksum failed */
    RxVlanTag   = (1 << 16), /* VLAN tag available */

    /*@@@@@@ offset 0 of rx descriptor => bits for RTL8168C/CP only     begin @@@@@@*/
    RxUDPT      = (1 << 18),
    RxTCPT      = (1 << 17),
    /*@@@@@@ offset 0 of rx descriptor => bits for RTL8168C/CP only     end @@@@@@*/

    /*@@@@@@ offset 4 of rx descriptor => bits for RTL8168C/CP only     begin @@@@@@*/
    RxV6F       = (1 << 31),
    RxV4F       = (1 << 30),
    /*@@@@@@ offset 4 of rx descriptor => bits for RTL8168C/CP only     end @@@@@@*/
};

enum bits {
    BIT_0 = (1 << 0),
    BIT_1 = (1 << 1),
    BIT_2 = (1 << 2),
    BIT_3 = (1 << 3),
    BIT_4 = (1 << 4),
    BIT_5 = (1 << 5),
    BIT_6 = (1 << 6),
    BIT_7 = (1 << 7),
    BIT_8 = (1 << 8),
    BIT_9 = (1 << 9),
    BIT_10 = (1 << 10),
    BIT_11 = (1 << 11),
    BIT_12 = (1 << 12),
    BIT_13 = (1 << 13),
    BIT_14 = (1 << 14),
    BIT_15 = (1 << 15),
    BIT_16 = (1 << 16),
    BIT_17 = (1 << 17),
    BIT_18 = (1 << 18),
    BIT_19 = (1 << 19),
    BIT_20 = (1 << 20),
    BIT_21 = (1 << 21),
    BIT_22 = (1 << 22),
    BIT_23 = (1 << 23),
    BIT_24 = (1 << 24),
    BIT_25 = (1 << 25),
    BIT_26 = (1 << 26),
    BIT_27 = (1 << 27),
    BIT_28 = (1 << 28),
    BIT_29 = (1 << 29),
    BIT_30 = (1 << 30),
    BIT_31 = (1 << 31)
};

enum effuse {
    EFUSE_SUPPORT = 1,
    EFUSE_NOT_SUPPORT = 0,
};

struct TxDesc {
    u32 opts1;
    u32 opts2;
    u64 addr;
};

struct RxDesc {
    u32 opts1;
    u32 opts2;
    u64 addr;
};

enum mcfg {
    CFG_METHOD_1=0,
    CFG_METHOD_2,
    CFG_METHOD_3,
    CFG_METHOD_4,
    CFG_METHOD_5,
    CFG_METHOD_6,
    CFG_METHOD_7,
    CFG_METHOD_8,
    CFG_METHOD_9 ,
    CFG_METHOD_10,
    CFG_METHOD_11,
    CFG_METHOD_12,
    CFG_METHOD_13,
    CFG_METHOD_14,
    CFG_METHOD_15,
    CFG_METHOD_16,
    CFG_METHOD_17,
    CFG_METHOD_18,
    CFG_METHOD_19,
    CFG_METHOD_20,
    CFG_METHOD_21,
    CFG_METHOD_22,
    CFG_METHOD_23,
    CFG_METHOD_24,
    CFG_METHOD_25,
    CFG_METHOD_26,
    CFG_METHOD_27,
    CFG_METHOD_MAX,
    CFG_METHOD_DEFAULT = 0xFF
};

#define _R(NAME,MAC,RCR,MASK, JumFrameSz) \
    { .name = NAME, .mcfg = MAC, .RCR_Cfg = RCR, .RxConfigMask = MASK, .jumbo_frame_sz = JumFrameSz }

static const struct {
    const char *name;
    u8 mcfg;
    u32 RCR_Cfg;
    u32 RxConfigMask;   /* Clears the bits supported by this chip */
    u32 jumbo_frame_sz;
} rtl_chip_info[] = {
    _R("RTL8168B/8111B",
    CFG_METHOD_1,
    (Reserved2_data << Reserved2_shift) | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_4k),

    _R("RTL8168B/8111B",
    CFG_METHOD_2,
    (Reserved2_data << Reserved2_shift) | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_4k),

    _R("RTL8168B/8111B",
    CFG_METHOD_3,
    (Reserved2_data << Reserved2_shift) | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_4k),

    _R("RTL8168C/8111C",
    CFG_METHOD_4, RxCfg_128_int_en | RxCfg_fet_multi_en | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_6k),

    _R("RTL8168C/8111C",
    CFG_METHOD_5,
    RxCfg_128_int_en | RxCfg_fet_multi_en | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_6k),

    _R("RTL8168C/8111C",
    CFG_METHOD_6,
    RxCfg_128_int_en | RxCfg_fet_multi_en | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_6k),

    _R("RTL8168CP/8111CP",
    CFG_METHOD_7,
    RxCfg_128_int_en | RxCfg_fet_multi_en | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_6k),

    _R("RTL8168CP/8111CP",
    CFG_METHOD_8,
    RxCfg_128_int_en | RxCfg_fet_multi_en | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_6k),

    _R("RTL8168D/8111D",
    CFG_METHOD_9,
    RxCfg_128_int_en | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168D/8111D",
    CFG_METHOD_10,
    RxCfg_128_int_en | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168DP/8111DP",
    CFG_METHOD_11,
    RxCfg_128_int_en | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168DP/8111DP",
    CFG_METHOD_12,
    RxCfg_128_int_en | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168DP/8111DP",
    CFG_METHOD_13,
    RxCfg_128_int_en | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168E/8111E",
    CFG_METHOD_14,
    RxCfg_128_int_en | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168E/8111E",
    CFG_METHOD_15,
    RxCfg_128_int_en | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168E-VL/8111E-VL",
    CFG_METHOD_16,
    RxEarly_off_V1 | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e0080,
    Jumbo_Frame_9k),

    _R("RTL8168E-VL/8111E-VL",
    CFG_METHOD_17,
    RxEarly_off_V1 | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168F/8111F",
    CFG_METHOD_18,
    RxEarly_off_V1 | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168F/8111F",
    CFG_METHOD_19,
    RxEarly_off_V1 | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8411",
    CFG_METHOD_20,
    (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168G/8111G",
    CFG_METHOD_21,
    RxEarly_off_V2 | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168G/8111G",
    CFG_METHOD_22,
    RxEarly_off_V2 | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168EP/8111EP",
    CFG_METHOD_23,
    RxEarly_off_V2 | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168GU/8111GU",
    CFG_METHOD_24,
    RxEarly_off_V2 | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168GU/8111GU",
    CFG_METHOD_25,
    RxCfg_fet_multi_en | RxCfg_128_int_en| (RX_DMA_BURST_128 << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("8411B",
    CFG_METHOD_26,
    RxEarly_off_V2 | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("RTL8168EP/8111EP",
    CFG_METHOD_27,
    RxEarly_off_V2 | (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    Jumbo_Frame_9k),

    _R("Unknown",
    CFG_METHOD_DEFAULT,
    (RX_DMA_BURST << RxCfgDMAShift),
    0xff7e1880,
    RX_BUF_SIZE)
};

struct rtl8168_private {
    struct eth_device *dev;
    phys_addr_t mmio_addr;	/* memory map physical address */
    int chipset;
    u32 mcfg;
    u32 cur_page;
    u32 cur_rx;	/* Index into the Rx descriptor buffer of next Rx pkt. */
    u32 cur_tx;	/* Index into the Tx descriptor buffer of next Rx pkt. */
    u32 dirty_rx;
    u32 dirty_tx;
    u8 *DescArrays;	/* Index of Descriptor buffer */
    struct TxDesc *TxDescArray;	/* Index of 256-alignment Tx Descriptor buffer */
    struct RxDesc *RxDescArray;	/* Index of 256-alignment Rx Descriptor buffer */
    u8 *RxBufferRings;	/* Index of Rx Buffer  */
    u8 *RxBufferRing[NUM_RX_DESC];	/* Index of Rx Buffer array */

    u8  efuse;
    u32 rx_buf_sz;
    u32 rtl8168_rx_config;

    u16 cp_cmd;
};

#define ALIGN_8                 (0x7)
#define ALIGN_16                (0xf)
#define ALIGN_32                (0x1f)
#define ALIGN_64                (0x3f)
#define ALIGN_256               (0xff)

typedef struct _GDUMP_TALLY {
    u64 TxOK;
    u64 RxOK;
    u64 TxERR;
    u32 RxERR;
    u16 MissPkt;
    u16 FAE;
    u32 Tx1Col;
    u32 TxMCol;
    u64 RxOKPhy;
    u64 RxOKBrd;
    u32 RxOKMul;
    u16 TxAbt;
    u16 TxUndrn; //only possible on jumbo frames
}

GDUMP_TALLY, *PGDUMP_TALLY;

static u8 dummy[sizeof( GDUMP_TALLY ) + ALIGN_64];
static void rtl8168_get_mac_address(struct rtl8168_private *tp);
static int rtl8168_init_ring(struct eth_device *dev);
static void rtl8168_hw_start(struct rtl8168_private *tp);

#ifdef CONFIG_OTP_UTIL
extern int otp_read(unsigned long ofst, unsigned long* pval);
#endif

#ifdef CONFIG_WOW
static void mac_ocp_write(struct rtl8168_private *tp, u16 reg_addr, u16 value)
{
    phys_addr_t ioaddr = tp->mmio_addr;
    u32 data32;

    data32 = reg_addr/2;
    data32 <<= OCPR_Addr_Reg_shift;
    data32 += value;
    data32 |= OCPR_Write;

    RTL_W32(MACOCP, data32);
}

static u16 mac_ocp_read(struct rtl8168_private *tp, u16 reg_addr)
{
    phys_addr_t ioaddr = tp->mmio_addr;
    u32 data32;
    u16 data16 = 0;

    data32 = reg_addr/2;
    data32 <<= OCPR_Addr_Reg_shift;

    RTL_W32(MACOCP, data32);
    data16 = (u16)RTL_R32(MACOCP);

    return data16;
}
#endif

static void mdio_write(struct rtl8168_private *tp,
                       u32 RegAddr,
                       u32 value)
{
    phys_addr_t ioaddr = tp->mmio_addr;
    int i;

    if (RegAddr == 0x1F) {
        tp->cur_page = value;
    }

    RTL_W32(PHYAR, PHYAR_Write |
		    (RegAddr & PHYAR_Reg_Mask) << PHYAR_Reg_shift |
		    (value & PHYAR_Data_Mask));
    //printf("WPHY addr %x value %x \n",RegAddr,value);
    for (i = 0; i < 10; i++) {
	    udelay(100);

	    /* Check if the RTL8168 has completed writing to the specified MII register */
	    if (!(RTL_R32(PHYAR) & PHYAR_Flag)) {
		    udelay(20);
		    break;
	    }
    }
}

static u32 mdio_read(struct rtl8168_private *tp,
                     u32 RegAddr)
{
    phys_addr_t ioaddr = tp->mmio_addr;
    int i, value = 0;

    RTL_W32(PHYAR,
		    PHYAR_Read | (RegAddr & PHYAR_Reg_Mask) << PHYAR_Reg_shift);

    for (i = 0; i < 10; i++) {
	    udelay(100);

	    /* Check if the RTL8168 has completed retrieving data from the specified MII register */
	    if (RTL_R32(PHYAR) & PHYAR_Flag) {
		    value = RTL_R32(PHYAR) & PHYAR_Data_Mask;
		    udelay(20);
		    break;
	    }
    }
    //printf("RPHY addr %x value %x \n",RegAddr,value);

    return value;
}

static u32 rtl8168_eri_read(phys_addr_t ioaddr, int addr, int len, int type)
{
    int i, val_shift, shift = 0;
    u32 value1 = 0, value2 = 0, mask;

    if (len > 4 || len <= 0)
        return -1;

    while (len > 0) {
        val_shift = addr % ERIAR_Addr_Align;
        addr = addr & ~0x3;

        RTL_W32(ERIAR,
                ERIAR_Read |
                type << ERIAR_Type_shift |
                ERIAR_ByteEn << ERIAR_ByteEn_shift |
                addr);

        for (i = 0; i < 10; i++) {
            udelay(100);

            /* Check if the RTL8168 has completed ERI read */
            if (RTL_R32(ERIAR) & ERIAR_Flag)
                break;
        }

        if (len == 1)       mask = (0xFF << (val_shift * 8)) & 0xFFFFFFFF;
        else if (len == 2)  mask = (0xFFFF << (val_shift * 8)) & 0xFFFFFFFF;
        else if (len == 3)  mask = (0xFFFFFF << (val_shift * 8)) & 0xFFFFFFFF;
        else            mask = (0xFFFFFFFF << (val_shift * 8)) & 0xFFFFFFFF;

        value1 = RTL_R32(ERIDR) & mask;
        value2 |= (value1 >> val_shift * 8) << shift * 8;

        if (len <= 4 - val_shift) {
            len = 0;
        } else {
            len -= (4 - val_shift);
            shift = 4 - val_shift;
            addr += 4;
        }
    }

    return value2;
}

static int rtl8168_eri_write(phys_addr_t ioaddr, int addr, int len, u32 value, int type)
{
    int i, val_shift, shift = 0;
    u32 value1 = 0, mask;

    if (len > 4 || len <= 0)
        return -1;

    while (len > 0) {
        val_shift = addr % ERIAR_Addr_Align;
        addr = addr & ~0x3;

        if (len == 1)       mask = (0xFF << (val_shift * 8)) & 0xFFFFFFFF;
        else if (len == 2)  mask = (0xFFFF << (val_shift * 8)) & 0xFFFFFFFF;
        else if (len == 3)  mask = (0xFFFFFF << (val_shift * 8)) & 0xFFFFFFFF;
        else            mask = (0xFFFFFFFF << (val_shift * 8)) & 0xFFFFFFFF;

        value1 = rtl8168_eri_read(ioaddr, addr, 4, type) & ~mask;
        value1 |= ((value << val_shift * 8) >> shift * 8);

        RTL_W32(ERIDR, value1);
        RTL_W32(ERIAR,
                ERIAR_Write |
                type << ERIAR_Type_shift |
                ERIAR_ByteEn << ERIAR_ByteEn_shift |
                addr);

        for (i = 0; i < 10; i++) {
            udelay(100);

            /* Check if the RTL8168 has completed ERI write */
            if (!(RTL_R32(ERIAR) & ERIAR_Flag))
                break;
        }

        if (len <= 4 - val_shift) {
            len = 0;
        } else {
            len -= (4 - val_shift);
            shift = 4 - val_shift;
            addr += 4;
        }
    }

    return 0;
}

/**************************************************************************
RECV - Receive a frame
***************************************************************************/
static int rtl8168_recv(struct eth_device *dev)
{
    /* return true if there's an ethernet packet ready to read */
    /* nic->packet should contain data on return */
    /* nic->packetlen should contain length of data */
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
    phys_addr_t ioaddr = tp->mmio_addr;
    int cur_rx;
    int length = 0;
    u16 sts = RTL_R16(IntrStatus);
    int count =0;
    int i=0;
    u32 addr,size;
#ifdef DEBUG_RTL8168_RX
    printf ("%s\n", __FUNCTION__);
#endif

   // printf("IntrStatus %x \n",IntrStatus);

    RTL_W16(IntrStatus, sts);

    rmb();

    // if ((sts & (RxOK)))
    //     printf("RxOK %x \n",sts);
    //  if ((sts & (RxDescUnavail | RxErr | RxOK)) == 0)
    //      return 0;
    //if (sts & (RxDescUnavail)) {
    //    printf("error: RX desc. unavailable ,status = 0x%x \n", sts);
    //}
    //
    //if (sts & (RxErr)) {
    //    printf("error: RX error ,status = 0x%x \n", sts);
    //}
    //
    //if (sts & (RxFIFOOver)) {
    //    printf("error: RX FIFO full ,status = 0x%x \n", sts);
    //    rtl8168_init_ring(dev);
    //    rtl8168_hw_start(tp);
    //    
    //    return 0;
    //}


    cur_rx = tp->cur_rx;

   // while(1) {
    addr=(u32)&tp->RxDescArray[cur_rx];
    addr &= ~(ARCH_ALIGN - 1);
    size = roundup(sizeof(struct RxDesc), ARCH_ALIGN);
//#ifndef CONFIG_NONCACHE_ALLOCATION
    invalidate_dcache_range(addr, addr + size);
//#endif

    //printf ("RX cur_rx = %X opts1 = %X \n", cur_rx, le32_to_cpu(tp->RxDescArray[cur_rx].opts1));

    if ((le32_to_cpu(tp->RxDescArray[cur_rx].opts1) & DescOwn) == 0) {

	    count = count +1;

	    if (!(le32_to_cpu(tp->RxDescArray[cur_rx].opts1) & RxRES)) {
		    length = (int) (le32_to_cpu(tp->RxDescArray[cur_rx].
					    opts1) & 0x00003fff) - 4;


		    invalidate_dcache_range((unsigned long)tp->RxBufferRing[cur_rx],(unsigned long)tp->RxBufferRing[cur_rx]+length);
		    dmb();
		    NetReceive(tp->RxBufferRing[cur_rx], length);

		    //   printf ("NetReceive\n");
	    } else {
		    puts("Error Rx");
	    }            		
	    /*
	     * Free the current buffer, restart the engine and move forward
	     * to the next buffer. Here we check if the whole cacheline of
	     * descriptors was already processed and if so, we mark it free
	     * as whole.
	     */		 		 
	    size = _RXDESC_PER_CACHELINE - 1;
	    if ((cur_rx & size) == size)		
	    {	 		   
		    for (i = cur_rx - size; i <= cur_rx ; i++) {
			    if (i == NUM_RX_DESC - 1){
				    tp->RxDescArray[i].opts1 =
					    cpu_to_le32(DescOwn | RingEnd | tp->rx_buf_sz);	
			    }
			    else{
				    tp->RxDescArray[i].opts1 =
					    cpu_to_le32(DescOwn | tp->rx_buf_sz);					
			    }
		    }

//#ifndef CONFIG_NONCACHE_ALLOCATION
		    flush_cache((unsigned long)&tp->RxDescArray[cur_rx-size],ARCH_ALIGN);			
//#endif
	    }		
	    cur_rx = (cur_rx + 1) % NUM_RX_DESC;
	    tp->cur_rx = cur_rx;

    } else {
	    //  printf ("NetReceive %d packets\n",count);
	    return count;
    }  

    return (0);		/* initially as this is called to flush the input */
}

#define HZ 1000

#ifdef DEBUG_RTL8168_TX
static void print_packet( u8 * buf, int length )
{
    int i;
    int remainder;
    int lines;

    printf("Packet of length %d \n", length );

    lines = length / 16;
    remainder = length % 16;

    for ( i = 0; i < lines ; i ++ ) {
        int cur;

        for ( cur = 0; cur < 8; cur ++ ) {
            u8 a, b;

            a = *(buf ++ );
            b = *(buf ++ );
            printf("%02x%02x ", a, b );
        }
        printf("\n");
    }
    for ( i = 0; i < remainder/2 ; i++ ) {
        u8 a, b;

        a = *(buf ++ );
        b = *(buf ++ );
        printf("%02x%02x ", a, b );
    }
    printf("\n");
}
#endif

/**************************************************************************
SEND - Transmit a frame
***************************************************************************/
static int rtl8168_send(struct eth_device *dev, void *packet, int length)
{
    /* send the packet to destination */
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
    phys_addr_t ioaddr = tp->mmio_addr;
    u32 to,/*lent,*/ len = length;
    int ret, cur_tx/*,dx=0*/;

//    u8 *testpacket;
//    int a,Temp=0;
//    u8 val=0;

#ifdef DEBUG_RTL8168_TX
    int stime = currticks();
    printf ("%s\n", __FUNCTION__);
    printf("sending %d bytes\n", len);
#endif

    cur_tx = tp->cur_tx;

#ifdef DEBUG_RTL8168_TX
    u8 *buf;
    buf = (u8 *) packet;
    printf("pointer %p\n",packet);

    print_packet (buf, length);
#endif

    flush_cache((unsigned long)packet,length);
    tp->TxDescArray[cur_tx].addr = cpu_to_le64((unsigned long)packet);

    if (cur_tx == (NUM_TX_DESC - 1)) {
        tp->TxDescArray[cur_tx].opts1 =
            cpu_to_le32((DescOwn | RingEnd | FirstFrag | LastFrag) |
                        ((len > ETH_ZLEN) ? len : ETH_ZLEN));
    } else {
        tp->TxDescArray[cur_tx].opts1 =
            cpu_to_le32((DescOwn | FirstFrag | LastFrag) |
                        ((len > ETH_ZLEN) ? len : ETH_ZLEN));
    }

    wmb();

//#ifndef CONFIG_NONCACHE_ALLOCATION
    flush_cache((unsigned long)&tp->TxDescArray[cur_tx],sizeof(struct TxDesc));
//#endif
    RTL_W8(ChipCmd, CmdTxEnb | CmdRxEnb);
    RTL_W8(TxPoll, NPQ);    /* set polling bit */

    tp->cur_tx = (cur_tx+1) % NUM_TX_DESC;
    to = currticks() + TX_TIMEOUT;
    do {
//#ifndef CONFIG_NONCACHE_ALLOCATION
        flush_cache((unsigned long)&tp->TxDescArray[cur_tx],
                    sizeof(struct TxDesc));
//#endif
        RTL_W8(TxPoll, NPQ);
    } while ((le32_to_cpu(tp->TxDescArray[cur_tx].opts1) & DescOwn)
             && (currticks() < to));	/* wait */

    if (currticks() >= to) {
#ifdef DEBUG_RTL8168_TX
        puts ("tx timeout/error\n");
        printf ("%s elapsed time : %lu\n", __FUNCTION__, currticks()-stime);
#endif
        ret = 0;
    } else {
#ifdef DEBUG_RTL8168_TX
        puts("tx done\n");
        printf ("%s elapsed time : %lu\n", __FUNCTION__, currticks()-stime);
#endif
        ret = length;
    }

    /* Delay to make net console (nc) work properly */
    udelay(20);
    return ret;
}

static void rtl8168_hw_set_rx_packet_filter(struct eth_device *dev)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
    phys_addr_t ioaddr = tp->mmio_addr;
    int rx_mode;
    u32 tmp = 0;

#ifdef DEBUG_RTL8168
    printf ("%s\n", __FUNCTION__);
#endif

    /* IFF_ALLMULTI */
    /* Too many to filter perfectly -- accept all multicasts. */
    rx_mode = RTL_R32(RxConfig);
    rx_mode |= AcceptBroadcast | AcceptMulticast | AcceptMyPhys;

    tp->rtl8168_rx_config = rtl_chip_info[tp->chipset].RCR_Cfg;
    tmp = tp->rtl8168_rx_config | rx_mode | (RTL_R32(RxConfig) & rtl_chip_info[tp->chipset].RxConfigMask);

    RTL_W32(RxConfig, tmp);

    RTL_W32(MAR0 + 0, 0xffffffff);
    RTL_W32(MAR0 + 4, 0xffffffff);
}

static void
rtl8168_enable_rxdvgate(struct eth_device *dev)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
    phys_addr_t ioaddr = tp->mmio_addr;

    RTL_W8(0xF2, RTL_R8(0xF2) | BIT_3); //? Power on reset dummy register
    mdelay(2);
}

static void
rtl8168_disable_rxdvgate(struct eth_device *dev)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
    phys_addr_t ioaddr = tp->mmio_addr;

    RTL_W8(0xF2, RTL_R8(0xF2) & ~BIT_3);
    mdelay(2);

}

static void
rtl8168_wait_txrx_fifo_empty(struct eth_device *dev)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
    phys_addr_t ioaddr = tp->mmio_addr;
    int i;

    for (i = 0; i < 10; i++) {
	    udelay(100);
	    if (RTL_R32(TxConfig) & BIT_11)
		    break;
    }

    for (i = 0; i < 10; i++) {
	    udelay(100);
	    if ((RTL_R8(MCUCmd_reg) & (Txfifo_empty | Rxfifo_empty)) == (Txfifo_empty | Rxfifo_empty))
		    break;

    }
}

static void
rtl8168_irq_mask_and_ack(phys_addr_t ioaddr)
{
    RTL_W16(IntrMask, 0x0000);
    RTL_W16(IntrStatus, RTL_R16(IntrStatus));
}

static void
rtl8168_nic_reset(struct eth_device *dev)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
    phys_addr_t ioaddr = tp->mmio_addr;
    int i;

#ifdef DEBUG_RTL8168
    printf("%s\n", __func__);
#endif
    RTL_W32(RxConfig, (RX_DMA_BURST << RxCfgDMAShift));

    rtl8168_enable_rxdvgate(dev);

    rtl8168_wait_txrx_fifo_empty(dev);

    mdelay(2);

    /* Software reset the chip. */
    RTL_W8(ChipCmd, CmdReset);

    /* Check that the chip has finished the reset. */
    for (i = 100; i > 0; i--) {
        if ((RTL_R8(ChipCmd) & CmdReset) == 0)
            break;
        udelay(10);
    }
    //printf("reset chip\n");
}

static void
rtl8168_hw_reset(struct eth_device *dev)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
    phys_addr_t ioaddr = tp->mmio_addr;

    /* Disable interrupts and clear interrupt status */
    rtl8168_irq_mask_and_ack(ioaddr);

    rtl8168_nic_reset(dev);
}

#ifdef CONFIG_WOW
static void
rtl8168_wait_ll_share_fifo_ready(struct eth_device *dev)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
    phys_addr_t ioaddr = tp->mmio_addr;
    int i;

    for (i = 0; i < 10; i++) {
        udelay(100);
        if (RTL_R16(0xD2) & BIT_9)
            break;
    }
}

static void
rtl8168_exit_oob(struct eth_device *dev)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
    phys_addr_t ioaddr = tp->mmio_addr;
    u16 data16;

    rtl8168_nic_reset(dev);

    RTL_W8(MCUCmd_reg, RTL_R8(MCUCmd_reg) & ~Now_is_oob);

    data16 = mac_ocp_read(tp, 0xE8DE) & ~BIT_14;
    mac_ocp_write(tp, 0xE8DE, data16);
    rtl8168_wait_ll_share_fifo_ready(dev);

    data16 = mac_ocp_read(tp, 0xE8DE) | BIT_15;
    mac_ocp_write(tp, 0xE8DE, data16);

    rtl8168_wait_ll_share_fifo_ready(dev);
}
#endif

static void
rtl8168_rx_desc_offset0_init(struct rtl8168_private *tp, int own)
{
    int i = 0;
    int ownbit = 0;

    if (own)
        ownbit = DescOwn;

    for (i = 0; i < NUM_RX_DESC; i++) {
        if (i == (NUM_RX_DESC - 1))
            tp->RxDescArray[i].opts1 = cpu_to_le32((ownbit | RingEnd) | (unsigned long)tp->rx_buf_sz);
        else
            tp->RxDescArray[i].opts1 = cpu_to_le32(ownbit | (unsigned long)tp->rx_buf_sz);

//#ifndef CONFIG_NONCACHE_ALLOCATION
        flush_cache((unsigned long)&tp->RxDescArray[i],sizeof(struct RxDesc));
//#endif
    }
}

static void
rtl8168_tally_counter_addr_fill(struct rtl8168_private *tp)
{
    phys_addr_t ioaddr = tp->mmio_addr;
    unsigned long tally;

    tally = ((unsigned long)dummy + ALIGN_64) & ~ALIGN_64;

    if (!tally)
        return;

    RTL_W32(CounterAddrLow, cpu_to_le32(tally));
    RTL_W32(CounterAddrHigh, 0);
}

static void
rtl8168_tally_counter_clear(struct rtl8168_private *tp)
{
    phys_addr_t ioaddr = tp->mmio_addr;
    unsigned long tally;

    tally = ((unsigned long)dummy + ALIGN_64) & ~ALIGN_64;

    if (!tally)
        return;

    tally |= BIT_0;
    RTL_W32(CounterAddrHigh, 0);
    RTL_W32(CounterAddrLow, cpu_to_le32(tally));
}

static void
rtl8168_desc_addr_fill(struct rtl8168_private *tp)
{
    phys_addr_t ioaddr = tp->mmio_addr;

    RTL_W32(TxDescStartAddrLow, (unsigned int)tp->TxDescArray);
    RTL_W32(TxDescStartAddrHigh, (unsigned long)0);
    RTL_W32(RxDescAddrLow, (unsigned int)tp->RxDescArray);
    RTL_W32(RxDescAddrHigh, (unsigned long)0);
#ifdef DEBUG_RTL8168
    printf("TxDes = %08x, %08x, RXDes = %08x, %08x\n", (unsigned int)tp->TxDescArray, rtd_inl(0xb8016020), (unsigned int)tp->RxDescArray, rtd_inl(0xb80160e4));
#endif
}

static void rtl8168_hw_start(struct rtl8168_private *tp)
{
    phys_addr_t ioaddr = tp->mmio_addr;
    struct eth_device *dev = tp->dev;
    u32 csi_tmp;

#ifdef DEBUG_RTL8168
    int stime = currticks();
    printf ("%s\n", __FUNCTION__);
#endif

    RTL_W32(RxConfig, (RX_DMA_BURST << RxCfgDMAShift));

    rtl8168_hw_reset(dev);

    rtl8168_get_mac_address(tp);

    rtl8168_rx_desc_offset0_init(tp, 1); //? init functin already initailze 

    RTL_W8(Cfg9346, Cfg9346_Unlock);

    RTL_W8(0xF1, RTL_R8(0xF1) & ~BIT_7);
    RTL_W8(Config2, RTL_R8(Config2) & ~BIT_7);
    RTL_W8(Config5, RTL_R8(Config5) & ~BIT_0);

    /* Max tx packet size */
    RTL_W8(MTPS, Reserved1_data);

    rtl8168_tally_counter_addr_fill(tp);

    rtl8168_desc_addr_fill(tp);

    /* Set DMA burst size and Interframe Gap Time */
    RTL_W32(TxConfig, (TX_DMA_BURST_128 << TxDMAShift) |
		    (InterFrameGap << TxInterFrameGapShift));

    tp->cp_cmd = (RTL_R16(CPlusCmd) | PktCntrDisable);

	/* Flow Control settings */    
    rtl8168_eri_write(ioaddr, 0xC8, 4, 0x00080002, ERIAR_ExGMAC);
    rtl8168_eri_write(ioaddr, 0xCC, 1, 0x38, ERIAR_ExGMAC);
    rtl8168_eri_write(ioaddr, 0xD0, 1, 0x48, ERIAR_ExGMAC);
    rtl8168_eri_write(ioaddr, 0xE8, 4, 0x00100006, ERIAR_ExGMAC);

    RTL_W32(TxConfig, RTL_R32(TxConfig) | BIT_7);

	/* Reset Packet filter */
	csi_tmp = rtl8168_eri_read(ioaddr, 0xDC, 1, ERIAR_ExGMAC);
    csi_tmp &= ~BIT_0;
    rtl8168_eri_write(ioaddr, 0xDC, 1, csi_tmp, ERIAR_ExGMAC);
    csi_tmp |= BIT_0;
    rtl8168_eri_write(ioaddr, 0xDC, 1, csi_tmp, ERIAR_ExGMAC);

    RTL_W8(Config3, RTL_R8(Config3) & ~Beacon_en);

    tp->cp_cmd = RTL_R16(CPlusCmd) &
	    ~(EnableBist | Macdbgo_oe | Force_halfdup |
			    Force_rxflow_en | Force_txflow_en |
			    Cxpl_dbg_sel | ASF | PktCntrDisable |
			    Macdbgo_sel);

    RTL_W8(0x1B, RTL_R8(0x1B) & ~0x07); //? for led gpio

    RTL_W8(TDFNR, 0x1);

    RTL_W8(Config2, RTL_R8(Config2) & ~PMSTS_En);

    RTL_W8(0xD0, RTL_R8(0xD0) | BIT_6);
    RTL_W8(0xF2, RTL_R8(0xF2) | BIT_6);

    RTL_W8(0xD0, RTL_R8(0xD0) | BIT_7);

	/* RSS disable */
	rtl8168_eri_write(ioaddr, 0xC0, 2, 0x0000, ERIAR_ExGMAC);
    rtl8168_eri_write(ioaddr, 0xB8, 4, 0x00000000, ERIAR_ExGMAC);

	/* Mac speed down */
    rtl8168_eri_write(ioaddr, 0x5F0, 2, 0x4f87, ERIAR_ExGMAC);

	/* Disable TX_pause_en */
    csi_tmp = rtl8168_eri_read(ioaddr, 0x1B0, 4, ERIAR_ExGMAC);
    csi_tmp &= ~BIT_12;
    rtl8168_eri_write(ioaddr, 0x1B0, 4, csi_tmp, ERIAR_ExGMAC);

	/* Disable clkeq_n_timer_sel and disable dynamic ASPM */
    csi_tmp = rtl8168_eri_read(ioaddr, 0x2FC, 1, ERIAR_ExGMAC);
    csi_tmp &= ~(BIT_0 | BIT_1 | BIT_2);
    csi_tmp |= BIT_0;
    rtl8168_eri_write(ioaddr, 0x2FC, 1, csi_tmp, ERIAR_ExGMAC);

    /* Set 10M IFG to default value */
    rtl8168_eri_write(ioaddr, 0x2F8, 2, 0x1D8F, ERIAR_ExGMAC);

    RTL_W16(CPlusCmd, tp->cp_cmd);

    RTL_W16(RxMaxSize, tp->rx_buf_sz);

    rtl8168_disable_rxdvgate(dev);

    /* Set Rx packet filter */
    rtl8168_hw_set_rx_packet_filter(dev);

    RTL_W8(Cfg9346, Cfg9346_Lock);

    wmb();

    //RTL_W8(ChipCmd, CmdTxEnb | CmdRxEnb);
    RTL_W8(ChipCmd, CmdRxEnb);

    wmb();

    udelay(10);

#ifdef DEBUG_RTL8168
    printf ("%s elapsed time : %lu\n", __FUNCTION__, currticks()-stime);
#endif
}

void rtl8168_init_ring_indexes(struct rtl8168_private *tp)
{
    tp->dirty_tx = 0;
    tp->dirty_rx = 0;
    tp->cur_tx = 0;
    tp->cur_rx = 0;
}

static inline void
rtl8168_mark_as_last_descriptor(struct RxDesc *desc)
{
    desc->opts1 |= cpu_to_le32(RingEnd);
}

static void
rtl8168_tx_desc_init(struct rtl8168_private *tp)
{
    int i = 0;

    memset(tp->TxDescArray, 0x0, NUM_TX_DESC * sizeof(struct TxDesc));

    for (i = 0; i < NUM_TX_DESC; i++) {
        if (i == (NUM_TX_DESC - 1))
            tp->TxDescArray[i].opts1 = cpu_to_le32(RingEnd);
    }
}

static void
rtl8168_rx_desc_init(struct rtl8168_private *tp)
{
    memset(tp->RxDescArray, 0x0, NUM_RX_DESC * sizeof(struct RxDesc));
}

static u32
rtl8168_rx_fill(struct rtl8168_private *tp,
                struct eth_device *dev,
                u32 start,
                u32 end)
{
    unsigned long rxb;
    u32 cur;

    rxb = (unsigned long)tp->RxBufferRings;
    rxb = (rxb + ALIGN_8) & ~ALIGN_8;

    for (cur = start; end - cur > 0; cur++) {
        if (cur == (NUM_RX_DESC - 1))
            tp->RxDescArray[cur].opts1 =
                cpu_to_le32((DescOwn | RingEnd) + tp->rx_buf_sz);
        else
            tp->RxDescArray[cur].opts1 =
                cpu_to_le32(DescOwn + tp->rx_buf_sz);

        if(cur > (NUM_RX_DESC - 1))
        {
            printf("[ERR] invalid cur(%d)\n", cur);
            continue;
        }

        tp->RxBufferRing[cur] = (u8 *)(rxb + cur * tp->rx_buf_sz);
        tp->RxDescArray[cur].addr =
            cpu_to_le64((unsigned long)(tp->RxBufferRing[cur]));

//#ifndef CONFIG_NONCACHE_ALLOCATION
        flush_cache((unsigned long)&tp->RxDescArray[cur],sizeof(struct RxDesc));
//#endif
        flush_cache((unsigned long)tp->RxBufferRing[cur], tp->rx_buf_sz);
    }

    return cur - start;
}

static int
rtl8168_init_ring(struct eth_device *dev)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
#ifdef DEBUG_RTL8168
    int stime = currticks();
    printf ("%s\n", __FUNCTION__);
#endif

    rtl8168_init_ring_indexes(tp);

    rtl8168_tx_desc_init(tp);
    rtl8168_rx_desc_init(tp);

    rtl8168_rx_fill(tp, dev, 0, NUM_RX_DESC);

    //rtl8168_mark_as_last_descriptor(tp->RxDescArray + NUM_RX_DESC - 1);

#ifdef DEBUG_RTL8168
    printf ("%s elapsed time : %lu\n", __FUNCTION__, currticks()-stime);
#endif

    return 0;
}

static void
rtl8168_hw_d3_para(struct eth_device *dev)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
    phys_addr_t ioaddr = tp->mmio_addr;

    RTL_W16(RxMaxSize, 0);

    RTL_W8(0xF1, RTL_R8(0xF1) & ~BIT_7);
    RTL_W8(Cfg9346, Cfg9346_Unlock);
    RTL_W8(Config2, RTL_R8(Config2) & ~BIT_7);
    RTL_W8(Config5, RTL_R8(Config5) & ~BIT_0);
    RTL_W8(Cfg9346, Cfg9346_Lock);

    rtl8168_eri_write(ioaddr, 0x2F8, 2, 0x0064, ERIAR_ExGMAC);

    /*disable ocp phy power saving*/
    mdio_write(tp, 0x1F, 0x0C41);
    mdio_write(tp, 0x13, 0x0000);
    mdio_write(tp, 0x13, 0x0500);
    mdio_write(tp, 0x1F, 0x0000);
    rtl8168_disable_rxdvgate(dev);
}

static void rtl8168_down(struct eth_device *dev)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;

    rtl8168_hw_reset(dev);

    rtl8168_hw_d3_para(dev);

    if (tp->RxBufferRings) {
        free(tp->RxBufferRings);
        tp->RxBufferRings = NULL;
    }

    if (tp->DescArrays) {
        free(tp->DescArrays);
        tp->DescArrays = NULL;
    }
}

#if 0
static int rtl8168_set_speed_xmii(struct rtl8168_private *tp,
                                  u8 autoneg, u16 speed, u8 duplex)
{
    u16 gbcr, anar, bmcr;

    mdio_write(tp, 0x1f, 0x0000);

    anar = mdio_read(tp, MII_ADVERTISE);
    anar &= ~(ADVERTISE_10HALF | ADVERTISE_10FULL |
              ADVERTISE_100HALF | ADVERTISE_100FULL);
    anar |= ADVERTISE_PAUSE_CAP | ADVERTISE_PAUSE_ASYM;

    gbcr = mdio_read(tp, MII_CTRL1000);
    gbcr &= ~(ADVERTISE_1000FULL | ADVERTISE_1000HALF);

    if (autoneg) {
        anar |= ADVERTISE_10HALF | ADVERTISE_10FULL |
                ADVERTISE_100HALF | ADVERTISE_100FULL;

        gbcr |= ADVERTISE_1000FULL | ADVERTISE_1000HALF;

        bmcr = BMCR_ANENABLE | BMCR_ANRESTART | BMCR_RESET;
    } else if (speed == SPEED_1000) {
        gbcr |= ADVERTISE_1000HALF;

        if (duplex == DUPLEX_FULL)
            gbcr |= ADVERTISE_1000FULL;

        bmcr = BMCR_SPEED1000;
    } else if (speed == SPEED_100) {
        anar |= ADVERTISE_100HALF;

        if (duplex == DUPLEX_FULL)
            anar |= ADVERTISE_100FULL;

        bmcr = BMCR_SPEED100;
    } else if (speed == SPEED_10) {
        anar |= ADVERTISE_10HALF;

        if (duplex == DUPLEX_FULL)
            anar |= ADVERTISE_10FULL;

        bmcr = 0;
    } else
        return -EINVAL;

    mdio_write(tp, MII_ADVERTISE, anar);
    mdio_write(tp, MII_CTRL1000, gbcr);
    mdio_write(tp, MII_BMCR, bmcr);

    return 0;
}
#endif

static void
rtl8168_hw_init(struct eth_device *dev)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
    phys_addr_t ioaddr = tp->mmio_addr;
    int count = 0;

    // ? disable aspm and clock request before access ephy
    RTL_W8(Cfg9346, Cfg9346_Unlock);
    RTL_W8(Config5, RTL_R8(Config5) & ~BIT_0);
    RTL_W8(Config2, RTL_R8(Config2) & ~BIT_7);
    RTL_W8(Cfg9346, Cfg9346_Lock);
    RTL_W8(0xF1, RTL_R8(0xF1) & ~BIT_7);  //??

    /* Wait phy status 0x3 */
    mdio_write(tp, 0x1F, 0x0a42);
    while((mdio_read(tp, 0x10) & 0x3) != 0x3 && count < 20) {
		udelay(100);
		count++;
    }

    if(count == 20) {
		printf("[ETH]Phy status error! (%04x)\n", mdio_read(tp, 0x10));
    }
    mdio_write(tp, 0x1F, 0x0000);

    /*disable ocp phy power saving*/
    mdio_write(tp, 0x1F, 0x0C41);
    mdio_write(tp, 0x13, 0x0000);
    mdio_write(tp, 0x13, 0x0500);
    mdio_write(tp, 0x1F, 0x0000);
   // printf("PHY status: %08x, %08x\n", mdio_read(tp, 0x00), RTL_R8(0x6c));
}

static void
rtl8168_phy_power_up (struct eth_device *dev)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
//    phys_addr_t ioaddr = tp->mmio_addr;
//    u32 csi_tmp;

    mdio_write(tp, 0x1F, 0x0000);
    //csi_tmp = rtl8168_eri_read(ioaddr, 0x1AB, 1, ERIAR_ExGMAC);
    //csi_tmp |=  ( BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 );
    //rtl8168_eri_write(ioaddr, 0x1AB, 1, csi_tmp, ERIAR_ExGMAC);
    mdio_write(tp, MII_BMCR, BMCR_ANENABLE);  // Enable auto negotiation
}

static void rtl8168_powerup_pll(struct eth_device *dev)
{
//    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
//    phys_addr_t ioaddr = tp->mmio_addr;

    //RTL_W8(PMCH, RTL_R8(PMCH) | BIT_7 | BIT_6);  // default value = 0xf0

    rtl8168_phy_power_up (dev);
}

/**************************************************************************
INIT - setting up the ethernet interface
***************************************************************************/
static int rtl8168_init(struct eth_device *dev, bd_t *bis)
{
    struct rtl8168_private *tp = (struct rtl8168_private *)dev->priv;
//    phys_addr_t ioaddr = tp->mmio_addr;

#ifdef DEBUG_RTL8168
    int stime = currticks();
    printf ("%s\n", __FUNCTION__);
#endif

//#ifdef CONFIG_NONCACHE_ALLOCATION
//    tp->DescArrays = (u8 *)malloc_noncache(R8168_TX_RING_BYTES +
//                                  	   R8168_RX_RING_BYTES +
//                                           ALIGN_256);
//#else
    tp->DescArrays = (u8 *)malloc(R8168_TX_RING_BYTES +
                                  R8168_RX_RING_BYTES +
                                  ALIGN_256);
//#endif
    if (!tp->DescArrays) {
        printf("Can not allocate memory of r8168\n");
        return -ENOMEM;
    }

    tp->RxBufferRings = (u8 *)memalign(64, NUM_RX_DESC * tp->rx_buf_sz + ALIGN_8);
    if (!tp->RxBufferRings) {
        printf("Can not allocate memory of r8168\n");
        free(tp->DescArrays);
        tp->DescArrays = 0;
        return -ENOMEM;
    }

    /* Tx Descriptors needs 256 bytes alignment; */
    tp->TxDescArray = (struct TxDesc *)((unsigned long)(tp->DescArrays +
                                        ALIGN_256) & ~ALIGN_256);

    /* Rx Descriptors needs 256 bytes alignment; */
    tp->RxDescArray = (struct RxDesc *)((u8 *)tp->TxDescArray +
                                        R8168_TX_RING_BYTES);

#ifdef DEBUG_RTL8168
    printf("TXDesc = %p, RXDesc = %p\n", tp->TxDescArray, tp->RxDescArray);
#endif

    rtl8168_init_ring(dev);

#ifdef CONFIG_WOW
    rtl8168_exit_oob(dev);
#endif

    rtl8168_tally_counter_clear(tp);

    rtl8168_hw_init(dev);

    rtl8168_hw_reset(dev); 

    rtl8168_powerup_pll(dev);

    rtl8168_hw_start(tp);

#ifdef DEBUG_RTL8168
    printf ("Default Set Speed To auto-nego\n");
#endif
   // rtl8168_set_speed_xmii(tp, AUTONEG_ENABLE, SPEED_1000, DUPLEX_FULL);

#ifdef DEBUG_RTL8168
    printf ("%s elapsed time : %lu\n", __FUNCTION__, currticks()-stime);
#endif
    return 0;
}

/**************************************************************************
HALT - Turn off ethernet interface
***************************************************************************/
static void rtl8168_halt(struct eth_device *dev)
{
#ifdef DEBUG_RTL8168
    printf ("%s\n", __FUNCTION__);
#endif
    rtl8168_down(dev);
}

static void rtl8168_get_mac_version(struct rtl8168_private *tp, phys_addr_t ioaddr)
{
    u32 val32;

    val32 = RTL_R32(TxConfig)  ;
#ifdef DEBUG_RTL8168
    printf("TXConfig = %08x\n", val32);
#endif
    if(val32 != 0x53900b00) {
		printf("[ETH] Mac version is incorrect.\n");
    }
    return ;
}

static void rtl8168_get_mac_address(struct rtl8168_private *tp)
{
    struct eth_device *dev = tp->dev;
    phys_addr_t ioaddr = tp->mmio_addr;
    int i;

    uint8_t env_enetaddr[6];
    char *default_enetaddr="00:11:22:33:44:55" ; 

    if (!eth_getenv_enetaddr("ethaddr", env_enetaddr))
	    eth_parse_enetaddr(default_enetaddr, env_enetaddr);	

    // add valid mac
    RTL_W8(Cfg9346, Cfg9346_Unlock);	
    RTL_W32(MAC0,  env_enetaddr[0]		
		    |env_enetaddr[1]<<8
		    |env_enetaddr[2]<<16
		    |env_enetaddr[3]<<24);
    RTL_W16(MAC4,  env_enetaddr[4]
				  |env_enetaddr[5]<<8);
    RTL_W8(Cfg9346, Cfg9346_Lock);

    for (i = 0; i < MAC_ADDR_LEN; i++)
        dev->enetaddr[i] = RTL_R8(MAC0 + i);

#ifdef DEBUG_RTL8168
    printf("MAC Address");
    for (i = 0; i < MAC_ADDR_LEN; i++)
        printf(":%02x", dev->enetaddr[i]);
    putc('\n');
#endif
}

static void write_phy(u32 RegAddr,u32 value)
{
    phys_addr_t ioaddr = REG_BASE;
    //uint32_t val = RTL_R32(PHYAR);
    int i;

    RTL_W32(PHYAR, PHYAR_Write |
            (RegAddr & PHYAR_Reg_Mask) << PHYAR_Reg_shift |
            (value & PHYAR_Data_Mask));
    //printf("WPHY addr %x value %x \n",RegAddr,value);
    for (i = 0; i < 10; i++) {
        udelay(100);

        /* Check if the RTL8168 has completed writing to the specified MII register */
        if (!(RTL_R32(PHYAR) & PHYAR_Flag)) {
            udelay(20);
            break;
        }
    }
    //printf("[ETH] mdio_write2 val=0x%x\n",val);
}

void rtl8168_swing(void)
{
    unsigned int avdd  = 0x804b;
    unsigned int dvdd  = 0x00bc;

#ifdef ETN_OTP
    unsigned char data = 0;
    if(rtk_otp_field_read(OTP_FIELD_ETN_INTERNAL_ANALOG_LDO,  &data,  4))
    {
#ifdef DEBUG_RTL8168_TEMP
        printf("[ETH] avdd otp data=0x%x \n", data);
#endif
        if(IS_ETN_OTP_INTERNAL_ANALOG_LDO_DONE(data))
        {
            avdd = (avdd &0xFF0F) | (ETN_OTP_INTERNAL_ANALOG_LDO_VAL(data) << 4);
        }
    }
    else
    {
        enable_console();
        printf("[ETH] fail to read otp! set default value avdd=0x%x \n", avdd);
    }

    if(rtk_otp_field_read(OTP_FIELD_ETN_INTERNAL_DIGITAL_LDO, &data,  4))
    {
#ifdef DEBUG_RTL8168_TEMP
        printf("[ETH] dvdd otp data=0x%x \n", data);
#endif
        if(IS_ETN_OTP_INTERNAL_DIGITAL_LDO_DONE(data))
        {
            dvdd = (dvdd & 0xFFF8) | ETN_OTP_INTERNAL_DIGITAL_LDO_VAL(data);
        }
    }
    else
    {
        enable_console();
        printf("[ETH] fail to read otp! set default value dvdd=0x%x \n", dvdd);
    }
#ifdef DEBUG_RTL8168_TEMP
    printf("\n[ETH] set value avdd=0x%x dvdd=0x%x \n",avdd, dvdd);
#endif
#endif    

    /* avdd */
    write_phy(0x1f, 0x0bd0);
    write_phy(0x13, avdd);

    /* dvdd */
    write_phy(0x1f, 0x0bd1);
    write_phy(0x12, dvdd);

    /* R */
    write_phy(0x1f, 0x0bc0);
    write_phy(0x14, 0xee08);
    write_phy(0x17, 0x00ee);
    write_phy(0x15, 0x0466);
    //100M Tr/Tf
    write_phy(0x1f, 0x0bd0);
    write_phy(0x11, 0x50cb);

}

static void r8168_mac_ocp_write(uint32_t reg, uint32_t data)
{
    uint32_t ioaddr = REG_BASE;

//    if (rtl_ocp_reg_failure(tp, reg))
//        return;

    RTL_W32(OCPDR, 0x80000000 | (reg << 15) | data);
    //printf("[ETH] 0x%x\n",0x80000000 | (reg << 15) | data);
}

static uint16_t r8168_mac_ocp_read(uint32_t reg)
{
    uint32_t ioaddr = REG_BASE;

    RTL_W32(OCPDR, reg << 15);


    return RTL_R32(OCPDR);
}

void rtl8168_phy_int_disable(void)
{
    uint32_t data;

    data = r8168_mac_ocp_read(0xfc1e);
    data &= ~(0x1802);
    r8168_mac_ocp_write(0xfc1e, data);
    mdelay(1);
}

void rtl8168_phy_int_enable(void)
{
    uint32_t data;

    data = r8168_mac_ocp_read(0xfc1e);
    data |= (0x1802);
    r8168_mac_ocp_write(0xfc1e, data);
}

int rtl8168_clock_enable(void)
{
	int count = 0;
#ifdef DEBUG_RTL8168
	int stime = currticks();
#endif

	/* GETN_RSTN_MASK bit[30]=0 */
	//rtd_clearbits(REG_STB_CTRL0, CTRL_ETN);

	//disable internal LDO
    rtd_outl(REG_STB_CTRL0,REG_STB_XTAL_EN|REG_STB_ETN_CLK_SEL|REG_STB_CTRL_ETN_LDO_EN|REG_STB_RSTN_MT);

	/* GETN RBUS & GETN clock */
	rtd_outl(REG_STB_CLKEN0, CLKEN_ETN_RBUS | CLKEN_ETN | CLKEN_WRITE_DATA); // CLKEN_GETN_RBUS bit[11]=1,CLKEN_GETN bit[8]=1
	rtd_outl(REG_STB_SRST0, RSTN_ETN | RSTN_WRITE_DATA); // RSTN_GETN_bit[8]=1

	rtd_outl(REG_PLLETN_CLKEN, REG_PLLETN_CLKEN_MAC_MASK | REG_PLLETN_CLKEN_SYS_MASK);  //CLKEN_GETN_MAC bit[2] = 0, CLKEN_GETN_SYS[1] = 0
	rtd_outl(REG_STB_CLKEN0, CLKEN_ETN_RBUS | CLKEN_ETN); // CLKEN_GETN_RBUS bit[11]=0,CLKEN_GETN bit[8]=0
	rtd_outl(REG_STB_SRST0, RSTN_ETN); //RSTN_GETN bit[8] = 0
	rtd_outl(REG_STB_SRST0, RSTN_ETN | RSTN_WRITE_DATA);  // RSTN_GETN bit[8] = 1
	rtd_outl(REG_STB_CLKEN0, CLKEN_ETN_RBUS | CLKEN_ETN | CLKEN_WRITE_DATA); // CLKEN_GETN_RBUS bit[11]=1,CLKEN_GETN bit[8]=1

	/* PLL ETN */
	rtd_outl(REG_PLLETN_RST, REG_PLLETN_RST_PHY_MASK | REG_PLLETN_CLKEN_MAC_MASK | REG_PLLETN_RST_SYS_MASK | REG_PLLETN_RST_WRITE_MASK);  //RSTN_GETN_PHY bit[3] = 1, RSTN_GETN_MAC bit[2] = 1, RSTN_GETN_SYS[1] = 1
	rtd_outl(REG_PLLETN_CLKEN, REG_PLLETN_CLKEN_MAC_MASK | REG_PLLETN_CLKEN_SYS_MASK | REG_PLLETN_CLKEN_WRITE_MASK);  //CLKEN_GETN_MAC bit[2] = 1, CLKEN_GETN_SYS[1] = 1

	/* AFE Power State */
    rtd_outl(REG_PLLETN_PHY_INFO,REG_PLLETN_PHY_INFO_AFE_MASK
                    | REG_PLLETN_PHY_INFO_POR_DVTH_CEN_DATA(1)
                    | REG_PLLETN_PHY_INFO_POR_AVTH_CEN_DATA(1));

#ifdef DEBUG_RTL8168_TEMP
    enable_console();
    printf("[ETH] temp=%d\n",rtd_inl(0xb80003A0)/1024);
#endif
	/* Check init_autoload_done = 1 */
	while(!(rtd_inl(REG_PLLETN_INIT_DONE) & REG_PLLETN_INIT_DONE_MASK) && count < 500) {    // wait plletn init done
		mdelay(1);
		count++;
	}
	if(count == 500) {
		enable_console();
		printf("[ETH] initdone failed!(%08x)\n", rtd_inl(REG_PLLETN_INIT_DONE));
		return 0;
	}


	//link patch
	rtl8168_phy_int_disable();
	rtl8168_swing();
	rtl8168_phy_int_enable();

#ifdef DEBUG_RTL8168
	printf ("%s initdone time : %lu\n", __FUNCTION__, currticks()-stime);
#endif

	return 1;
}

int rtl8168_initialize(bd_t *bis)
{
	int ret = 0;
    struct eth_device *dev;
    struct rtl8168_private *tp = NULL;
    u32 iobase;
   // int i;

    iobase = REG_BASE;

    printf ("r8168: REALTEK RTL8168 @0x%x\n", iobase);

    dev = (struct eth_device *)malloc(sizeof(struct eth_device));
    if (!dev) {
	    printf("Can not allocate memory of r8168\n");
		ret = -ENOMEM;
	    goto err;
    }
    memset(dev, 0, sizeof(struct eth_device));

    tp = (struct rtl8168_private *)malloc(sizeof(struct rtl8168_private));
    if (!tp) {
	    printf("Can not allocate memory of r8168\n");
		ret = -ENOMEM;
	    goto err;
    }
    memset(tp, 0, sizeof(struct rtl8168_private));

    tp->mmio_addr = iobase;
    tp->dev = dev;
    tp->rx_buf_sz = RX_BUF_SIZE;

    /* for clock related settings */
    if(!rtl8168_clock_enable()) {
		goto err;
	}

#ifdef CHECK_PHY_STATUS	
    /* Check phy status = 1 */
    count = 0;
    mdio_write(tp, 0x1F, 0x0a42);
    while((mdio_read(tp, 0x10) & 0x1) != 0x1 && count < 500) {
		mdelay(1);
		count++;
    }

    if(count == 500) {
		printf("[ETH]Phy status is not ready! (%04x)\n", mdio_read(tp, 0x10));
		goto err;
    }
    mdio_write(tp, 0x1F, 0x0000);
#endif

    /* Identify chip attached to board */
    rtl8168_get_mac_version(tp, tp->mmio_addr);

#if defined(CONFIG_LG_CHG) && defined(Config_Board_TV006)
    verify_mac_address();
#endif

    rtl8168_get_mac_address(tp);

    tp->chipset = 24;

    dev->priv = (void *)tp;
    dev->iobase = (int)tp->mmio_addr;

    dev->init = rtl8168_init;
    dev->halt = rtl8168_halt;
    dev->send = rtl8168_send;
    dev->recv = rtl8168_recv;

    eth_register (dev);

    // LED setting
    rtd_outl((REG_BASE+CustomLED),0x000670ca);			

	return ret;

err: 
	if(dev)
		free(dev);
	if(tp)
		free(tp);

	printf("%s fail\n", __func__);	
    return ret;
}
